@@ -2,18 +2,19 @@
 
 #include "Python.h"
 
-static char *argv0;
+void initxyzzy(void); /* Forward */
 
-main(argc, argv)
-	int argc;
-	char **argv;
+main(int argc, char **argv)
 {
-	/* Save a copy of argv0 */
-	argv0 = argv[0];
+	/* Pass argv[0] to the Python interpreter */
+	Py_SetProgramName(argv[0]);
 
 	/* Initialize the Python interpreter.  Required. */
 	Py_Initialize();
 
+	/* Add a static module */
+	initxyzzy();
+
 	/* Define sys.argv.  It is up to the application if you
 	   want this; you can also let it undefined (since the Python 
 	   code is generally not a main program it has no business
@@ -26,6 +27,8 @@ main(argc, argv)
 	/* Execute some Python statements (in module __main__) */
 	PyRun_SimpleString("import sys\n");
 	PyRun_SimpleString("print sys.builtin_module_names\n");
+	PyRun_SimpleString("print sys.modules.keys()\n");
+	PyRun_SimpleString("print sys.executable\n");
 	PyRun_SimpleString("print sys.argv\n");
 
 	/* Note that you can call any public function of the Python
@@ -39,9 +42,25 @@ main(argc, argv)
 	/*NOTREACHED*/
 }
 
-/* This function is called by the interpreter to get its own name */
-char *
-getprogramname()
+/* A static module */
+
+/* 'self' is not used */
+static PyObject *
+xyzzy_foo(PyObject *self, PyObject* args)
 {
-	return argv0;
+	if (!PyArg_ParseTuple(args, ""))
+		return NULL;
+	return PyInt_FromLong(42L);
+}
+
+static PyMethodDef xyzzy_methods[] = {
+	{"foo",		xyzzy_foo,	1},
+	{NULL,		NULL}		/* sentinel */
+};
+
+void
+initxyzzy(void)
+{
+	PyImport_AddModule("xyzzy");
+	Py_InitModule("xyzzy", xyzzy_methods);
 }
