@@ -1,5 +1,7 @@
-# Define names for all type symbols known in the standard interpreter.
-# Types that are part of optional modules (e.g. array) are not listed.
+"""Define names for all type symbols known in the standard interpreter.
+
+Types that are part of optional modules (e.g. array) are not listed.
+"""
 
 import sys
 
@@ -15,6 +17,7 @@ except NameError:
     pass
 
 StringType = type('')
+UnicodeType = type(u'')
 BufferType = type(buffer(''))
 
 TupleType = type(())
