@@ -1,33 +1,3 @@
-/***********************************************************
-Copyright 1991-1995 by Stichting Mathematisch Centrum, Amsterdam,
-The Netherlands.
-
-                        All Rights Reserved
-
-Permission to use, copy, modify, and distribute this software and its
-documentation for any purpose and without fee is hereby granted,
-provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in
-supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI or Corporation for National Research Initiatives or
-CNRI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior
-permission.
-
-While CWI is the initial source for this software, a modified version
-is made available by the Corporation for National Research Initiatives
-(CNRI) at the Internet address ftp://ftp.python.org.
-
-STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
-REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
-MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
-CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
-DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
-PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
-TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-PERFORMANCE OF THIS SOFTWARE.
-
-******************************************************************/
 
 /* imageopmodule - Various operations on pictures */
 
@@ -56,9 +26,7 @@ typedef unsigned long Py_UInt32;
 static PyObject *ImageopError;
  
 static PyObject *
-imageop_crop(self, args)
-	PyObject *self;
-PyObject *args;
+imageop_crop(PyObject *self, PyObject *args)
 {
 	char *cp, *ncp;
 	short *nsp;
@@ -112,9 +80,7 @@ PyObject *args;
 }
  
 static PyObject *
-imageop_scale(self, args)
-	PyObject *self;
-PyObject *args;
+imageop_scale(PyObject *self, PyObject *args)
 {
 	char *cp, *ncp;
 	short *nsp;
@@ -161,9 +127,7 @@ PyObject *args;
 /* Note: this routine can use a bit of optimizing */
 
 static PyObject *
-imageop_tovideo(self, args)
-	PyObject *self;
-PyObject *args;
+imageop_tovideo(PyObject *self, PyObject *args)
 {
 	int maxx, maxy, x, y, len;
 	int i;
@@ -217,9 +181,7 @@ PyObject *args;
 }
 
 static PyObject *
-imageop_grey2mono(self, args)
-	PyObject *self;
-PyObject *args;
+imageop_grey2mono(PyObject *self, PyObject *args)
 {
 	int tres, x, y, len;
 	unsigned char *cp, *ncp;
@@ -259,9 +221,7 @@ PyObject *args;
 }
 
 static PyObject *
-imageop_grey2grey4(self, args)
-	PyObject *self;
-PyObject *args;
+imageop_grey2grey4(PyObject *self, PyObject *args)
 {
 	int x, y, len;
 	unsigned char *cp, *ncp;
@@ -300,9 +260,7 @@ PyObject *args;
 }
 
 static PyObject *
-imageop_grey2grey2(self, args)
-	PyObject *self;
-PyObject *args;
+imageop_grey2grey2(PyObject *self, PyObject *args)
 {
 	int x, y, len;
 	unsigned char *cp, *ncp;
@@ -341,9 +299,7 @@ PyObject *args;
 }
 
 static PyObject *
-imageop_dither2mono(self, args)
-	PyObject *self;
-PyObject *args;
+imageop_dither2mono(PyObject *self, PyObject *args)
 {
 	int sum, x, y, len;
 	unsigned char *cp, *ncp;
@@ -387,9 +343,7 @@ PyObject *args;
 }
 
 static PyObject *
-imageop_dither2grey2(self, args)
-	PyObject *self;
-PyObject *args;
+imageop_dither2grey2(PyObject *self, PyObject *args)
 {
 	int x, y, len;
 	unsigned char *cp, *ncp;
@@ -432,9 +386,7 @@ PyObject *args;
 }
 
 static PyObject *
-imageop_mono2grey(self, args)
-	PyObject *self;
-PyObject *args;
+imageop_mono2grey(PyObject *self, PyObject *args)
 {
 	int v0, v1, x, y, len, nlen;
 	unsigned char *cp, *ncp;
@@ -471,9 +423,7 @@ PyObject *args;
 }
 
 static PyObject *
-imageop_grey22grey(self, args)
-	PyObject *self;
-PyObject *args;
+imageop_grey22grey(PyObject *self, PyObject *args)
 {
 	int x, y, len, nlen;
 	unsigned char *cp, *ncp;
@@ -509,9 +459,7 @@ PyObject *args;
 }
 
 static PyObject *
-imageop_grey42grey(self, args)
-	PyObject *self;
-PyObject *args;
+imageop_grey42grey(PyObject *self, PyObject *args)
 {
 	int x, y, len, nlen;
 	unsigned char *cp, *ncp;
@@ -546,9 +494,7 @@ PyObject *args;
 }
 
 static PyObject *
-imageop_rgb2rgb8(self, args)
-	PyObject *self;
-PyObject *args;
+imageop_rgb2rgb8(PyObject *self, PyObject *args)
 {
 	int x, y, len, nlen;
 	Py_UInt32 *cp;
@@ -590,9 +536,7 @@ PyObject *args;
 }
 
 static PyObject *
-imageop_rgb82rgb(self, args)
-	PyObject *self;
-PyObject *args;
+imageop_rgb82rgb(PyObject *self, PyObject *args)
 {
 	int x, y, len, nlen;
 	unsigned char *cp;
@@ -633,9 +577,7 @@ PyObject *args;
 }
 
 static PyObject *
-imageop_rgb2grey(self, args)
-	PyObject *self;
-PyObject *args;
+imageop_rgb2grey(PyObject *self, PyObject *args)
 {
 	int x, y, len, nlen;
 	Py_UInt32 *cp;
@@ -671,9 +613,7 @@ PyObject *args;
 }
 
 static PyObject *
-imageop_grey2rgb(self, args)
-	PyObject *self;
-PyObject *args;
+imageop_grey2rgb(PyObject *self, PyObject *args)
 {
 	int x, y, len, nlen;
 	unsigned char *cp;
@@ -705,9 +645,7 @@ PyObject *args;
 
 /*
 static object *
-imageop_mul(self, args)
-	object *self;
-object *args;
+imageop_mul(object *self, object *args)
 {
 	char *cp, *ncp;
 	int len, size, x, y;
@@ -759,7 +697,7 @@ static PyMethodDef imageop_methods[] = {
 
 
 DL_EXPORT(void)
-initimageop()
+initimageop(void)
 {
 	PyObject *m, *d;
 	m = Py_InitModule("imageop", imageop_methods);
