@@ -3,36 +3,6 @@ XXX support range parameter on search
 XXX support mstop parameter on search
 */
 
-/***********************************************************
-Copyright 1991-1995 by Stichting Mathematisch Centrum, Amsterdam,
-The Netherlands.
-
-                        All Rights Reserved
-
-Permission to use, copy, modify, and distribute this software and its
-documentation for any purpose and without fee is hereby granted,
-provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in
-supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI or Corporation for National Research Initiatives or
-CNRI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior
-permission.
-
-While CWI is the initial source for this software, a modified version
-is made available by the Corporation for National Research Initiatives
-(CNRI) at the Internet address ftp://ftp.python.org.
-
-STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
-REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
-MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
-CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
-DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
-PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
-TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-PERFORMANCE OF THIS SOFTWARE.
-
-******************************************************************/
 
 /* Regular expression objects */
 /* This uses Tatu Ylonen's copyleft-free reimplementation of
@@ -40,6 +10,8 @@ PERFORMANCE OF THIS SOFTWARE.
 
 #include "Python.h"
 
+#include <ctype.h>
+
 #include "regexpr.h"
 
 static PyObject *RegexError;	/* Exception */	
@@ -59,25 +31,25 @@ typedef struct {
 /* Regex object methods */
 
 static void
-reg_dealloc(re)
-	regexobject *re;
+reg_dealloc(regexobject *re)
 {
-	PyMem_XDEL(re->re_patbuf.buffer);
+	if (re->re_patbuf.buffer)
+		free(re->re_patbuf.buffer);
 	Py_XDECREF(re->re_translate);
 	Py_XDECREF(re->re_lastok);
 	Py_XDECREF(re->re_groupindex);
 	Py_XDECREF(re->re_givenpat);
 	Py_XDECREF(re->re_realpat);
-	PyMem_DEL(re);
+	PyObject_Del(re);
 }
 
 static PyObject *
-makeresult(regs)
-	struct re_registers *regs;
+makeresult(struct re_registers *regs)
 {
 	PyObject *v;
 	int i;
 	static PyObject *filler = NULL;
+
 	if (filler == NULL) {
 		filler = Py_BuildValue("(ii)", -1, -1);
 		if (filler == NULL)
@@ -86,6 +58,7 @@ makeresult(regs)
 	v = PyTuple_New(RE_NREGS);
 	if (v == NULL)
 		return NULL;
+
 	for (i = 0; i < RE_NREGS; i++) {
 		int lo = regs->start[i];
 		int hi = regs->end[i];
@@ -96,45 +69,41 @@ makeresult(regs)
 		}
 		else
 			w = Py_BuildValue("(ii)", lo, hi);
-		if (w == NULL) {
-			Py_XDECREF(v);
+		if (w == NULL || PyTuple_SetItem(v, i, w) < 0) {
+			Py_DECREF(v);
 			return NULL;
 		}
-		PyTuple_SetItem(v, i, w);
 	}
 	return v;
 }
 
 static PyObject *
-reg_match(re, args)
-	regexobject *re;
-	PyObject *args;
+regobj_match(regexobject *re, PyObject *args)
 {
 	PyObject *argstring;
 	char *buffer;
 	int size;
-	int offset;
+	int offset = 0;
 	int result;
-	if (PyArg_Parse(args, "S", &argstring)) {
-		offset = 0;
-	}
-	else {
-		PyErr_Clear();
-		if (!PyArg_Parse(args, "(Si)", &argstring, &offset))
-			return NULL;
-	}
-	buffer = PyString_AsString(argstring);
-	size = PyString_Size(argstring);
+
+	if (!PyArg_ParseTuple(args, "O|i:match", &argstring, &offset))
+		return NULL;
+	if (!PyArg_Parse(argstring, "t#", &buffer, &size))
+		return NULL;
+
 	if (offset < 0 || offset > size) {
 		PyErr_SetString(RegexError, "match offset out of range");
 		return NULL;
 	}
 	Py_XDECREF(re->re_lastok);
 	re->re_lastok = NULL;
-	result = re_match(&re->re_patbuf, buffer, size, offset, &re->re_regs);
+	result = _Py_re_match(&re->re_patbuf, (unsigned char *)buffer, size, offset,
+			      &re->re_regs);
 	if (result < -1) {
-		/* Failure like stack overflow */
-		PyErr_SetString(RegexError, "match failure");
+		/* Serious failure of some sort; if re_match didn't 
+		   set an exception, raise a generic error */
+	        if (!PyErr_Occurred())
+		        PyErr_SetString(RegexError, "match failure");
 		return NULL;
 	}
 	if (result >= 0) {
@@ -145,27 +114,20 @@ reg_match(re, args)
 }
 
 static PyObject *
-reg_search(re, args)
-	regexobject *re;
-	PyObject *args;
+regobj_search(regexobject *re, PyObject *args)
 {
 	PyObject *argstring;
 	char *buffer;
 	int size;
-	int offset;
+	int offset = 0;
 	int range;
 	int result;
 	
-	if (PyArg_Parse(args, "S", &argstring)) {
-		offset = 0;
-	}
-	else {
-		PyErr_Clear();
-		if (!PyArg_Parse(args, "(Si)", &argstring, &offset))
-			return NULL;
-	}
-	buffer = PyString_AsString(argstring);
-	size = PyString_Size(argstring);
+	if (!PyArg_ParseTuple(args, "O|i:search", &argstring, &offset))
+		return NULL;
+	if (!PyArg_Parse(argstring, "t#:search", &buffer, &size))
+		return NULL;
+
 	if (offset < 0 || offset > size) {
 		PyErr_SetString(RegexError, "search offset out of range");
 		return NULL;
@@ -177,11 +139,13 @@ reg_search(re, args)
 	range = size - offset;
 	Py_XDECREF(re->re_lastok);
 	re->re_lastok = NULL;
-	result = re_search(&re->re_patbuf, buffer, size, offset, range,
+	result = _Py_re_search(&re->re_patbuf, (unsigned char *)buffer, size, offset, range,
 			   &re->re_regs);
 	if (result < -1) {
-		/* Failure like stack overflow */
-		PyErr_SetString(RegexError, "match failure");
+		/* Serious failure of some sort; if re_match didn't 
+		   set an exception, raise a generic error */
+	        if (!PyErr_Occurred())
+	  	        PyErr_SetString(RegexError, "match failure");
 		return NULL;
 	}
 	if (result >= 0) {
@@ -191,52 +155,35 @@ reg_search(re, args)
 	return PyInt_FromLong((long)result); /* Position of the match or -1 */
 }
 
-static PyObject *
-reg_group(re, args)
-	regexobject *re;
-	PyObject *args;
+/* get the group from the regex where index can be a string (group name) or
+   an integer index [0 .. 99]
+ */
+static PyObject*
+group_from_index(regexobject *re, PyObject *index)
 {
 	int i, a, b;
-	if (args != NULL && PyTuple_Check(args)) {
-		int n = PyTuple_Size(args);
-		PyObject *res = PyTuple_New(n);
-		if (res == NULL)
+	char *v;
+
+	if (PyString_Check(index))
+		if (re->re_groupindex == NULL ||
+		    !(index = PyDict_GetItem(re->re_groupindex, index)))
+		{
+			PyErr_SetString(RegexError,
+					"group() group name doesn't exist");
 			return NULL;
-		for (i = 0; i < n; i++) {
-			PyObject *v = reg_group(re, PyTuple_GetItem(args, i));
-			if (v == NULL) {
-				Py_DECREF(res);
-				return NULL;
-			}
-			PyTuple_SetItem(res, i, v);
 		}
-		return res;
-	}
-	if (!PyArg_Parse(args, "i", &i)) {
-		PyObject *n;
-		PyErr_Clear();
-		if (!PyArg_Parse(args, "S", &n))
-			return NULL;
-		else {
-			PyObject *index;
-			if (re->re_groupindex == NULL)
-				index = NULL;
-			else
-				index = PyDict_GetItem(re->re_groupindex, n);
-			if (index == NULL) {
-				PyErr_SetString(RegexError, "group() group name doesn't exist");
-				return NULL;
-			}
-			i = PyInt_AsLong(index);
-		}
-	}
+
+	i = PyInt_AsLong(index);
+	if (i == -1 && PyErr_Occurred())
+		return NULL;
+
 	if (i < 0 || i >= RE_NREGS) {
 		PyErr_SetString(RegexError, "group() index out of range");
 		return NULL;
 	}
 	if (re->re_lastok == NULL) {
 		PyErr_SetString(RegexError,
-		    "group() only valid after successful match/search");
+			   "group() only valid after successful match/search");
 		return NULL;
 	}
 	a = re->re_regs.start[i];
@@ -245,20 +192,77 @@ reg_group(re, args)
 		Py_INCREF(Py_None);
 		return Py_None;
 	}
-	return PyString_FromStringAndSize(PyString_AsString(re->re_lastok)+a, b-a);
+	
+	if (!(v = PyString_AsString(re->re_lastok)))
+		return NULL;
+
+	return PyString_FromStringAndSize(v+a, b-a);
 }
 
+
+static PyObject *
+regobj_group(regexobject *re, PyObject *args)
+{
+	int n = PyTuple_Size(args);
+	int i;
+	PyObject *res = NULL;
+
+	if (n < 0)
+		return NULL;
+	if (n == 0) {
+		PyErr_SetString(PyExc_TypeError, "not enough arguments");
+		return NULL;
+	}
+	if (n == 1) {
+		/* return value is a single string */
+		PyObject *index = PyTuple_GetItem(args, 0);
+		if (!index)
+			return NULL;
+		
+		return group_from_index(re, index);
+	}
+
+	/* return value is a tuple */
+	if (!(res = PyTuple_New(n)))
+		return NULL;
+
+	for (i = 0; i < n; i++) {
+		PyObject *index = PyTuple_GetItem(args, i);
+		PyObject *group = NULL;
+
+		if (!index)
+			goto finally;
+		if (!(group = group_from_index(re, index)))
+			goto finally;
+		if (PyTuple_SetItem(res, i, group) < 0)
+			goto finally;
+	}
+	return res;
+
+  finally:
+	Py_DECREF(res);
+	return NULL;
+}
+
+
 static struct PyMethodDef reg_methods[] = {
-	{"match",	(PyCFunction)reg_match},
-	{"search",	(PyCFunction)reg_search},
-	{"group",	(PyCFunction)reg_group},
+	{"match",	(PyCFunction)regobj_match, 1},
+	{"search",	(PyCFunction)regobj_search, 1},
+	{"group",	(PyCFunction)regobj_group, 1},
 	{NULL,		NULL}		/* sentinel */
 };
 
+
+
+static char* members[] = {
+	"last", "regs", "translate",
+	"groupindex", "realpat", "givenpat",
+	NULL
+};
+
+
 static PyObject *
-reg_getattr(re, name)
-	regexobject *re;
-	char *name;
+regobj_getattr(regexobject *re, char *name)
 {
 	if (strcmp(name, "regs") == 0) {
 		if (re->re_lastok == NULL) {
@@ -308,18 +312,25 @@ reg_getattr(re, name)
 		return re->re_givenpat;
 	}
 	if (strcmp(name, "__members__") == 0) {
-		PyObject *list = PyList_New(6);
-		if (list) {
-			PyList_SetItem(list, 0, PyString_FromString("last"));
-			PyList_SetItem(list, 1, PyString_FromString("regs"));
-			PyList_SetItem(list, 2, PyString_FromString("translate"));
-			PyList_SetItem(list, 3, PyString_FromString("groupindex"));
-			PyList_SetItem(list, 4, PyString_FromString("realpat"));
-			PyList_SetItem(list, 5, PyString_FromString("givenpat"));
-			if (PyErr_Occurred()) {
+		int i = 0;
+		PyObject *list = NULL;
+
+		/* okay, so it's unlikely this list will change that often.
+		   still, it's easier to change it in just one place.
+		 */
+		while (members[i])
+			i++;
+		if (!(list = PyList_New(i)))
+			return NULL;
+
+		i = 0;
+		while (members[i]) {
+			PyObject* v = PyString_FromString(members[i]);
+			if (!v || PyList_SetItem(list, i, v) < 0) {
 				Py_DECREF(list);
-				list = NULL;
+				return NULL;
 			}
+			i++;
 		}
 		return list;
 	}
@@ -327,47 +338,55 @@ reg_getattr(re, name)
 }
 
 static PyTypeObject Regextype = {
-	PyObject_HEAD_INIT(&PyType_Type)
-	0,			/*ob_size*/
-	"regex",		/*tp_name*/
-	sizeof(regexobject),	/*tp_size*/
-	0,			/*tp_itemsize*/
+	PyObject_HEAD_INIT(NULL)
+	0,				     /*ob_size*/
+	"regex",			     /*tp_name*/
+	sizeof(regexobject),		     /*tp_size*/
+	0,				     /*tp_itemsize*/
 	/* methods */
-	(destructor)reg_dealloc, /*tp_dealloc*/
-	0,			/*tp_print*/
-	(getattrfunc)reg_getattr, /*tp_getattr*/
-	0,			/*tp_setattr*/
-	0,			/*tp_compare*/
-	0,			/*tp_repr*/
+	(destructor)reg_dealloc,	     /*tp_dealloc*/
+	0,				     /*tp_print*/
+	(getattrfunc)regobj_getattr,	     /*tp_getattr*/
+	0,				     /*tp_setattr*/
+	0,				     /*tp_compare*/
+	0,				     /*tp_repr*/
 };
 
+/* reference counting invariants:
+   pattern: borrowed
+   translate: borrowed
+   givenpat: borrowed
+   groupindex: transferred
+*/
 static PyObject *
-newregexobject(pattern, translate, givenpat, groupindex)
-	PyObject *pattern;
-	PyObject *translate;
-	PyObject *givenpat;
-	PyObject *groupindex;
+newregexobject(PyObject *pattern, PyObject *translate, PyObject *givenpat, PyObject *groupindex)
 {
 	regexobject *re;
-	char *pat = PyString_AsString(pattern);
-	int size = PyString_Size(pattern);
+	char *pat;
+	int size;
 
+	if (!PyArg_Parse(pattern, "t#", &pat, &size))
+		return NULL;
+	
 	if (translate != NULL && PyString_Size(translate) != 256) {
 		PyErr_SetString(RegexError,
-			   "translation table must be 256 bytes");
+				"translation table must be 256 bytes");
 		return NULL;
 	}
-	re = PyObject_NEW(regexobject, &Regextype);
+	re = PyObject_New(regexobject, &Regextype);
 	if (re != NULL) {
 		char *error;
 		re->re_patbuf.buffer = NULL;
 		re->re_patbuf.allocated = 0;
-		re->re_patbuf.fastmap = re->re_fastmap;
-		if (translate)
-			re->re_patbuf.translate = PyString_AsString(translate);
+		re->re_patbuf.fastmap = (unsigned char *)re->re_fastmap;
+		if (translate) {
+			re->re_patbuf.translate = (unsigned char *)PyString_AsString(translate);
+			if (!re->re_patbuf.translate)
+				goto finally;
+			Py_INCREF(translate);
+		}
 		else
 			re->re_patbuf.translate = NULL;
-		Py_XINCREF(translate);
 		re->re_translate = translate;
 		re->re_lastok = NULL;
 		re->re_groupindex = groupindex;
@@ -375,56 +394,57 @@ newregexobject(pattern, translate, givenpat, groupindex)
 		re->re_realpat = pattern;
 		Py_INCREF(givenpat);
 		re->re_givenpat = givenpat;
-		error = re_compile_pattern(pat, size, &re->re_patbuf);
+		error = _Py_re_compile_pattern((unsigned char *)pat, size, &re->re_patbuf);
 		if (error != NULL) {
 			PyErr_SetString(RegexError, error);
-			Py_DECREF(re);
-			re = NULL;
+			goto finally;
 		}
 	}
 	return (PyObject *)re;
+  finally:
+	Py_DECREF(re);
+	return NULL;
 }
 
 static PyObject *
-regex_compile(self, args)
-	PyObject *self;
-	PyObject *args;
+regex_compile(PyObject *self, PyObject *args)
 {
 	PyObject *pat = NULL;
 	PyObject *tran = NULL;
-	if (!PyArg_Parse(args, "S", &pat)) {
-		PyErr_Clear();
-		if (!PyArg_Parse(args, "(SS)", &pat, &tran))
-			return NULL;
-	}
+
+	if (!PyArg_ParseTuple(args, "S|S:compile", &pat, &tran))
+		return NULL;
 	return newregexobject(pat, tran, pat, NULL);
 }
 
 static PyObject *
-symcomp(pattern, gdict)
-	PyObject *pattern;
-	PyObject *gdict;
+symcomp(PyObject *pattern, PyObject *gdict)
 {
-	char *opat = PyString_AsString(pattern);
-	char *oend = opat + PyString_Size(pattern);
+	char *opat, *oend, *o, *n, *g, *v;
 	int group_count = 0;
+	int sz;
 	int escaped = 0;
-	char *o = opat;
-	char *n;
 	char name_buf[128];
-	char *g;
 	PyObject *npattern;
 	int require_escape = re_syntax & RE_NO_BK_PARENS ? 0 : 1;
 
+	if (!(opat = PyString_AsString(pattern)))
+		return NULL;
+
+	if ((sz = PyString_Size(pattern)) < 0)
+		return NULL;
+
+	oend = opat + sz;
+	o = opat;
+
 	if (oend == opat) {
 		Py_INCREF(pattern);
 		return pattern;
 	}
 
-	npattern = PyString_FromStringAndSize((char*)NULL, PyString_Size(pattern));
-	if (npattern == NULL)
+	if (!(npattern = PyString_FromStringAndSize((char*)NULL, sz)) ||
+	    !(n = PyString_AsString(npattern)))
 		return NULL;
-	n = PyString_AsString(npattern);
 
 	while (o < oend) {
 		if (*o == '(' && escaped == require_escape) {
@@ -441,20 +461,25 @@ symcomp(pattern, gdict)
 			g = name_buf;
 			for (++o; o < oend;) {
 				if (*o == '>') {
-					PyObject *group_name = NULL;
-					PyObject *group_index = NULL;
-					*g++ = '\0';
-					group_name = PyString_FromString(name_buf);
-					group_index = PyInt_FromLong(group_count);
-					if (group_name == NULL || group_index == NULL
-					    || PyDict_SetItem(gdict, group_name, group_index) != 0) {
-						Py_XDECREF(group_name);
-						Py_XDECREF(group_index);
-						Py_XDECREF(npattern);
-						return NULL;
-					}
-					++o; /* eat the '>' */
-					break;
+				    PyObject *group_name = NULL;
+				    PyObject *group_index = NULL;
+				    *g++ = '\0';
+				    group_name = PyString_FromString(name_buf);
+				    group_index = PyInt_FromLong(group_count);
+				    if (group_name == NULL ||
+					group_index == NULL ||
+					PyDict_SetItem(gdict, group_name,
+						       group_index) != 0)
+				    {
+					    Py_XDECREF(group_name);
+					    Py_XDECREF(group_index);
+					    Py_XDECREF(npattern);
+					    return NULL;
+				    }
+				    Py_DECREF(group_name);
+				    Py_DECREF(group_index);
+				    ++o;     /* eat the '>' */
+				    break;
 				}
 				if (!isalnum(Py_CHARMASK(*o)) && *o != '_') {
 					o = backtrack;
@@ -465,7 +490,7 @@ symcomp(pattern, gdict)
 		}
 		else if (*o == '[' && !escaped) {
 			*n++ = *o;
-			++o;	/* eat the char following '[' */
+			++o;		     /* eat the char following '[' */
 			*n++ = *o;
 			while (o < oend && *o != ']') {
 				++o;
@@ -486,7 +511,12 @@ symcomp(pattern, gdict)
 		}
 	}
 
-	if (_PyString_Resize(&npattern, n - PyString_AsString(npattern)) == 0)
+	if (!(v = PyString_AsString(npattern))) {
+		Py_DECREF(npattern);
+		return NULL;
+	}
+	/* _PyString_Resize() decrements npattern on failure */
+	if (_PyString_Resize(&npattern, n - v) == 0)
 		return npattern;
 	else {
 		return NULL;
@@ -495,27 +525,26 @@ symcomp(pattern, gdict)
 }
 
 static PyObject *
-regex_symcomp(self, args)
-	PyObject *self;
-	PyObject *args;
+regex_symcomp(PyObject *self, PyObject *args)
 {
 	PyObject *pattern;
 	PyObject *tran = NULL;
 	PyObject *gdict = NULL;
 	PyObject *npattern;
-	if (!PyArg_Parse(args, "S", &pattern)) {
-		PyErr_Clear();
-		if (!PyArg_Parse(args, "(SS)", &pattern, &tran))
-			return NULL;
-	}
+	PyObject *retval = NULL;
+
+	if (!PyArg_ParseTuple(args, "S|S:symcomp", &pattern, &tran))
+		return NULL;
+
 	gdict = PyDict_New();
-	if (gdict == NULL
-	    || (npattern = symcomp(pattern, gdict)) == NULL) {
+	if (gdict == NULL || (npattern = symcomp(pattern, gdict)) == NULL) {
 		Py_DECREF(gdict);
 		Py_DECREF(pattern);
 		return NULL;
 	}
-	return newregexobject(npattern, tran, pattern, gdict);
+	retval = newregexobject(npattern, tran, pattern, gdict);
+	Py_DECREF(npattern);
+	return retval;
 }
 
 
@@ -523,93 +552,143 @@ static PyObject *cache_pat;
 static PyObject *cache_prog;
 
 static int
-update_cache(pat)
-	PyObject *pat;
+update_cache(PyObject *pat)
 {
+	PyObject *tuple = Py_BuildValue("(O)", pat);
+	int status = 0;
+
+	if (!tuple)
+		return -1;
+
 	if (pat != cache_pat) {
 		Py_XDECREF(cache_pat);
 		cache_pat = NULL;
 		Py_XDECREF(cache_prog);
-		cache_prog = regex_compile((PyObject *)NULL, pat);
-		if (cache_prog == NULL)
-			return -1;
+		cache_prog = regex_compile((PyObject *)NULL, tuple);
+		if (cache_prog == NULL) {
+			status = -1;
+			goto finally;
+		}
 		cache_pat = pat;
 		Py_INCREF(cache_pat);
 	}
-	return 0;
+  finally:
+	Py_DECREF(tuple);
+	return status;
 }
 
 static PyObject *
-regex_match(self, args)
-	PyObject *self;
-	PyObject *args;
+regex_match(PyObject *self, PyObject *args)
 {
 	PyObject *pat, *string;
+	PyObject *tuple, *v;
+
 	if (!PyArg_Parse(args, "(SS)", &pat, &string))
 		return NULL;
 	if (update_cache(pat) < 0)
 		return NULL;
-	return reg_match((regexobject *)cache_prog, string);
+
+	if (!(tuple = Py_BuildValue("(S)", string)))
+		return NULL;
+	v = regobj_match((regexobject *)cache_prog, tuple);
+	Py_DECREF(tuple);
+	return v;
 }
 
 static PyObject *
-regex_search(self, args)
-	PyObject *self;
-	PyObject *args;
+regex_search(PyObject *self, PyObject *args)
 {
 	PyObject *pat, *string;
+	PyObject *tuple, *v;
+
 	if (!PyArg_Parse(args, "(SS)", &pat, &string))
 		return NULL;
 	if (update_cache(pat) < 0)
 		return NULL;
-	return reg_search((regexobject *)cache_prog, string);
+
+	if (!(tuple = Py_BuildValue("(S)", string)))
+		return NULL;
+	v = regobj_search((regexobject *)cache_prog, tuple);
+	Py_DECREF(tuple);
+	return v;
 }
 
 static PyObject *
-regex_set_syntax(self, args)
-	PyObject *self, *args;
+regex_set_syntax(PyObject *self, PyObject *args)
 {
 	int syntax;
 	if (!PyArg_Parse(args, "i", &syntax))
 		return NULL;
 	syntax = re_set_syntax(syntax);
+	/* wipe the global pattern cache */
+	Py_XDECREF(cache_pat);
+	cache_pat = NULL;
+	Py_XDECREF(cache_prog);
+	cache_prog = NULL;
 	return PyInt_FromLong((long)syntax);
 }
 
+static PyObject *
+regex_get_syntax(PyObject *self, PyObject *args)
+{
+	if (!PyArg_Parse(args, ""))
+		return NULL;
+	return PyInt_FromLong((long)re_syntax);
+}
+
+
 static struct PyMethodDef regex_global_methods[] = {
-	{"compile",	regex_compile, 0},
-	{"symcomp",	regex_symcomp, 0},
+	{"compile",	regex_compile, 1},
+	{"symcomp",	regex_symcomp, 1},
 	{"match",	regex_match, 0},
 	{"search",	regex_search, 0},
 	{"set_syntax",	regex_set_syntax, 0},
-	{NULL,		NULL}		/* sentinel */
+	{"get_syntax",  regex_get_syntax, 0},
+	{NULL,		NULL}		     /* sentinel */
 };
 
-void
-initregex()
+DL_EXPORT(void)
+initregex(void)
 {
 	PyObject *m, *d, *v;
+	int i;
+	char *s;
 	
+	/* Initialize object type */
+	Regextype.ob_type = &PyType_Type;
+
 	m = Py_InitModule("regex", regex_global_methods);
 	d = PyModule_GetDict(m);
+
+	if (PyErr_Warn(PyExc_DeprecationWarning,
+		       "the regex module is deprecated; "
+		       "please use the re module") < 0)
+		return;
 	
 	/* Initialize regex.error exception */
-	RegexError = PyString_FromString("regex.error");
-	if (RegexError == NULL || PyDict_SetItemString(d, "error", RegexError) != 0)
-		Py_FatalError("can't define regex.error");
-
+	v = RegexError = PyErr_NewException("regex.error", NULL, NULL);
+	if (v == NULL || PyDict_SetItemString(d, "error", v) != 0)
+		goto finally;
+	
 	/* Initialize regex.casefold constant */
-	v = PyString_FromStringAndSize((char *)NULL, 256);
-	if (v != NULL) {
-		int i;
-		char *s = PyString_AsString(v);
-		for (i = 0; i < 256; i++) {
-			if (isupper(i))
-				s[i] = tolower(i);
-			else
-				s[i] = i;
-		}
-		PyDict_SetItemString(d, "casefold", v);
-		Py_DECREF(v);
+	if (!(v = PyString_FromStringAndSize((char *)NULL, 256)))
+		goto finally;
+	
+	if (!(s = PyString_AsString(v)))
+		goto finally;
+
+	for (i = 0; i < 256; i++) {
+		if (isupper(i))
+			s[i] = tolower(i);
+		else
+			s[i] = i;
 	}
+	if (PyDict_SetItemString(d, "casefold", v) < 0)
+		goto finally;
+	Py_DECREF(v);
+
+	if (!PyErr_Occurred())
+		return;
+  finally:
+	/* Nothing */ ;
 }
