@@ -1,36 +1,5 @@
-/***********************************************************
-Copyright 1991-1995 by Stichting Mathematisch Centrum, Amsterdam,
-The Netherlands.
-
-                        All Rights Reserved
-
-Permission to use, copy, modify, and distribute this software and its
-documentation for any purpose and without fee is hereby granted,
-provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in
-supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI or Corporation for National Research Initiatives or
-CNRI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior
-permission.
-
-While CWI is the initial source for this software, a modified version
-is made available by the Corporation for National Research Initiatives
-(CNRI) at the Internet address ftp://ftp.python.org.
-
-STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
-REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
-MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
-CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
-DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
-PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
-TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-PERFORMANCE OF THIS SOFTWARE.
-
-******************************************************************/
 
 #include "Python.h"
-#include "mytime.h" /* needed for SunOS4.1 */
 #include <sys/resource.h>
 #include <sys/time.h>
 #include <unistd.h>
@@ -43,24 +12,18 @@ PERFORMANCE OF THIS SOFTWARE.
    but we can't declare the prototype, to avoid errors
    when the header files declare it different.
    Worse, on some Linuxes, getpagesize() returns a size_t... */
-#ifndef linux
-int getrusage();
-int getpagesize();
-#endif
 
 #define doubletime(TV) ((double)(TV).tv_sec + (TV).tv_usec * 0.000001)
 
 static PyObject *ResourceError;
 
 static PyObject *
-resource_getrusage(self, args)
-	PyObject *self;
-	PyObject *args;
+resource_getrusage(PyObject *self, PyObject *args)
 {
 	int who;
 	struct rusage ru;
 
-	if (!PyArg_ParseTuple(args, "i", &who))
+	if (!PyArg_ParseTuple(args, "i:getrusage", &who))
 		return NULL;
 
 	if (getrusage(who, &ru) == -1) {
@@ -93,21 +56,19 @@ resource_getrusage(self, args)
 		ru.ru_msgsnd,		     /* messages sent */
 		ru.ru_msgrcv,		     /* messages received */
 		ru.ru_nsignals,		     /* signals received */
-		ru.ru_nvcsw,		     /* voluntary context switchs */
-		ru.ru_nivcsw		     /* involuntary context switchs */
+		ru.ru_nvcsw,		     /* voluntary context switches */
+		ru.ru_nivcsw		     /* involuntary context switches */
 		);
 }
 
 
 static PyObject *
-resource_getrlimit(self, args)
-	PyObject *self;
-	PyObject *args;
+resource_getrlimit(PyObject *self, PyObject *args)
 {
 	struct rlimit rl;
 	int resource;
 
-	if (!PyArg_ParseTuple(args, "i", &resource)) 
+	if (!PyArg_ParseTuple(args, "i:getrlimit", &resource)) 
 		return NULL;
 
 	if (resource < 0 || resource >= RLIM_NLIMITS) {
@@ -132,15 +93,13 @@ resource_getrlimit(self, args)
 }
 
 static PyObject *
-resource_setrlimit(self, args)
-	PyObject *self;
-	PyObject *args;
+resource_setrlimit(PyObject *self, PyObject *args)
 {
 	struct rlimit rl;
 	int resource;
 	PyObject *curobj, *maxobj;
 
-	if (!PyArg_ParseTuple(args, "i(OO)", &resource, &curobj, &maxobj))
+	if (!PyArg_ParseTuple(args, "i(OO):setrlimit", &resource, &curobj, &maxobj))
 		return NULL;
 
 	if (resource < 0 || resource >= RLIM_NLIMITS) {
@@ -178,11 +137,9 @@ resource_setrlimit(self, args)
 }
 
 static PyObject *
-resource_getpagesize(self, args)
-	PyObject *self;
-	PyObject *args;
+resource_getpagesize(PyObject *self, PyObject *args)
 {
-	if (!PyArg_ParseTuple(args, ""))
+	if (!PyArg_ParseTuple(args, ":getpagesize"))
 		return NULL;
 	return Py_BuildValue("i", getpagesize());
 }
@@ -212,7 +169,7 @@ ins(PyObject *dict, char *name, int value)
 	/* errors will be checked by initresource() */
 }
 
-void initresource()
+void initresource(void)
 {
 	PyObject *m, *d;
 
