@@ -1,33 +1,3 @@
-/***********************************************************
-Copyright 1991-1995 by Stichting Mathematisch Centrum, Amsterdam,
-The Netherlands.
-
-                        All Rights Reserved
-
-Permission to use, copy, modify, and distribute this software and its
-documentation for any purpose and without fee is hereby granted,
-provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in
-supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI or Corporation for National Research Initiatives or
-CNRI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior
-permission.
-
-While CWI is the initial source for this software, a modified version
-is made available by the Corporation for National Research Initiatives
-(CNRI) at the Internet address ftp://ftp.python.org.
-
-STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
-REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
-MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
-CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
-DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
-PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
-TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-PERFORMANCE OF THIS SOFTWARE.
-
-******************************************************************/
 
 /* List object implementation */
 
@@ -43,8 +13,7 @@ PERFORMANCE OF THIS SOFTWARE.
 	((((n)+(PyTryBlock)-1)/(PyTryBlock))*(PyTryBlock))
 
 static int
-roundupsize(n)
-	int n;
+roundupsize(int n)
 {
 	if (n < 500)
 		return ROUNDUP(n, 10);
@@ -55,8 +24,7 @@ roundupsize(n)
 #define NRESIZE(var, type, nitems) PyMem_RESIZE(var, type, roundupsize(nitems))
 
 PyObject *
-PyList_New(size)
-	int size;
+PyList_New(int size)
 {
 	int i;
 	PyListObject *op;
@@ -70,31 +38,32 @@ PyList_New(size)
 	if (nbytes / sizeof(PyObject *) != (size_t)size) {
 		return PyErr_NoMemory();
 	}
-	op = (PyListObject *) malloc(sizeof(PyListObject));
+	/* PyObject_NewVar is inlined */
+	op = (PyListObject *) PyObject_MALLOC(sizeof(PyListObject)
+						+ PyGC_HEAD_SIZE);
 	if (op == NULL) {
 		return PyErr_NoMemory();
 	}
+	op = (PyListObject *) PyObject_FROM_GC(op);
 	if (size <= 0) {
 		op->ob_item = NULL;
 	}
 	else {
-		op->ob_item = (PyObject **) malloc(nbytes);
+		op->ob_item = (PyObject **) PyMem_MALLOC(nbytes);
 		if (op->ob_item == NULL) {
-			free((ANY *)op);
+			PyObject_FREE(PyObject_AS_GC(op));
 			return PyErr_NoMemory();
 		}
 	}
-	op->ob_type = &PyList_Type;
-	op->ob_size = size;
+	PyObject_INIT_VAR(op, &PyList_Type, size);
 	for (i = 0; i < size; i++)
 		op->ob_item[i] = NULL;
-	_Py_NewReference(op);
+	PyObject_GC_Init(op);
 	return (PyObject *) op;
 }
 
 int
-PyList_Size(op)
-	PyObject *op;
+PyList_Size(PyObject *op)
 {
 	if (!PyList_Check(op)) {
 		PyErr_BadInternalCall();
@@ -107,9 +76,7 @@ PyList_Size(op)
 static PyObject *indexerr;
 
 PyObject *
-PyList_GetItem(op, i)
-	PyObject *op;
-	int i;
+PyList_GetItem(PyObject *op, int i)
 {
 	if (!PyList_Check(op)) {
 		PyErr_BadInternalCall();
@@ -126,10 +93,8 @@ PyList_GetItem(op, i)
 }
 
 int
-PyList_SetItem(op, i, newitem)
-	register PyObject *op;
-	register int i;
-	register PyObject *newitem;
+PyList_SetItem(register PyObject *op, register int i,
+               register PyObject *newitem)
 {
 	register PyObject *olditem;
 	register PyObject **p;
@@ -152,10 +117,7 @@ PyList_SetItem(op, i, newitem)
 }
 
 static int
-ins1(self, where, v)
-	PyListObject *self;
-	int where;
-	PyObject *v;
+ins1(PyListObject *self, int where, PyObject *v)
 {
 	int i;
 	PyObject **items;
@@ -163,6 +125,11 @@ ins1(self, where, v)
 		PyErr_BadInternalCall();
 		return -1;
 	}
+	if (self->ob_size == INT_MAX) {
+		PyErr_SetString(PyExc_OverflowError,
+			"cannot add more objects to list");
+		return -1;
+	}
 	items = self->ob_item;
 	NRESIZE(items, PyObject *, self->ob_size+1);
 	if (items == NULL) {
@@ -183,10 +150,7 @@ ins1(self, where, v)
 }
 
 int
-PyList_Insert(op, where, newitem)
-	PyObject *op;
-	int where;
-	PyObject *newitem;
+PyList_Insert(PyObject *op, int where, PyObject *newitem)
 {
 	if (!PyList_Check(op)) {
 		PyErr_BadInternalCall();
@@ -196,9 +160,7 @@ PyList_Insert(op, where, newitem)
 }
 
 int
-PyList_Append(op, newitem)
-	PyObject *op;
-	PyObject *newitem;
+PyList_Append(PyObject *op, PyObject *newitem)
 {
 	if (!PyList_Check(op)) {
 		PyErr_BadInternalCall();
@@ -211,24 +173,29 @@ PyList_Append(op, newitem)
 /* Methods */
 
 static void
-list_dealloc(op)
-	PyListObject *op;
+list_dealloc(PyListObject *op)
 {
 	int i;
+	PyObject_GC_Fini(op);
+	Py_TRASHCAN_SAFE_BEGIN(op)
 	if (op->ob_item != NULL) {
-		for (i = 0; i < op->ob_size; i++) {
+		/* Do it backwards, for Christian Tismer.
+		   There's a simple test case where somehow this reduces
+		   thrashing when a *very* large list is created and
+		   immediately deleted. */
+		i = op->ob_size;
+		while (--i >= 0) {
 			Py_XDECREF(op->ob_item[i]);
 		}
-		free((ANY *)op->ob_item);
+		PyMem_FREE(op->ob_item);
 	}
-	free((ANY *)op);
+	op = (PyListObject *) PyObject_AS_GC(op);
+	PyObject_DEL(op);
+	Py_TRASHCAN_SAFE_END(op)
 }
 
 static int
-list_print(op, fp, flags)
-	PyListObject *op;
-	FILE *fp;
-	int flags;
+list_print(PyListObject *op, FILE *fp, int flags)
 {
 	int i;
 
@@ -254,8 +221,7 @@ list_print(op, fp, flags)
 }
 
 static PyObject *
-list_repr(v)
-	PyListObject *v;
+list_repr(PyListObject *v)
 {
 	PyObject *s, *comma;
 	int i;
@@ -280,29 +246,32 @@ list_repr(v)
 }
 
 static int
-list_compare(v, w)
-	PyListObject *v, *w;
-{
-	int i;
-	for (i = 0; i < v->ob_size && i < w->ob_size; i++) {
-		int cmp = PyObject_Compare(v->ob_item[i], w->ob_item[i]);
-		if (cmp != 0)
-			return cmp;
-	}
-	return v->ob_size - w->ob_size;
-}
-
-static int
-list_length(a)
-	PyListObject *a;
+list_length(PyListObject *a)
 {
 	return a->ob_size;
 }
 
-static PyObject *
-list_item(a, i)
-	PyListObject *a;
+
+
+static int
+list_contains(PyListObject *a, PyObject *el)
+{
 	int i;
+
+	for (i = 0; i < a->ob_size; ++i) {
+		int cmp = PyObject_RichCompareBool(el, PyList_GET_ITEM(a, i),
+						   Py_EQ);
+		if (cmp > 0)
+			return 1;
+		else if (cmp < 0)
+			return -1;
+	}
+	return 0;
+}
+
+
+static PyObject *
+list_item(PyListObject *a, int i)
 {
 	if (i < 0 || i >= a->ob_size) {
 		if (indexerr == NULL)
@@ -316,9 +285,7 @@ list_item(a, i)
 }
 
 static PyObject *
-list_slice(a, ilow, ihigh)
-	PyListObject *a;
-	int ilow, ihigh;
+list_slice(PyListObject *a, int ilow, int ihigh)
 {
 	PyListObject *np;
 	int i;
@@ -342,9 +309,7 @@ list_slice(a, ilow, ihigh)
 }
 
 PyObject *
-PyList_GetSlice(a, ilow, ihigh)
-	PyObject *a;
-	int ilow, ihigh;
+PyList_GetSlice(PyObject *a, int ilow, int ihigh)
 {
 	if (!PyList_Check(a)) {
 		PyErr_BadInternalCall();
@@ -354,15 +319,15 @@ PyList_GetSlice(a, ilow, ihigh)
 }
 
 static PyObject *
-list_concat(a, bb)
-	PyListObject *a;
-	PyObject *bb;
+list_concat(PyListObject *a, PyObject *bb)
 {
 	int size;
 	int i;
 	PyListObject *np;
 	if (!PyList_Check(bb)) {
-		PyErr_BadArgument();
+		PyErr_Format(PyExc_TypeError,
+			  "can only concatenate list (not \"%.200s\") to list",
+			  bb->ob_type->tp_name);
 		return NULL;
 	}
 #define b ((PyListObject *)bb)
@@ -386,9 +351,7 @@ list_concat(a, bb)
 }
 
 static PyObject *
-list_repeat(a, n)
-	PyListObject *a;
-	int n;
+list_repeat(PyListObject *a, int n)
 {
 	int i, j;
 	int size;
@@ -412,10 +375,7 @@ list_repeat(a, n)
 }
 
 static int
-list_ass_slice(a, ilow, ihigh, v)
-	PyListObject *a;
-	int ilow, ihigh;
-	PyObject *v;
+list_ass_slice(PyListObject *a, int ilow, int ihigh, PyObject *v)
 {
 	/* Because [X]DECREF can recursively invoke list operations on
 	   this list, we must postpone all [X]DECREF activity until
@@ -443,7 +403,9 @@ list_ass_slice(a, ilow, ihigh, v)
 		}
 	}
 	else {
-		PyErr_BadArgument();
+		PyErr_Format(PyExc_TypeError,
+			     "must assign list (not \"%.200s\") to slice",
+			     v->ob_type->tp_name);
 		return -1;
 	}
 	if (ilow < 0)
@@ -474,7 +436,8 @@ list_ass_slice(a, ilow, ihigh, v)
 	else { /* Insert d items; recycle ihigh-ilow items */
 		NRESIZE(item, PyObject *, a->ob_size + d);
 		if (item == NULL) {
-			PyMem_XDEL(recycle);
+			if (recycle != NULL)
+				PyMem_DEL(recycle);
 			PyErr_NoMemory();
 			return -1;
 		}
@@ -500,10 +463,7 @@ list_ass_slice(a, ilow, ihigh, v)
 }
 
 int
-PyList_SetSlice(a, ilow, ihigh, v)
-	PyObject *a;
-	int ilow, ihigh;
-	PyObject *v;
+PyList_SetSlice(PyObject *a, int ilow, int ihigh, PyObject *v)
 {
 	if (!PyList_Check(a)) {
 		PyErr_BadInternalCall();
@@ -512,11 +472,52 @@ PyList_SetSlice(a, ilow, ihigh, v)
 	return list_ass_slice((PyListObject *)a, ilow, ihigh, v);
 }
 
+static PyObject *
+list_inplace_repeat(PyListObject *self, int n)
+{
+	PyObject **items;
+	int size, i, j;
+
+
+	size = PyList_GET_SIZE(self);
+	if (size == 0) {
+		Py_INCREF(self);
+		return (PyObject *)self;
+	}
+
+	items = self->ob_item;
+
+	if (n < 1) {
+		self->ob_item = NULL;
+		self->ob_size = 0;
+		for (i = 0; i < size; i++)
+			Py_XDECREF(items[i]);
+		PyMem_DEL(items);
+		Py_INCREF(self);
+		return (PyObject *)self;
+	}
+
+	NRESIZE(items, PyObject*, size*n);
+	if (items == NULL) {
+		PyErr_NoMemory();
+		goto finally;
+	}
+	self->ob_item = items;
+	for (i = 1; i < n; i++) { /* Start counting at 1, not 0 */
+		for (j = 0; j < size; j++) {
+			PyObject *o = PyList_GET_ITEM(self, j);
+			Py_INCREF(o);
+			PyList_SET_ITEM(self, self->ob_size++, o);
+		}
+	}
+	Py_INCREF(self);
+	return (PyObject *)self;
+  finally:
+  	return NULL;
+}
+
 static int
-list_ass_item(a, i, v)
-	PyListObject *a;
-	int i;
-	PyObject *v;
+list_ass_item(PyListObject *a, int i, PyObject *v)
 {
 	PyObject *old_value;
 	if (i < 0 || i >= a->ob_size) {
@@ -534,10 +535,7 @@ list_ass_item(a, i, v)
 }
 
 static PyObject *
-ins(self, where, v)
-	PyListObject *self;
-	int where;
-	PyObject *v;
+ins(PyListObject *self, int where, PyObject *v)
 {
 	if (ins1(self, where, v) != 0)
 		return NULL;
@@ -546,52 +544,52 @@ ins(self, where, v)
 }
 
 static PyObject *
-listinsert(self, args)
-	PyListObject *self;
-	PyObject *args;
+listinsert(PyListObject *self, PyObject *args)
 {
 	int i;
 	PyObject *v;
-	if (!PyArg_Parse(args, "(iO)", &i, &v))
+	if (!PyArg_ParseTuple(args, "iO:insert", &i, &v))
 		return NULL;
 	return ins(self, i, v);
 }
 
+/* Define NO_STRICT_LIST_APPEND to enable multi-argument append() */
+
+#ifndef NO_STRICT_LIST_APPEND
+#define PyArg_ParseTuple_Compat1 PyArg_ParseTuple
+#else
+#define PyArg_ParseTuple_Compat1(args, format, ret) \
+( \
+	PyTuple_GET_SIZE(args) > 1 ? (*ret = args, 1) : \
+	PyTuple_GET_SIZE(args) == 1 ? (*ret = PyTuple_GET_ITEM(args, 0), 1) : \
+	PyArg_ParseTuple(args, format, ret) \
+)
+#endif
+
+
 static PyObject *
-listappend(self, args)
-	PyListObject *self;
-	PyObject *args;
+listappend(PyListObject *self, PyObject *args)
 {
 	PyObject *v;
-	if (!PyArg_Parse(args, "O", &v))
+	if (!PyArg_ParseTuple_Compat1(args, "O:append", &v))
 		return NULL;
 	return ins(self, (int) self->ob_size, v);
 }
 
-static PyObject *
-listextend(self, args)
-	PyListObject *self;
-	PyObject *args;
+static int
+listextend_internal(PyListObject *self, PyObject *b)
 {
-	PyObject *b = NULL, *res = NULL;
 	PyObject **items;
 	int selflen = PyList_GET_SIZE(self);
 	int blen;
 	register int i;
 
-	if (!PyArg_ParseTuple(args, "O", &b))
-		return NULL;
-
-	if (!PyList_Check(b)) {
-		PyErr_SetString(PyExc_TypeError,
-				"list.extend() argument must be a list");
-		return NULL;
-	}
-	if (PyList_GET_SIZE(b) == 0) {
+	if (PyObject_Size(b) == 0) {
 		/* short circuit when b is empty */
-		Py_INCREF(Py_None);
-		return Py_None;
+		Py_DECREF(b);
+		return 0;
 	}
+
 	if (self == (PyListObject*)b) {
 		/* as in list_ass_slice() we must special case the
 		 * situation: a.extend(a)
@@ -599,54 +597,81 @@ listextend(self, args)
 		 * XXX: I think this way ought to be faster than using
 		 * list_slice() the way list_ass_slice() does.
 		 */
+		Py_DECREF(b);
 		b = PyList_New(selflen);
 		if (!b)
-			return NULL;
+			return -1;
 		for (i = 0; i < selflen; i++) {
 			PyObject *o = PyList_GET_ITEM(self, i);
 			Py_INCREF(o);
 			PyList_SET_ITEM(b, i, o);
 		}
 	}
-	else
-		/* we want b to have the same refcount semantics for the
-		 * Py_XDECREF() in the finally clause regardless of which
-		 * branch in the above conditional we took.
-		 */
-		Py_INCREF(b);
 
-	blen = PyList_GET_SIZE(b);
+	blen = PyObject_Size(b);
+
 	/* resize a using idiom */
 	items = self->ob_item;
 	NRESIZE(items, PyObject*, selflen + blen);
-	if (items == NULL ) {
+	if (items == NULL) {
 		PyErr_NoMemory();
-		goto finally;
+		Py_DECREF(b);
+		return -1;
 	}
+
 	self->ob_item = items;
 
-	/* populate the end self with b's items */
+	/* populate the end of self with b's items */
 	for (i = 0; i < blen; i++) {
-		PyObject *o = PyList_GET_ITEM(b, i);
+		PyObject *o = PySequence_Fast_GET_ITEM(b, i);
 		Py_INCREF(o);
 		PyList_SET_ITEM(self, self->ob_size++, o);
 	}
-	res = Py_None;
-	Py_INCREF(res);
-  finally:
-	Py_XDECREF(b);
-	return res;
+	Py_DECREF(b);
+	return 0;
 }
 
 
 static PyObject *
-listpop(self, args)
-	PyListObject *self;
-	PyObject *args;
+list_inplace_concat(PyListObject *self, PyObject *other)
+{
+	other = PySequence_Fast(other, "argument to += must be a sequence");
+	if (!other)
+		return NULL;
+
+	if (listextend_internal(self, other) < 0)
+		return NULL;
+
+	Py_INCREF(self);
+	return (PyObject *)self;
+}
+
+static PyObject *
+listextend(PyListObject *self, PyObject *args)
+{
+
+	PyObject *b;
+	
+	if (!PyArg_ParseTuple(args, "O:extend", &b))
+		return NULL;
+
+	b = PySequence_Fast(b, "list.extend() argument must be a sequence");
+	if (!b)
+		return NULL;
+
+	if (listextend_internal(self, b) < 0)
+		return NULL;
+
+	Py_INCREF(Py_None);
+	return Py_None;
+}
+
+static PyObject *
+listpop(PyListObject *self, PyObject *args)
 {
 	int i = -1;
 	PyObject *v;
-	if (!PyArg_ParseTuple(args, "|i", &i))
+	if (!PyArg_ParseTuple(args, "|i:pop", &i))
 		return NULL;
 	if (self->ob_size == 0) {
 		/* Special-case most common failure cause */
@@ -682,19 +707,22 @@ listpop(self, args)
    supplied function is NULL. */
 
 static int
-docompare(x, y, compare)
-	PyObject *x;
-	PyObject *y;
-	PyObject *compare;
+docompare(PyObject *x, PyObject *y, PyObject *compare)
 {
 	PyObject *args, *res;
 	int i;
 
 	if (compare == NULL) {
-		i = PyObject_Compare(x, y);
-		if (i && PyErr_Occurred())
-			i = CMPERROR;
-		return i;
+		/* NOTE: we rely on the fact here that the sorting algorithm
+		   only ever checks whether k<0, i.e., whether x<y.  So we
+		   invoke the rich comparison function with Py_LT ('<'), and
+		   return -1 when it returns true and 0 when it returns
+		   false. */
+		i = PyObject_RichCompareBool(x, y, Py_LT);
+		if (i < 0)
+			return CMPERROR;
+		else
+			return -i;
 	}
 
 	args = Py_BuildValue("(OO)", x, y);
@@ -707,7 +735,7 @@ docompare(x, y, compare)
 	if (!PyInt_Check(res)) {
 		Py_DECREF(res);
 		PyErr_SetString(PyExc_TypeError,
-				"comparison function should return int");
+				"comparison function must return int");
 		return CMPERROR;
 	}
 	i = PyInt_AsLong(res);
@@ -771,11 +799,8 @@ docompare(x, y, compare)
 */
 
 static int
-binarysort(lo, hi, start, compare)
-	PyObject **lo;
-	PyObject **hi;
-	PyObject **start;
-	PyObject *compare;/* Comparison function object, or NULL for default */
+binarysort(PyObject **lo, PyObject **hi, PyObject **start, PyObject *compare)
+     /* compare -- comparison function object, or NULL for default */
 {
 	/* assert lo <= start <= hi
 	   assert [lo, start) is sorted */
@@ -905,10 +930,8 @@ static long cutoff[] = {
 };
 
 static int
-samplesortslice(lo, hi, compare)
-	PyObject **lo;
-	PyObject **hi;
-	PyObject *compare;/* Comparison function object, or NULL for default */
+samplesortslice(PyObject **lo, PyObject **hi, PyObject *compare)
+     /* compare -- comparison function object, or NULL for default */
 {
 	register PyObject **l, **r;
 	register PyObject *tmp, *pivot;
@@ -1211,12 +1234,15 @@ samplesortslice(lo, hi, compare)
 staticforward PyTypeObject immutable_list_type;
 
 static PyObject *
-listsort(self, compare)
-	PyListObject *self;
-	PyObject *compare;
+listsort(PyListObject *self, PyObject *args)
 {
 	int err;
+	PyObject *compare = NULL;
 
+	if (args != NULL) {
+		if (!PyArg_ParseTuple(args, "|O:sort", &compare))
+			return NULL;
+	}
 	self->ob_type = &immutable_list_type;
 	err = samplesortslice(self->ob_item,
 			      self->ob_item + self->ob_size,
@@ -1229,8 +1255,7 @@ listsort(self, compare)
 }
 
 int
-PyList_Sort(v)
-	PyObject *v;
+PyList_Sort(PyObject *v)
 {
 	if (v == NULL || !PyList_Check(v)) {
 		PyErr_BadInternalCall();
@@ -1243,50 +1268,47 @@ PyList_Sort(v)
 	return 0;
 }
 
-static PyObject *
-listreverse(self, args)
-	PyListObject *self;
-	PyObject *args;
+static void
+_listreverse(PyListObject *self)
 {
 	register PyObject **p, **q;
 	register PyObject *tmp;
 	
-	if (args != NULL) {
-		PyErr_BadArgument();
-		return NULL;
-	}
-
 	if (self->ob_size > 1) {
 		for (p = self->ob_item, q = self->ob_item + self->ob_size - 1;
-						p < q; p++, q--) {
+		     p < q;
+		     p++, q--)
+		{
 			tmp = *p;
 			*p = *q;
 			*q = tmp;
 		}
 	}
-	
+}
+
+static PyObject *
+listreverse(PyListObject *self, PyObject *args)
+{
+	if (!PyArg_ParseTuple(args, ":reverse"))
+		return NULL;
+	_listreverse(self);
 	Py_INCREF(Py_None);
 	return Py_None;
 }
 
 int
-PyList_Reverse(v)
-	PyObject *v;
+PyList_Reverse(PyObject *v)
 {
 	if (v == NULL || !PyList_Check(v)) {
 		PyErr_BadInternalCall();
 		return -1;
 	}
-	v = listreverse((PyListObject *)v, (PyObject *)NULL);
-	if (v == NULL)
-		return -1;
-	Py_DECREF(v);
+	_listreverse((PyListObject *)v);
 	return 0;
 }
 
 PyObject *
-PyList_AsTuple(v)
-	PyObject *v;
+PyList_AsTuple(PyObject *v)
 {
 	PyObject *w;
 	PyObject **p;
@@ -1300,8 +1322,8 @@ PyList_AsTuple(v)
 	if (w == NULL)
 		return NULL;
 	p = ((PyTupleObject *)w)->ob_item;
-	memcpy((ANY *)p,
-	       (ANY *)((PyListObject *)v)->ob_item,
+	memcpy((void *)p,
+	       (void *)((PyListObject *)v)->ob_item,
 	       n*sizeof(PyObject *));
 	while (--n >= 0) {
 		Py_INCREF(*p);
@@ -1311,20 +1333,18 @@ PyList_AsTuple(v)
 }
 
 static PyObject *
-listindex(self, args)
-	PyListObject *self;
-	PyObject *args;
+listindex(PyListObject *self, PyObject *args)
 {
 	int i;
-	
-	if (args == NULL) {
-		PyErr_BadArgument();
+	PyObject *v;
+
+	if (!PyArg_ParseTuple_Compat1(args, "O:index", &v))
 		return NULL;
-	}
 	for (i = 0; i < self->ob_size; i++) {
-		if (PyObject_Compare(self->ob_item[i], args) == 0)
+		int cmp = PyObject_RichCompareBool(self->ob_item[i], v, Py_EQ);
+		if (cmp > 0)
 			return PyInt_FromLong((long)i);
-		if (PyErr_Occurred())
+		else if (cmp < 0)
 			return NULL;
 	}
 	PyErr_SetString(PyExc_ValueError, "list.index(x): x not in list");
@@ -1332,52 +1352,144 @@ listindex(self, args)
 }
 
 static PyObject *
-listcount(self, args)
-	PyListObject *self;
-	PyObject *args;
+listcount(PyListObject *self, PyObject *args)
 {
 	int count = 0;
 	int i;
-	
-	if (args == NULL) {
-		PyErr_BadArgument();
+	PyObject *v;
+
+	if (!PyArg_ParseTuple_Compat1(args, "O:count", &v))
 		return NULL;
-	}
 	for (i = 0; i < self->ob_size; i++) {
-		if (PyObject_Compare(self->ob_item[i], args) == 0)
+		int cmp = PyObject_RichCompareBool(self->ob_item[i], v, Py_EQ);
+		if (cmp > 0)
 			count++;
-		if (PyErr_Occurred())
+		else if (cmp < 0)
 			return NULL;
 	}
 	return PyInt_FromLong((long)count);
 }
 
 static PyObject *
-listremove(self, args)
-	PyListObject *self;
-	PyObject *args;
+listremove(PyListObject *self, PyObject *args)
 {
 	int i;
-	
-	if (args == NULL) {
-		PyErr_BadArgument();
+	PyObject *v;
+
+	if (!PyArg_ParseTuple_Compat1(args, "O:remove", &v))
 		return NULL;
-	}
 	for (i = 0; i < self->ob_size; i++) {
-		if (PyObject_Compare(self->ob_item[i], args) == 0) {
+		int cmp = PyObject_RichCompareBool(self->ob_item[i], v, Py_EQ);
+		if (cmp > 0) {
 			if (list_ass_slice(self, i, i+1,
 					   (PyObject *)NULL) != 0)
 				return NULL;
 			Py_INCREF(Py_None);
 			return Py_None;
 		}
-		if (PyErr_Occurred())
+		else if (cmp < 0)
 			return NULL;
 	}
 	PyErr_SetString(PyExc_ValueError, "list.remove(x): x not in list");
 	return NULL;
 }
 
+static int
+list_traverse(PyListObject *o, visitproc visit, void *arg)
+{
+	int i, err;
+	PyObject *x;
+
+	for (i = o->ob_size; --i >= 0; ) {
+		x = o->ob_item[i];
+		if (x != NULL) {
+			err = visit(x, arg);
+			if (err)
+				return err;
+		}
+	}
+	return 0;
+}
+
+static int
+list_clear(PyListObject *lp)
+{
+	(void) PyList_SetSlice((PyObject *)lp, 0, lp->ob_size, 0);
+	return 0;
+}
+
+static PyObject *
+list_richcompare(PyObject *v, PyObject *w, int op)
+{
+	PyListObject *vl, *wl;
+	int i;
+
+	if (!PyList_Check(v) || !PyList_Check(w)) {
+		Py_INCREF(Py_NotImplemented);
+		return Py_NotImplemented;
+	}
+
+	vl = (PyListObject *)v;
+	wl = (PyListObject *)w;
+
+	if (vl->ob_size != wl->ob_size && (op == Py_EQ || op == Py_NE)) {
+		/* Shortcut: if the lengths differ, the lists differ */
+		PyObject *res;
+		if (op == Py_EQ)
+			res = Py_False;
+		else
+			res = Py_True;
+		Py_INCREF(res);
+		return res;
+	}
+
+	/* Search for the first index where items are different */
+	for (i = 0; i < vl->ob_size && i < wl->ob_size; i++) {
+		int k = PyObject_RichCompareBool(vl->ob_item[i],
+						 wl->ob_item[i], Py_EQ);
+		if (k < 0)
+			return NULL;
+		if (!k)
+			break;
+	}
+
+	if (i >= vl->ob_size || i >= wl->ob_size) {
+		/* No more items to compare -- compare sizes */
+		int vs = vl->ob_size;
+		int ws = wl->ob_size;
+		int cmp;
+		PyObject *res;
+		switch (op) {
+		case Py_LT: cmp = vs <  ws; break;
+		case Py_LE: cmp = vs <= ws; break;
+		case Py_EQ: cmp = vs == ws; break;
+		case Py_NE: cmp = vs != ws; break;
+		case Py_GT: cmp = vs >  ws; break;
+		case Py_GE: cmp = vs >= ws; break;
+		default: return NULL; /* cannot happen */
+		}
+		if (cmp)
+			res = Py_True;
+		else
+			res = Py_False;
+		Py_INCREF(res);
+		return res;
+	}
+
+	/* We have an item that differs -- shortcuts for EQ/NE */
+	if (op == Py_EQ) {
+		Py_INCREF(Py_False);
+		return Py_False;
+	}
+	if (op == Py_NE) {
+		Py_INCREF(Py_True);
+		return Py_True;
+	}
+
+	/* Compare the final item again using the proper operator */
+	return PyObject_RichCompare(vl->ob_item[i], wl->ob_item[i], op);
+}
+
 static char append_doc[] =
 "L.append(object) -- append object to end";
 static char extend_doc[] =
@@ -1398,51 +1510,63 @@ static char sort_doc[] =
 "L.sort([cmpfunc]) -- sort *IN PLACE*; if given, cmpfunc(x, y) -> -1, 0, 1";
 
 static PyMethodDef list_methods[] = {
-	{"append",	(PyCFunction)listappend, 0, append_doc},
-	{"insert",	(PyCFunction)listinsert, 0, insert_doc},
-	{"extend",      (PyCFunction)listextend, 1, extend_doc},
-	{"pop",		(PyCFunction)listpop, 1, pop_doc},
-	{"remove",	(PyCFunction)listremove, 0, remove_doc},
-	{"index",	(PyCFunction)listindex, 0, index_doc},
-	{"count",	(PyCFunction)listcount, 0, count_doc},
-	{"reverse",	(PyCFunction)listreverse, 0, reverse_doc},
-	{"sort",	(PyCFunction)listsort, 0, sort_doc},
+	{"append",	(PyCFunction)listappend,  METH_VARARGS, append_doc},
+	{"insert",	(PyCFunction)listinsert,  METH_VARARGS, insert_doc},
+	{"extend",      (PyCFunction)listextend,  METH_VARARGS, extend_doc},
+	{"pop",		(PyCFunction)listpop, 	  METH_VARARGS, pop_doc},
+	{"remove",	(PyCFunction)listremove,  METH_VARARGS, remove_doc},
+	{"index",	(PyCFunction)listindex,   METH_VARARGS, index_doc},
+	{"count",	(PyCFunction)listcount,   METH_VARARGS, count_doc},
+	{"reverse",	(PyCFunction)listreverse, METH_VARARGS, reverse_doc},
+	{"sort",	(PyCFunction)listsort, 	  METH_VARARGS, sort_doc},
 	{NULL,		NULL}		/* sentinel */
 };
 
 static PyObject *
-list_getattr(f, name)
-	PyListObject *f;
-	char *name;
+list_getattr(PyListObject *f, char *name)
 {
 	return Py_FindMethod(list_methods, (PyObject *)f, name);
 }
 
 static PySequenceMethods list_as_sequence = {
-	(inquiry)list_length, /*sq_length*/
-	(binaryfunc)list_concat, /*sq_concat*/
-	(intargfunc)list_repeat, /*sq_repeat*/
-	(intargfunc)list_item, /*sq_item*/
-	(intintargfunc)list_slice, /*sq_slice*/
-	(intobjargproc)list_ass_item, /*sq_ass_item*/
-	(intintobjargproc)list_ass_slice, /*sq_ass_slice*/
+	(inquiry)list_length,			/* sq_length */
+	(binaryfunc)list_concat,		/* sq_concat */
+	(intargfunc)list_repeat,		/* sq_repeat */
+	(intargfunc)list_item,			/* sq_item */
+	(intintargfunc)list_slice,		/* sq_slice */
+	(intobjargproc)list_ass_item,		/* sq_ass_item */
+	(intintobjargproc)list_ass_slice,	/* sq_ass_slice */
+	(objobjproc)list_contains,		/* sq_contains */
+	(binaryfunc)list_inplace_concat,	/* sq_inplace_concat */
+	(intargfunc)list_inplace_repeat,	/* sq_inplace_repeat */
 };
 
 PyTypeObject PyList_Type = {
 	PyObject_HEAD_INIT(&PyType_Type)
 	0,
 	"list",
-	sizeof(PyListObject),
+	sizeof(PyListObject) + PyGC_HEAD_SIZE,
 	0,
-	(destructor)list_dealloc, /*tp_dealloc*/
-	(printfunc)list_print, /*tp_print*/
-	(getattrfunc)list_getattr, /*tp_getattr*/
-	0,		/*tp_setattr*/
-	(cmpfunc)list_compare, /*tp_compare*/
-	(reprfunc)list_repr, /*tp_repr*/
-	0,		/*tp_as_number*/
-	&list_as_sequence,	/*tp_as_sequence*/
-	0,		/*tp_as_mapping*/
+	(destructor)list_dealloc,		/* tp_dealloc */
+	(printfunc)list_print,			/* tp_print */
+	(getattrfunc)list_getattr,		/* tp_getattr */
+	0,					/* tp_setattr */
+	0,					/* tp_compare */
+	(reprfunc)list_repr,			/* tp_repr */
+	0,					/* tp_as_number */
+	&list_as_sequence,			/* tp_as_sequence */
+	0,					/* tp_as_mapping */
+	0,					/* tp_hash */
+	0,					/* tp_call */
+	0,					/* tp_str */
+	0,					/* tp_getattro */
+	0,					/* tp_setattro */
+	0,					/* tp_as_buffer */
+	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_GC,	/* tp_flags */
+ 	0,					/* tp_doc */
+ 	(traverseproc)list_traverse,		/* tp_traverse */
+ 	(inquiry)list_clear,			/* tp_clear */
+	list_richcompare,			/* tp_richcompare */
 };
 
 
@@ -1453,7 +1577,7 @@ PyTypeObject PyList_Type = {
    compare a list that's being sorted... */
 
 static PyObject *
-immutable_list_op(/*No args!*/)
+immutable_list_op(void)
 {
 	PyErr_SetString(PyExc_TypeError,
 			"a list cannot be modified while it is being sorted");
@@ -1472,44 +1596,54 @@ static PyMethodDef immutable_list_methods[] = {
 };
 
 static PyObject *
-immutable_list_getattr(f, name)
-	PyListObject *f;
-	char *name;
+immutable_list_getattr(PyListObject *f, char *name)
 {
 	return Py_FindMethod(immutable_list_methods, (PyObject *)f, name);
 }
 
 static int
-immutable_list_ass(/*No args!*/)
+immutable_list_ass(void)
 {
 	immutable_list_op();
 	return -1;
 }
 
 static PySequenceMethods immutable_list_as_sequence = {
-	(inquiry)list_length, /*sq_length*/
-	(binaryfunc)list_concat, /*sq_concat*/
-	(intargfunc)list_repeat, /*sq_repeat*/
-	(intargfunc)list_item, /*sq_item*/
-	(intintargfunc)list_slice, /*sq_slice*/
-	(intobjargproc)immutable_list_ass, /*sq_ass_item*/
-	(intintobjargproc)immutable_list_ass, /*sq_ass_slice*/
+	(inquiry)list_length,			/* sq_length */
+	(binaryfunc)list_concat,		/* sq_concat */
+	(intargfunc)list_repeat,		/* sq_repeat */
+	(intargfunc)list_item,			/* sq_item */
+	(intintargfunc)list_slice,		/* sq_slice */
+	(intobjargproc)immutable_list_ass,	/* sq_ass_item */
+	(intintobjargproc)immutable_list_ass,	/* sq_ass_slice */
+	(objobjproc)list_contains,		/* sq_contains */
 };
 
 static PyTypeObject immutable_list_type = {
 	PyObject_HEAD_INIT(&PyType_Type)
 	0,
 	"list (immutable, during sort)",
-	sizeof(PyListObject),
+	sizeof(PyListObject) + PyGC_HEAD_SIZE,
 	0,
-	0,		/*tp_dealloc*/ /* Cannot happen */
-	(printfunc)list_print, /*tp_print*/
-	(getattrfunc)immutable_list_getattr, /*tp_getattr*/
-	0,		/*tp_setattr*/
-	0,		/*tp_compare*/ /* Won't be called */
-	(reprfunc)list_repr, /*tp_repr*/
-	0,		/*tp_as_number*/
-	&immutable_list_as_sequence,	/*tp_as_sequence*/
-	0,		/*tp_as_mapping*/
+	0, /* Cannot happen */			/* tp_dealloc */
+	(printfunc)list_print,			/* tp_print */
+	(getattrfunc)immutable_list_getattr,	/* tp_getattr */
+	0,					/* tp_setattr */
+	0, /* Won't be called */		/* tp_compare */
+	(reprfunc)list_repr,			/* tp_repr */
+	0,					/* tp_as_number */
+	&immutable_list_as_sequence,		/* tp_as_sequence */
+	0,					/* tp_as_mapping */
+	0,					/* tp_hash */
+	0,					/* tp_call */
+	0,					/* tp_str */
+	0,					/* tp_getattro */
+	0,					/* tp_setattro */
+	0,					/* tp_as_buffer */
+	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_GC,	/* tp_flags */
+ 	0,					/* tp_doc */
+ 	(traverseproc)list_traverse,		/* tp_traverse */
+	0,					/* tp_clear */
+	list_richcompare,			/* tp_richcompare */
+	/* NOTE: This is *not* the standard list_type struct! */
 };
-
