@@ -1,33 +1,3 @@
-/***********************************************************
-Copyright 1991-1995 by Stichting Mathematisch Centrum, Amsterdam,
-The Netherlands.
-
-                        All Rights Reserved
-
-Permission to use, copy, modify, and distribute this software and its
-documentation for any purpose and without fee is hereby granted,
-provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in
-supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI or Corporation for National Research Initiatives or
-CNRI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior
-permission.
-
-While CWI is the initial source for this software, a modified version
-is made available by the Corporation for National Research Initiatives
-(CNRI) at the Internet address ftp://ftp.python.org.
-
-STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
-REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
-MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
-CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
-DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
-PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
-TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-PERFORMANCE OF THIS SOFTWARE.
-
-******************************************************************/
 
 /* Long (arbitrary precision) integer object implementation */
 
@@ -35,7 +5,6 @@ PERFORMANCE OF THIS SOFTWARE.
 
 #include "Python.h"
 #include "longintrepr.h"
-#include "mymath.h"
 
 #include <assert.h>
 #include <ctype.h>
@@ -43,11 +12,11 @@ PERFORMANCE OF THIS SOFTWARE.
 #define ABS(x) ((x) < 0 ? -(x) : (x))
 
 /* Forward */
-static PyLongObject *long_normalize Py_PROTO((PyLongObject *));
-static PyLongObject *mul1 Py_PROTO((PyLongObject *, wdigit));
-static PyLongObject *muladd1 Py_PROTO((PyLongObject *, wdigit, wdigit));
-static PyLongObject *divrem1 Py_PROTO((PyLongObject *, wdigit, digit *));
-static PyObject *long_format Py_PROTO((PyObject *aa, int base));
+static PyLongObject *long_normalize(PyLongObject *);
+static PyLongObject *mul1(PyLongObject *, wdigit);
+static PyLongObject *muladd1(PyLongObject *, wdigit, wdigit);
+static PyLongObject *divrem1(PyLongObject *, wdigit, digit *);
+static PyObject *long_format(PyObject *aa, int base, int addL);
 
 static int ticker;	/* XXX Could be shared with ceval? */
 
@@ -62,8 +31,7 @@ static int ticker;	/* XXX Could be shared with ceval? */
    of the algorithms used, this could save at most be one word anyway. */
 
 static PyLongObject *
-long_normalize(v)
-	register PyLongObject *v;
+long_normalize(register PyLongObject *v)
 {
 	int j = ABS(v->ob_size);
 	register int i = j;
@@ -79,8 +47,7 @@ long_normalize(v)
    Return NULL and set exception if we run out of memory. */
 
 PyLongObject *
-_PyLong_New(size)
-	int size;
+_PyLong_New(int size)
 {
 	return PyObject_NEW_VAR(PyLongObject, &PyLong_Type, size);
 }
@@ -88,8 +55,7 @@ _PyLong_New(size)
 /* Create a new long int object from a C long int */
 
 PyObject *
-PyLong_FromLong(ival)
-	long ival;
+PyLong_FromLong(long ival)
 {
 	/* Assume a C long fits in at most 5 'digits' */
 	/* Works on both 32- and 64-bit machines */
@@ -113,8 +79,7 @@ PyLong_FromLong(ival)
 /* Create a new long int object from a C unsigned long int */
 
 PyObject *
-PyLong_FromUnsignedLong(ival)
-	unsigned long ival;
+PyLong_FromUnsignedLong(unsigned long ival)
 {
 	/* Assume a C long fits in at most 5 'digits' */
 	/* Works on both 32- and 64-bit machines */
@@ -134,17 +99,17 @@ PyLong_FromUnsignedLong(ival)
 /* Create a new long int object from a C double */
 
 PyObject *
-#ifdef MPW
 PyLong_FromDouble(double dval)
-#else
-PyLong_FromDouble(dval)
-	double dval;
-#endif /* MPW */
 {
 	PyLongObject *v;
 	double frac;
 	int i, ndig, expo, neg;
 	neg = 0;
+	if (Py_IS_INFINITY(dval)) {
+		PyErr_SetString(PyExc_OverflowError,
+			"cannot convert float infinity to long");
+		return NULL;
+	}
 	if (dval < 0.0) {
 		neg = 1;
 		dval = -dval;
@@ -172,8 +137,7 @@ PyLong_FromDouble(dval)
    Returns -1 and sets an error condition if overflow occurs. */
 
 long
-PyLong_AsLong(vv)
-	PyObject *vv;
+PyLong_AsLong(PyObject *vv)
 {
 	/* This version by Tim Peters */
 	register PyLongObject *v;
@@ -209,7 +173,7 @@ PyLong_AsLong(vv)
 
  overflow:
 	PyErr_SetString(PyExc_OverflowError,
-			"long int too long to convert");
+			"long int too large to convert");
 	return -1;
 }
 
@@ -217,8 +181,7 @@ PyLong_AsLong(vv)
    Returns -1 and sets an error condition if overflow occurs. */
 
 unsigned long
-PyLong_AsUnsignedLong(vv)
-	PyObject *vv;
+PyLong_AsUnsignedLong(PyObject *vv)
 {
 	register PyLongObject *v;
 	unsigned long x, prev;
@@ -241,7 +204,7 @@ PyLong_AsUnsignedLong(vv)
 		x = (x << SHIFT) + v->ob_digit[i];
 		if ((x >> SHIFT) != prev) {
 			PyErr_SetString(PyExc_OverflowError,
-				"long int too long to convert");
+				"long int too large to convert");
 			return (unsigned long) -1;
 		}
 	}
@@ -251,8 +214,7 @@ PyLong_AsUnsignedLong(vv)
 /* Get a C double from a long int object. */
 
 double
-PyLong_AsDouble(vv)
-	PyObject *vv;
+PyLong_AsDouble(PyObject *vv)
 {
 	register PyLongObject *v;
 	double x;
@@ -280,8 +242,7 @@ PyLong_AsDouble(vv)
 /* Create a new long (or int) object from a C pointer */
 
 PyObject *
-PyLong_FromVoidPtr(p)
-	void *p;
+PyLong_FromVoidPtr(void *p)
 {
 #if SIZEOF_VOID_P == SIZEOF_LONG
 	return PyInt_FromLong((long)p);
@@ -300,8 +261,7 @@ PyLong_FromVoidPtr(p)
 /* Get a C pointer from a long object (or an int object in some cases) */
 
 void *
-PyLong_AsVoidPtr(vv)
-	PyObject *vv;
+PyLong_AsVoidPtr(PyObject *vv)
 {
 	/* This function will allow int or long objects. If vv is neither,
 	   then the PyLong_AsLong*() functions will raise the exception:
@@ -343,17 +303,16 @@ PyLong_AsVoidPtr(vv)
 /* Create a new long int object from a C LONG_LONG int */
 
 PyObject *
-PyLong_FromLongLong(ival)
-	LONG_LONG ival;
+PyLong_FromLongLong(LONG_LONG ival)
 {
 #if SIZEOF_LONG_LONG == SIZEOF_LONG
 	/* In case the compiler is faking it. */
 	return PyLong_FromLong( (long)ival );
 #else
-	if( ival <= (LONG_LONG)LONG_MAX ) {
+	if ((LONG_LONG)LONG_MIN <= ival && ival <= (LONG_LONG)LONG_MAX) {
 		return PyLong_FromLong( (long)ival );
 	}
-	else if( ival <= (unsigned LONG_LONG)ULONG_MAX ) {
+	else if (0 <= ival && ival <= (unsigned LONG_LONG)ULONG_MAX) {
 		return PyLong_FromUnsignedLong( (unsigned long)ival );
 	}
 	else {
@@ -385,8 +344,7 @@ PyLong_FromLongLong(ival)
 
 /* Create a new long int object from a C unsigned LONG_LONG int */
 PyObject *
-PyLong_FromUnsignedLongLong(ival)
-	unsigned LONG_LONG ival;
+PyLong_FromUnsignedLongLong(unsigned LONG_LONG ival)
 {
 #if SIZEOF_LONG_LONG == SIZEOF_LONG
 	/* In case the compiler is faking it. */
@@ -419,8 +377,7 @@ PyLong_FromUnsignedLongLong(ival)
    Returns -1 and sets an error condition if overflow occurs. */
 
 LONG_LONG
-PyLong_AsLongLong(vv)
-	PyObject *vv;
+PyLong_AsLongLong(PyObject *vv)
 {
 #if SIZEOF_LONG_LONG == SIZEOF_LONG
 	/* In case the compiler is faking it. */
@@ -460,8 +417,7 @@ PyLong_AsLongLong(vv)
 }
 
 unsigned LONG_LONG
-PyLong_AsUnsignedLongLong(vv)
-	PyObject *vv;
+PyLong_AsUnsignedLongLong(PyObject *vv)
 {
 #if SIZEOF_LONG_LONG == 4
 	/* In case the compiler is faking it. */
@@ -501,12 +457,44 @@ PyLong_AsUnsignedLongLong(vv)
 }
 #endif /* HAVE_LONG_LONG */
 
+
+static int
+convert_binop(PyObject *v, PyObject *w, PyLongObject **a, PyLongObject **b) {
+	if (PyLong_Check(v)) { 
+		*a = (PyLongObject *) v;
+		Py_INCREF(v);
+	}
+	else if (PyInt_Check(v)) {
+		*a = (PyLongObject *) PyLong_FromLong(PyInt_AS_LONG(v));
+	}
+	else {
+		return 0;
+	}
+	if (PyLong_Check(w)) { 
+		*b = (PyLongObject *) w;
+		Py_INCREF(w);
+	}
+	else if (PyInt_Check(w)) {
+		*b = (PyLongObject *) PyLong_FromLong(PyInt_AS_LONG(w));
+	}
+	else {
+		Py_DECREF(*a);
+		return 0;
+	}
+	return 1;
+}
+
+#define CONVERT_BINOP(v, w, a, b) \
+	if (!convert_binop(v, w, a, b)) { \
+		Py_INCREF(Py_NotImplemented); \
+		return Py_NotImplemented; \
+	}
+
+
 /* Multiply by a single digit, ignoring the sign. */
 
 static PyLongObject *
-mul1(a, n)
-	PyLongObject *a;
-	wdigit n;
+mul1(PyLongObject *a, wdigit n)
 {
 	return muladd1(a, n, (digit)0);
 }
@@ -514,10 +502,7 @@ mul1(a, n)
 /* Multiply by a single digit and add a single digit, ignoring the sign. */
 
 static PyLongObject *
-muladd1(a, n, extra)
-	PyLongObject *a;
-	wdigit n;
-	wdigit extra;
+muladd1(PyLongObject *a, wdigit n, wdigit extra)
 {
 	int size_a = ABS(a->ob_size);
 	PyLongObject *z = _PyLong_New(size_a+1);
@@ -540,10 +525,7 @@ muladd1(a, n, extra)
    The sign of a is ignored; n should not be zero. */
 
 static PyLongObject *
-divrem1(a, n, prem)
-	PyLongObject *a;
-	wdigit n;
-	digit *prem;
+divrem1(PyLongObject *a, wdigit n, digit *prem)
 {
 	int size = ABS(a->ob_size);
 	PyLongObject *z;
@@ -565,13 +547,10 @@ divrem1(a, n, prem)
 
 /* Convert a long int object to a string, using a given conversion base.
    Return a string object.
-   If base is 8 or 16, add the proper prefix '0' or '0x'.
-   External linkage: used in bltinmodule.c by hex() and oct(). */
+   If base is 8 or 16, add the proper prefix '0' or '0x'. */
 
 static PyObject *
-long_format(aa, base)
-	PyObject *aa;
-	int base;
+long_format(PyObject *aa, int base, int addL)
 {
 	register PyLongObject *a = (PyLongObject *)aa;
 	PyStringObject *str;
@@ -594,13 +573,14 @@ long_format(aa, base)
 		++bits;
 		i >>= 1;
 	}
-	i = 6 + (size_a*SHIFT + bits-1) / bits;
+	i = 5 + (addL ? 1 : 0) + (size_a*SHIFT + bits-1) / bits;
 	str = (PyStringObject *) PyString_FromStringAndSize((char *)0, i);
 	if (str == NULL)
 		return NULL;
 	p = PyString_AS_STRING(str) + i;
 	*p = '\0';
-	*--p = 'L';
+        if (addL)
+                *--p = 'L';
 	if (a->ob_size < 0)
 		sign = '-';
 	
@@ -615,7 +595,8 @@ long_format(aa, base)
 		int last = abs(a->ob_size);
 		int basebits = 1;
 		i = base;
-		while ((i >>= 1) > 1) ++basebits;
+		while ((i >>= 1) > 1)
+			++basebits;
 		
 		i = 0;
 		for (;;) {
@@ -697,33 +678,16 @@ long_format(aa, base)
 	return (PyObject *)str;
 }
 
-#if 0
-/* Convert a string to a long int object, in a given base.
-   Base zero implies a default depending on the number.
-   External linkage: used in compile.c and stropmodule.c. */
-
 PyObject *
-long_scan(str, base)
-	char *str;
-	int base;
-{
-	return PyLong_FromString(str, (char **)NULL, base);
-}
-#endif
-
-PyObject *
-PyLong_FromString(str, pend, base)
-	char *str;
-	char **pend;
-	int base;
+PyLong_FromString(char *str, char **pend, int base)
 {
 	int sign = 1;
-	char *start;
+	char *start, *orig_str = str;
 	PyLongObject *z;
 	
 	if ((base != 0 && base < 2) || base > 36) {
 		PyErr_SetString(PyExc_ValueError,
-				"invalid base for long literal");
+				"long() arg 2 must be >= 2 and <= 36");
 		return NULL;
 	}
 	while (*str != '\0' && isspace(Py_CHARMASK(*str)))
@@ -766,39 +730,62 @@ PyLong_FromString(str, pend, base)
 	}
 	if (z == NULL)
 		return NULL;
-	if (str == start) {
-		PyErr_SetString(PyExc_ValueError,
-				"no digits in long int constant");
-		Py_DECREF(z);
-		return NULL;
-	}
+	if (str == start)
+		goto onError;
 	if (sign < 0 && z != NULL && z->ob_size != 0)
 		z->ob_size = -(z->ob_size);
+	if (*str == 'L' || *str == 'l')
+		str++;
+	while (*str && isspace(Py_CHARMASK(*str)))
+		str++;
+	if (*str != '\0')
+		goto onError;
 	if (pend)
 		*pend = str;
 	return (PyObject *) z;
+
+ onError:
+	PyErr_Format(PyExc_ValueError, 
+		     "invalid literal for long(): %.200s", orig_str);
+	Py_XDECREF(z);
+	return NULL;
 }
 
+PyObject *
+PyLong_FromUnicode(Py_UNICODE *u, int length, int base)
+{
+	char buffer[256];
+
+	if (length >= sizeof(buffer)) {
+		PyErr_SetString(PyExc_ValueError,
+				"long() literal too large to convert");
+		return NULL;
+	}
+	if (PyUnicode_EncodeDecimal(u, length, buffer, NULL))
+		return NULL;
+
+	return PyLong_FromString(buffer, NULL, base);
+}
+
+/* forward */
 static PyLongObject *x_divrem
-	Py_PROTO((PyLongObject *, PyLongObject *, PyLongObject **));
-static PyObject *long_pos Py_PROTO((PyLongObject *));
-static int long_divrem Py_PROTO((PyLongObject *, PyLongObject *,
-	PyLongObject **, PyLongObject **));
+	(PyLongObject *, PyLongObject *, PyLongObject **);
+static PyObject *long_pos(PyLongObject *);
+static int long_divrem(PyLongObject *, PyLongObject *,
+	PyLongObject **, PyLongObject **);
 
 /* Long division with remainder, top-level routine */
 
 static int
-long_divrem(a, b, pdiv, prem)
-	PyLongObject *a, *b;
-	PyLongObject **pdiv;
-	PyLongObject **prem;
+long_divrem(PyLongObject *a, PyLongObject *b,
+	    PyLongObject **pdiv, PyLongObject **prem)
 {
 	int size_a = ABS(a->ob_size), size_b = ABS(b->ob_size);
 	PyLongObject *z;
 	
 	if (size_b == 0) {
 		PyErr_SetString(PyExc_ZeroDivisionError,
-				"long division or modulo");
+				"long division or modulo by zero");
 		return -1;
 	}
 	if (size_a < size_b ||
@@ -837,9 +824,7 @@ long_divrem(a, b, pdiv, prem)
 /* Unsigned long division with remainder -- the algorithm */
 
 static PyLongObject *
-x_divrem(v1, w1, prem)
-	PyLongObject *v1, *w1;
-	PyLongObject **prem;
+x_divrem(PyLongObject *v1, PyLongObject *w1, PyLongObject **prem)
 {
 	int size_v = ABS(v1->ob_size), size_w = ABS(w1->ob_size);
 	digit d = (digit) ((twodigits)BASE / (w1->ob_digit[size_w-1] + 1));
@@ -893,7 +878,9 @@ x_divrem(v1, w1, prem)
 			carry += v->ob_digit[i+k] - z
 				+ ((twodigits)zz << SHIFT);
 			v->ob_digit[i+k] = carry & MASK;
-			carry = (carry >> SHIFT) - zz;
+			carry = Py_ARITHMETIC_RIGHT_SHIFT(BASE_TWODIGITS_TYPE,
+							  carry, SHIFT);
+			carry -= zz;
 		}
 		
 		if (i+k < size_v) {
@@ -910,7 +897,9 @@ x_divrem(v1, w1, prem)
 			for (i = 0; i < size_w && i+k < size_v; ++i) {
 				carry += v->ob_digit[i+k] + w->ob_digit[i];
 				v->ob_digit[i+k] = carry & MASK;
-				carry >>= SHIFT;
+				carry = Py_ARITHMETIC_RIGHT_SHIFT(
+						BASE_TWODIGITS_TYPE,
+						carry, SHIFT);
 			}
 		}
 	} /* for j, k */
@@ -933,48 +922,26 @@ x_divrem(v1, w1, prem)
 
 /* Methods */
 
-/* Forward */
-static void long_dealloc Py_PROTO((PyObject *));
-static PyObject *long_repr Py_PROTO((PyObject *));
-static int long_compare Py_PROTO((PyLongObject *, PyLongObject *));
-static long long_hash Py_PROTO((PyLongObject *));
-
-static PyObject *long_add Py_PROTO((PyLongObject *, PyLongObject *));
-static PyObject *long_sub Py_PROTO((PyLongObject *, PyLongObject *));
-static PyObject *long_mul Py_PROTO((PyLongObject *, PyLongObject *));
-static PyObject *long_div Py_PROTO((PyLongObject *, PyLongObject *));
-static PyObject *long_mod Py_PROTO((PyLongObject *, PyLongObject *));
-static PyObject *long_divmod Py_PROTO((PyLongObject *, PyLongObject *));
-static PyObject *long_pow
-	Py_PROTO((PyLongObject *, PyLongObject *, PyLongObject *));
-static PyObject *long_neg Py_PROTO((PyLongObject *));
-static PyObject *long_pos Py_PROTO((PyLongObject *));
-static PyObject *long_abs Py_PROTO((PyLongObject *));
-static int long_nonzero Py_PROTO((PyLongObject *));
-static PyObject *long_invert Py_PROTO((PyLongObject *));
-static PyObject *long_lshift Py_PROTO((PyLongObject *, PyLongObject *));
-static PyObject *long_rshift Py_PROTO((PyLongObject *, PyLongObject *));
-static PyObject *long_and Py_PROTO((PyLongObject *, PyLongObject *));
-static PyObject *long_xor Py_PROTO((PyLongObject *, PyLongObject *));
-static PyObject *long_or Py_PROTO((PyLongObject *, PyLongObject *));
-
 static void
-long_dealloc(v)
-	PyObject *v;
+long_dealloc(PyObject *v)
 {
-	PyMem_DEL(v);
+	PyObject_DEL(v);
 }
 
 static PyObject *
-long_repr(v)
-	PyObject *v;
+long_repr(PyObject *v)
 {
-	return long_format(v, 10);
+	return long_format(v, 10, 1);
+}
+
+static PyObject *
+long_str(PyObject *v)
+{
+	return long_format(v, 10, 0);
 }
 
 static int
-long_compare(a, b)
-	PyLongObject *a, *b;
+long_compare(PyLongObject *a, PyLongObject *b)
 {
 	int sign;
 	
@@ -1000,8 +967,7 @@ long_compare(a, b)
 }
 
 static long
-long_hash(v)
-	PyLongObject *v;
+long_hash(PyLongObject *v)
 {
 	long x;
 	int i, sign;
@@ -1030,10 +996,8 @@ long_hash(v)
 
 /* Add the absolute values of two long integers. */
 
-static PyLongObject *x_add Py_PROTO((PyLongObject *, PyLongObject *));
 static PyLongObject *
-x_add(a, b)
-	PyLongObject *a, *b;
+x_add(PyLongObject *a, PyLongObject *b)
 {
 	int size_a = ABS(a->ob_size), size_b = ABS(b->ob_size);
 	PyLongObject *z;
@@ -1053,8 +1017,6 @@ x_add(a, b)
 	for (i = 0; i < size_b; ++i) {
 		carry += a->ob_digit[i] + b->ob_digit[i];
 		z->ob_digit[i] = carry & MASK;
-		/* The following assumes unsigned shifts don't
-		   propagate the sign bit. */
 		carry >>= SHIFT;
 	}
 	for (; i < size_a; ++i) {
@@ -1068,10 +1030,8 @@ x_add(a, b)
 
 /* Subtract the absolute values of two integers. */
 
-static PyLongObject *x_sub Py_PROTO((PyLongObject *, PyLongObject *));
 static PyLongObject *
-x_sub(a, b)
-	PyLongObject *a, *b;
+x_sub(PyLongObject *a, PyLongObject *b)
 {
 	int size_a = ABS(a->ob_size), size_b = ABS(b->ob_size);
 	PyLongObject *z;
@@ -1115,6 +1075,7 @@ x_sub(a, b)
 		borrow = a->ob_digit[i] - borrow;
 		z->ob_digit[i] = borrow & MASK;
 		borrow >>= SHIFT;
+		borrow &= 1; /* Keep only one sign bit */
 	}
 	assert(borrow == 0);
 	if (sign < 0)
@@ -1123,12 +1084,12 @@ x_sub(a, b)
 }
 
 static PyObject *
-long_add(a, b)
-	PyLongObject *a;
-	PyLongObject *b;
+long_add(PyLongObject *v, PyLongObject *w)
 {
-	PyLongObject *z;
+	PyLongObject *a, *b, *z;
 	
+	CONVERT_BINOP((PyObject *)v, (PyObject *)w, &a, &b);
+
 	if (a->ob_size < 0) {
 		if (b->ob_size < 0) {
 			z = x_add(a, b);
@@ -1144,16 +1105,18 @@ long_add(a, b)
 		else
 			z = x_add(a, b);
 	}
+	Py_DECREF(a);
+	Py_DECREF(b);
 	return (PyObject *)z;
 }
 
 static PyObject *
-long_sub(a, b)
-	PyLongObject *a;
-	PyLongObject *b;
+long_sub(PyLongObject *v, PyLongObject *w)
 {
-	PyLongObject *z;
+	PyLongObject *a, *b, *z;
 	
+	CONVERT_BINOP((PyObject *)v, (PyObject *)w, &a, &b);
+
 	if (a->ob_size < 0) {
 		if (b->ob_size < 0)
 			z = x_sub(a, b);
@@ -1168,24 +1131,58 @@ long_sub(a, b)
 		else
 			z = x_sub(a, b);
 	}
+	Py_DECREF(a);
+	Py_DECREF(b);
 	return (PyObject *)z;
 }
 
 static PyObject *
-long_mul(a, b)
-	PyLongObject *a;
-	PyLongObject *b;
+long_repeat(PyObject *v, PyLongObject *w)
 {
+	/* sequence * long */
+	long n = PyLong_AsLong((PyObject *) w);
+	if (n == -1 && PyErr_Occurred())
+		return NULL;
+	else
+		return (*v->ob_type->tp_as_sequence->sq_repeat)(v, n);
+}
+
+static PyObject *
+long_mul(PyLongObject *v, PyLongObject *w)
+{
+	PyLongObject *a, *b, *z;
 	int size_a;
 	int size_b;
-	PyLongObject *z;
 	int i;
 	
+	if (v->ob_type->tp_as_sequence &&
+			v->ob_type->tp_as_sequence->sq_repeat) {
+		return long_repeat((PyObject *)v, w);
+	}
+	else if (w->ob_type->tp_as_sequence &&
+			w->ob_type->tp_as_sequence->sq_repeat) {
+		return long_repeat((PyObject *)w, v);
+	}
+
+	CONVERT_BINOP((PyObject *)v, (PyObject *)w, &a, &b);
+
 	size_a = ABS(a->ob_size);
 	size_b = ABS(b->ob_size);
+	if (size_a > size_b) {
+		/* we are faster with the small object on the left */
+		int hold_sa = size_a;
+		PyLongObject *hold_a = a;
+		size_a = size_b;
+		size_b = hold_sa;
+		a = b;
+		b = hold_a;
+	}
 	z = _PyLong_New(size_a + size_b);
-	if (z == NULL)
+	if (z == NULL) {
+		Py_DECREF(a);
+		Py_DECREF(b);
 		return NULL;
+	}
 	for (i = 0; i < z->ob_size; ++i)
 		z->ob_digit[i] = 0;
 	for (i = 0; i < size_a; ++i) {
@@ -1194,6 +1191,8 @@ long_mul(a, b)
 		int j;
 		
 		SIGCHECK({
+			Py_DECREF(a);
+			Py_DECREF(b);
 			Py_DECREF(z);
 			return NULL;
 		})
@@ -1213,6 +1212,8 @@ long_mul(a, b)
 		z->ob_size = -(z->ob_size);
 	if (b->ob_size < 0)
 		z->ob_size = -(z->ob_size);
+	Py_DECREF(a);
+	Py_DECREF(b);
 	return (PyObject *) long_normalize(z);
 }
 
@@ -1231,14 +1232,9 @@ long_mul(a, b)
    have different signs.  We then subtract one from the 'div'
    part of the outcome to keep the invariant intact. */
 
-static int l_divmod Py_PROTO((PyLongObject *, PyLongObject *,
-	PyLongObject **, PyLongObject **));
 static int
-l_divmod(v, w, pdiv, pmod)
-	PyLongObject *v;
-	PyLongObject *w;
-	PyLongObject **pdiv;
-	PyLongObject **pmod;
+l_divmod(PyLongObject *v, PyLongObject *w, 
+	 PyLongObject **pdiv, PyLongObject **pmod)
 {
 	PyLongObject *div, *mod;
 	
@@ -1273,38 +1269,54 @@ l_divmod(v, w, pdiv, pmod)
 }
 
 static PyObject *
-long_div(v, w)
-	PyLongObject *v;
-	PyLongObject *w;
+long_div(PyObject *v, PyObject *w)
 {
-	PyLongObject *div, *mod;
-	if (l_divmod(v, w, &div, &mod) < 0)
+	PyLongObject *a, *b, *div, *mod;
+
+	CONVERT_BINOP(v, w, &a, &b);
+
+	if (l_divmod(a, b, &div, &mod) < 0) {
+		Py_DECREF(a);
+		Py_DECREF(b);
 		return NULL;
+	}
+	Py_DECREF(a);
+	Py_DECREF(b);
 	Py_DECREF(mod);
 	return (PyObject *)div;
 }
 
 static PyObject *
-long_mod(v, w)
-	PyLongObject *v;
-	PyLongObject *w;
+long_mod(PyObject *v, PyObject *w)
 {
-	PyLongObject *div, *mod;
-	if (l_divmod(v, w, &div, &mod) < 0)
+	PyLongObject *a, *b, *div, *mod;
+
+	CONVERT_BINOP(v, w, &a, &b);
+
+	if (l_divmod(a, b, &div, &mod) < 0) {
+		Py_DECREF(a);
+		Py_DECREF(b);
 		return NULL;
+	}
+	Py_DECREF(a);
+	Py_DECREF(b);
 	Py_DECREF(div);
 	return (PyObject *)mod;
 }
 
 static PyObject *
-long_divmod(v, w)
-	PyLongObject *v;
-	PyLongObject *w;
+long_divmod(PyObject *v, PyObject *w)
 {
+	PyLongObject *a, *b, *div, *mod;
 	PyObject *z;
-	PyLongObject *div, *mod;
-	if (l_divmod(v, w, &div, &mod) < 0)
+
+	CONVERT_BINOP(v, w, &a, &b);
+
+	if (l_divmod(a, b, &div, &mod) < 0) {
+		Py_DECREF(a);
+		Py_DECREF(b);
 		return NULL;
+	}
 	z = PyTuple_New(2);
 	if (z != NULL) {
 		PyTuple_SetItem(z, 0, (PyObject *) div);
@@ -1314,26 +1326,46 @@ long_divmod(v, w)
 		Py_DECREF(div);
 		Py_DECREF(mod);
 	}
+	Py_DECREF(a);
+	Py_DECREF(b);
 	return z;
 }
 
 static PyObject *
-long_pow(a, b, c)
-	PyLongObject *a;
-	PyLongObject *b;
-	PyLongObject *c;
+long_pow(PyObject *v, PyObject *w, PyObject *x)
 {
+	PyLongObject *a, *b;
+	PyObject *c;
 	PyLongObject *z, *div, *mod;
 	int size_b, i;
+
+	CONVERT_BINOP(v, w, &a, &b);
+	if (PyLong_Check(x) || Py_None == x) { 
+		c = x;
+		Py_INCREF(x);
+	}
+	else if (PyInt_Check(x)) {
+		c = PyLong_FromLong(PyInt_AS_LONG(x));
+	}
+	else {
+		Py_DECREF(a);
+		Py_DECREF(b);
+		Py_INCREF(Py_NotImplemented);
+		return Py_NotImplemented;
+	}
 	
 	size_b = b->ob_size;
 	if (size_b < 0) {
-		PyErr_SetString(PyExc_ValueError,
-				"long integer to the negative power");
-		return NULL;
+		if (a->ob_size)
+			PyErr_SetString(PyExc_ValueError,
+					"long integer to a negative power");
+		else
+			PyErr_SetString(PyExc_ZeroDivisionError,
+					"zero to a negative power");
+		z = NULL;
+		goto error;
 	}
 	z = (PyLongObject *)PyLong_FromLong(1L);
-	Py_INCREF(a);
 	for (i = 0; i < size_b; ++i) {
 		digit bi = b->ob_digit[i];
 		int j;
@@ -1344,8 +1376,13 @@ long_pow(a, b, c)
 			if (bi & 1) {
 				temp = (PyLongObject *)long_mul(z, a);
 				Py_DECREF(z);
-			 	if ((PyObject*)c!=Py_None && temp!=NULL) {
-			 		l_divmod(temp, c, &div, &mod);
+			 	if (c!=Py_None && temp!=NULL) {
+			 		if (l_divmod(temp,(PyLongObject *)c,
+							&div,&mod) < 0) {
+						Py_DECREF(temp);
+						z = NULL;
+						goto error;
+					}
 				 	Py_XDECREF(div);
 				 	Py_DECREF(temp);
 				 	temp = mod;
@@ -1359,8 +1396,13 @@ long_pow(a, b, c)
 				break;
 			temp = (PyLongObject *)long_mul(a, a);
 			Py_DECREF(a);
-		 	if ((PyObject*)c!=Py_None && temp!=NULL) {
-			 	l_divmod(temp, c, &div, &mod);
+		 	if (c!=Py_None && temp!=NULL) {
+			 	if (l_divmod(temp, (PyLongObject *)c, &div,
+							&mod) < 0) {
+					Py_DECREF(temp);
+					z = NULL;
+					goto error;
+				}
 			 	Py_XDECREF(div);
 			 	Py_DECREF(temp);
 			 	temp = mod;
@@ -1375,19 +1417,26 @@ long_pow(a, b, c)
 		if (a == NULL || z == NULL)
 			break;
 	}
-	Py_XDECREF(a);
-	if ((PyObject*)c!=Py_None && z!=NULL) {
-			l_divmod(z, c, &div, &mod);
+	if (c!=Py_None && z!=NULL) {
+		if (l_divmod(z, (PyLongObject *)c, &div, &mod) < 0) {
+			Py_DECREF(z);
+			z = NULL;
+		}
+		else {
 			Py_XDECREF(div);
 			Py_DECREF(z);
-			z=mod;
+			z = mod;
+		}
 	}
+  error:
+	Py_XDECREF(a);
+	Py_DECREF(b);
+	Py_DECREF(c);
 	return (PyObject *)z;
 }
 
 static PyObject *
-long_invert(v)
-	PyLongObject *v;
+long_invert(PyLongObject *v)
 {
 	/* Implement ~x as -(x+1) */
 	PyLongObject *x;
@@ -1405,16 +1454,14 @@ long_invert(v)
 }
 
 static PyObject *
-long_pos(v)
-	PyLongObject *v;
+long_pos(PyLongObject *v)
 {
 	Py_INCREF(v);
 	return (PyObject *)v;
 }
 
 static PyObject *
-long_neg(v)
-	PyLongObject *v;
+long_neg(PyLongObject *v)
 {
 	PyLongObject *z;
 	int i, n;
@@ -1434,8 +1481,7 @@ long_neg(v)
 }
 
 static PyObject *
-long_abs(v)
-	PyLongObject *v;
+long_abs(PyLongObject *v)
 {
 	if (v->ob_size < 0)
 		return long_neg(v);
@@ -1446,88 +1492,100 @@ long_abs(v)
 }
 
 static int
-long_nonzero(v)
-	PyLongObject *v;
+long_nonzero(PyLongObject *v)
 {
 	return ABS(v->ob_size) != 0;
 }
 
 static PyObject *
-long_rshift(a, b)
-	PyLongObject *a;
-	PyLongObject *b;
+long_rshift(PyLongObject *v, PyLongObject *w)
 {
-	PyLongObject *z;
+	PyLongObject *a, *b;
+	PyLongObject *z = NULL;
 	long shiftby;
 	int newsize, wordshift, loshift, hishift, i, j;
 	digit lomask, himask;
 	
+	CONVERT_BINOP((PyObject *)v, (PyObject *)w, &a, &b);
+
 	if (a->ob_size < 0) {
 		/* Right shifting negative numbers is harder */
-		PyLongObject *a1, *a2, *a3;
+		PyLongObject *a1, *a2;
 		a1 = (PyLongObject *) long_invert(a);
-		if (a1 == NULL) return NULL;
+		if (a1 == NULL)
+			goto rshift_error;
 		a2 = (PyLongObject *) long_rshift(a1, b);
 		Py_DECREF(a1);
-		if (a2 == NULL) return NULL;
-		a3 = (PyLongObject *) long_invert(a2);
+		if (a2 == NULL)
+			goto rshift_error;
+		z = (PyLongObject *) long_invert(a2);
 		Py_DECREF(a2);
-		return (PyObject *) a3;
 	}
-	
-	shiftby = PyLong_AsLong((PyObject *)b);
-	if (shiftby == -1L && PyErr_Occurred())
-		return NULL;
-	if (shiftby < 0) {
-		PyErr_SetString(PyExc_ValueError, "negative shift count");
-		return NULL;
+	else {
+		
+		shiftby = PyLong_AsLong((PyObject *)b);
+		if (shiftby == -1L && PyErr_Occurred())
+			goto rshift_error;
+		if (shiftby < 0) {
+			PyErr_SetString(PyExc_ValueError,
+					"negative shift count");
+			goto rshift_error;
+		}
+		wordshift = shiftby / SHIFT;
+		newsize = ABS(a->ob_size) - wordshift;
+		if (newsize <= 0) {
+			z = _PyLong_New(0);
+			Py_DECREF(a);
+			Py_DECREF(b);
+			return (PyObject *)z;
+		}
+		loshift = shiftby % SHIFT;
+		hishift = SHIFT - loshift;
+		lomask = ((digit)1 << hishift) - 1;
+		himask = MASK ^ lomask;
+		z = _PyLong_New(newsize);
+		if (z == NULL)
+			goto rshift_error;
+		if (a->ob_size < 0)
+			z->ob_size = -(z->ob_size);
+		for (i = 0, j = wordshift; i < newsize; i++, j++) {
+			z->ob_digit[i] = (a->ob_digit[j] >> loshift) & lomask;
+			if (i+1 < newsize)
+				z->ob_digit[i] |=
+				  (a->ob_digit[j+1] << hishift) & himask;
+		}
+		z = long_normalize(z);
 	}
-	wordshift = shiftby / SHIFT;
-	newsize = ABS(a->ob_size) - wordshift;
-	if (newsize <= 0) {
-		z = _PyLong_New(0);
-		return (PyObject *)z;
-	}
-	loshift = shiftby % SHIFT;
-	hishift = SHIFT - loshift;
-	lomask = ((digit)1 << hishift) - 1;
-	himask = MASK ^ lomask;
-	z = _PyLong_New(newsize);
-	if (z == NULL)
-		return NULL;
-	if (a->ob_size < 0)
-		z->ob_size = -(z->ob_size);
-	for (i = 0, j = wordshift; i < newsize; i++, j++) {
-		z->ob_digit[i] = (a->ob_digit[j] >> loshift) & lomask;
-		if (i+1 < newsize)
-			z->ob_digit[i] |=
-			  (a->ob_digit[j+1] << hishift) & himask;
-	}
-	return (PyObject *) long_normalize(z);
+rshift_error:
+	Py_DECREF(a);
+	Py_DECREF(b);
+	return (PyObject *) z;
+
 }
 
 static PyObject *
-long_lshift(a, b)
-	PyLongObject *a;
-	PyLongObject *b;
+long_lshift(PyObject *v, PyObject *w)
 {
 	/* This version due to Tim Peters */
-	PyLongObject *z;
+	PyLongObject *a, *b;
+	PyLongObject *z = NULL;
 	long shiftby;
 	int oldsize, newsize, wordshift, remshift, i, j;
 	twodigits accum;
 	
+	CONVERT_BINOP(v, w, &a, &b);
+
 	shiftby = PyLong_AsLong((PyObject *)b);
 	if (shiftby == -1L && PyErr_Occurred())
-		return NULL;
+		goto lshift_error;
 	if (shiftby < 0) {
 		PyErr_SetString(PyExc_ValueError, "negative shift count");
-		return NULL;
+		goto lshift_error;
 	}
 	if ((long)(int)shiftby != shiftby) {
 		PyErr_SetString(PyExc_ValueError,
 				"outrageous left shift count");
-		return NULL;
+		goto lshift_error;
 	}
 	/* wordshift, remshift = divmod(shiftby, SHIFT) */
 	wordshift = (int)shiftby / SHIFT;
@@ -1539,7 +1597,7 @@ long_lshift(a, b)
 		++newsize;
 	z = _PyLong_New(newsize);
 	if (z == NULL)
-		return NULL;
+		goto lshift_error;
 	if (a->ob_size < 0)
 		z->ob_size = -(z->ob_size);
 	for (i = 0; i < wordshift; i++)
@@ -1554,7 +1612,11 @@ long_lshift(a, b)
 		z->ob_digit[newsize-1] = (digit)accum;
 	else	
 		assert(!accum);
-	return (PyObject *) long_normalize(z);
+	z = long_normalize(z);
+lshift_error:
+	Py_DECREF(a);
+	Py_DECREF(b);
+	return (PyObject *) z;
 }
 
 
@@ -1563,12 +1625,10 @@ long_lshift(a, b)
 #define MAX(x, y) ((x) < (y) ? (y) : (x))
 #define MIN(x, y) ((x) > (y) ? (y) : (x))
 
-static PyObject *long_bitwise Py_PROTO((PyLongObject *, int, PyLongObject *));
 static PyObject *
-long_bitwise(a, op, b)
-	PyLongObject *a;
-	int op; /* '&', '|', '^' */
-	PyLongObject *b;
+long_bitwise(PyLongObject *a,
+	     int op,  /* '&', '|', '^' */
+	     PyLongObject *b)
 {
 	digit maska, maskb; /* 0 or MASK */
 	int negz;
@@ -1667,36 +1727,46 @@ long_bitwise(a, op, b)
 }
 
 static PyObject *
-long_and(a, b)
-	PyLongObject *a;
-	PyLongObject *b;
+long_and(PyObject *v, PyObject *w)
 {
-	return long_bitwise(a, '&', b);
+	PyLongObject *a, *b;
+	PyObject *c;
+	CONVERT_BINOP(v, w, &a, &b);
+	c = long_bitwise(a, '&', b);
+	Py_DECREF(a);
+	Py_DECREF(b);
+	return c;
 }
 
 static PyObject *
-long_xor(a, b)
-	PyLongObject *a;
-	PyLongObject *b;
+long_xor(PyObject *v, PyObject *w)
 {
-	return long_bitwise(a, '^', b);
+	PyLongObject *a, *b;
+	PyObject *c;
+	CONVERT_BINOP(v, w, &a, &b);
+	c = long_bitwise(a, '^', b);
+	Py_DECREF(a);
+	Py_DECREF(b);
+	return c;
 }
 
 static PyObject *
-long_or(a, b)
-	PyLongObject *a;
-	PyLongObject *b;
+long_or(PyObject *v, PyObject *w)
 {
-	return long_bitwise(a, '|', b);
+	PyLongObject *a, *b;
+	PyObject *c;
+	CONVERT_BINOP(v, w, &a, &b);
+	c = long_bitwise(a, '|', b);
+	Py_DECREF(a);
+	Py_DECREF(b);
+	return c;
 }
 
 static int
-long_coerce(pv, pw)
-	PyObject **pv;
-	PyObject **pw;
+long_coerce(PyObject **pv, PyObject **pw)
 {
 	if (PyInt_Check(*pw)) {
-		*pw = PyLong_FromLong(PyInt_AsLong(*pw));
+		*pw = PyLong_FromLong(PyInt_AS_LONG(*pw));
 		Py_INCREF(*pv);
 		return 0;
 	}
@@ -1704,8 +1774,7 @@ long_coerce(pv, pw)
 }
 
 static PyObject *
-long_int(v)
-	PyObject *v;
+long_int(PyObject *v)
 {
 	long x;
 	x = PyLong_AsLong(v);
@@ -1715,16 +1784,14 @@ long_int(v)
 }
 
 static PyObject *
-long_long(v)
-	PyObject *v;
+long_long(PyObject *v)
 {
 	Py_INCREF(v);
 	return v;
 }
 
 static PyObject *
-long_float(v)
-	PyObject *v;
+long_float(PyObject *v)
 {
 	double result;
 	PyFPE_START_PROTECT("long_float", return 0)
@@ -1734,50 +1801,52 @@ long_float(v)
 }
 
 static PyObject *
-long_oct(v)
-	PyObject *v;
+long_oct(PyObject *v)
 {
-	return long_format(v, 8);
+	return long_format(v, 8, 1);
 }
 
 static PyObject *
-long_hex(v)
-	PyObject *v;
+long_hex(PyObject *v)
 {
-	return long_format(v, 16);
+	return long_format(v, 16, 1);
 }
 
-
-#define UF (unaryfunc)
-#define BF (binaryfunc)
-#define TF (ternaryfunc)
-#define IF (inquiry)
-
 static PyNumberMethods long_as_number = {
-	BF long_add,	/*nb_add*/
-	BF long_sub,	/*nb_subtract*/
-	BF long_mul,	/*nb_multiply*/
-	BF long_div,	/*nb_divide*/
-	BF long_mod,	/*nb_remainder*/
-	BF long_divmod,	/*nb_divmod*/
-	TF long_pow,	/*nb_power*/
-	UF long_neg,	/*nb_negative*/
-	UF long_pos,	/*tp_positive*/
-	UF long_abs,	/*tp_absolute*/
-	IF long_nonzero,/*tp_nonzero*/
-	UF long_invert,	/*nb_invert*/
-	BF long_lshift,	/*nb_lshift*/
-	BF long_rshift,	/*nb_rshift*/
-	BF long_and,	/*nb_and*/
-	BF long_xor,	/*nb_xor*/
-	BF long_or,	/*nb_or*/
-	(int (*) Py_FPROTO((PyObject **, PyObject **)))
-	(coercion)long_coerce, /*nb_coerce*/
-	UF long_int,	/*nb_int*/
-	UF long_long,	/*nb_long*/
-	UF long_float,	/*nb_float*/
-	UF long_oct,	/*nb_oct*/
-	UF long_hex,	/*nb_hex*/
+	(binaryfunc)	long_add,	/*nb_add*/
+	(binaryfunc)	long_sub,	/*nb_subtract*/
+	(binaryfunc)	long_mul,	/*nb_multiply*/
+	(binaryfunc)	long_div,	/*nb_divide*/
+	(binaryfunc)	long_mod,	/*nb_remainder*/
+	(binaryfunc)	long_divmod,	/*nb_divmod*/
+	(ternaryfunc)	long_pow,	/*nb_power*/
+	(unaryfunc) 	long_neg,	/*nb_negative*/
+	(unaryfunc) 	long_pos,	/*tp_positive*/
+	(unaryfunc) 	long_abs,	/*tp_absolute*/
+	(inquiry)	long_nonzero,	/*tp_nonzero*/
+	(unaryfunc)	long_invert,	/*nb_invert*/
+	(binaryfunc)	long_lshift,	/*nb_lshift*/
+	(binaryfunc)	long_rshift,	/*nb_rshift*/
+	(binaryfunc)	long_and,	/*nb_and*/
+	(binaryfunc)	long_xor,	/*nb_xor*/
+	(binaryfunc)	long_or,	/*nb_or*/
+	(coercion)	long_coerce,	/*nb_coerce*/
+	(unaryfunc)	long_int,	/*nb_int*/
+	(unaryfunc)	long_long,	/*nb_long*/
+	(unaryfunc)	long_float,	/*nb_float*/
+	(unaryfunc)	long_oct,	/*nb_oct*/
+	(unaryfunc)	long_hex,	/*nb_hex*/
+	0,				/*nb_inplace_add*/
+	0,				/*nb_inplace_subtract*/
+	0,				/*nb_inplace_multiply*/
+	0,				/*nb_inplace_divide*/
+	0,				/*nb_inplace_remainder*/
+	0,				/*nb_inplace_power*/
+	0,				/*nb_inplace_lshift*/
+	0,				/*nb_inplace_rshift*/
+	0,				/*nb_inplace_and*/
+	0,				/*nb_inplace_xor*/
+	0,				/*nb_inplace_or*/
 };
 
 PyTypeObject PyLong_Type = {
@@ -1786,16 +1855,20 @@ PyTypeObject PyLong_Type = {
 	"long int",
 	sizeof(PyLongObject) - sizeof(digit),
 	sizeof(digit),
-	(destructor)long_dealloc, /*tp_dealloc*/
-	0,		/*tp_print*/
-	0,		/*tp_getattr*/
-	0,		/*tp_setattr*/
-	(int (*) Py_FPROTO((PyObject *, PyObject *)))
-	(cmpfunc)long_compare, /*tp_compare*/
-	(reprfunc)long_repr, /*tp_repr*/
-	&long_as_number,/*tp_as_number*/
-	0,		/*tp_as_sequence*/
-	0,		/*tp_as_mapping*/
-	(long (*) Py_FPROTO((PyObject *)))
-	(hashfunc)long_hash, /*tp_hash*/
+	(destructor)long_dealloc,	/*tp_dealloc*/
+	0,				/*tp_print*/
+	0,				/*tp_getattr*/
+	0,				/*tp_setattr*/
+	(cmpfunc)long_compare,		/*tp_compare*/
+	(reprfunc)long_repr,		/*tp_repr*/
+	&long_as_number,		/*tp_as_number*/
+	0,				/*tp_as_sequence*/
+	0,				/*tp_as_mapping*/
+	(hashfunc)long_hash,		/*tp_hash*/
+        0,              		/*tp_call*/
+        (reprfunc)long_str,		/*tp_str*/
+	0,				/*tp_getattro*/
+	0,				/*tp_setattro*/
+	0,				/*tp_as_buffer*/
+	Py_TPFLAGS_CHECKTYPES		/*tp_flags*/
 };
