@@ -1,33 +1,3 @@
-/***********************************************************
-Copyright 1991-1995 by Stichting Mathematisch Centrum, Amsterdam,
-The Netherlands.
-
-                        All Rights Reserved
-
-Permission to use, copy, modify, and distribute this software and its
-documentation for any purpose and without fee is hereby granted,
-provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in
-supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI or Corporation for National Research Initiatives or
-CNRI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior
-permission.
-
-While CWI is the initial source for this software, a modified version
-is made available by the Corporation for National Research Initiatives
-(CNRI) at the Internet address ftp://ftp.python.org.
-
-STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
-REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
-MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
-CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
-DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
-PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
-TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-PERFORMANCE OF THIS SOFTWARE.
-
-******************************************************************/
 
 /* Grammar implementation */
 
@@ -39,17 +9,16 @@ PERFORMANCE OF THIS SOFTWARE.
 #include "token.h"
 #include "grammar.h"
 
-extern int debugging;
+extern int Py_DebugFlag;
 
 grammar *
-newgrammar(start)
-	int start;
+newgrammar(int start)
 {
 	grammar *g;
 	
-	g = NEW(grammar, 1);
+	g = PyMem_NEW(grammar, 1);
 	if (g == NULL)
-		fatal("no mem for new grammar");
+		Py_FatalError("no mem for new grammar");
 	g->g_ndfas = 0;
 	g->g_dfa = NULL;
 	g->g_start = start;
@@ -60,16 +29,13 @@ newgrammar(start)
 }
 
 dfa *
-adddfa(g, type, name)
-	grammar *g;
-	int type;
-	char *name;
+adddfa(grammar *g, int type, char *name)
 {
 	dfa *d;
 	
-	RESIZE(g->g_dfa, dfa, g->g_ndfas + 1);
+	PyMem_RESIZE(g->g_dfa, dfa, g->g_ndfas + 1);
 	if (g->g_dfa == NULL)
-		fatal("no mem to resize dfa in adddfa");
+		Py_FatalError("no mem to resize dfa in adddfa");
 	d = &g->g_dfa[g->g_ndfas++];
 	d->d_type = type;
 	d->d_name = name;
@@ -81,14 +47,13 @@ adddfa(g, type, name)
 }
 
 int
-addstate(d)
-	dfa *d;
+addstate(dfa *d)
 {
 	state *s;
 	
-	RESIZE(d->d_state, state, d->d_nstates + 1);
+	PyMem_RESIZE(d->d_state, state, d->d_nstates + 1);
 	if (d->d_state == NULL)
-		fatal("no mem to resize state in addstate");
+		Py_FatalError("no mem to resize state in addstate");
 	s = &d->d_state[d->d_nstates++];
 	s->s_narcs = 0;
 	s->s_arc = NULL;
@@ -100,9 +65,7 @@ addstate(d)
 }
 
 void
-addarc(d, from, to, lbl)
-	dfa *d;
-	int lbl;
+addarc(dfa *d, int from, int to, int lbl)
 {
 	state *s;
 	arc *a;
@@ -111,19 +74,16 @@ addarc(d, from, to, lbl)
 	assert(0 <= to && to < d->d_nstates);
 	
 	s = &d->d_state[from];
-	RESIZE(s->s_arc, arc, s->s_narcs + 1);
+	PyMem_RESIZE(s->s_arc, arc, s->s_narcs + 1);
 	if (s->s_arc == NULL)
-		fatal("no mem to resize arc list in addarc");
+		Py_FatalError("no mem to resize arc list in addarc");
 	a = &s->s_arc[s->s_narcs++];
 	a->a_lbl = lbl;
 	a->a_arrow = to;
 }
 
 int
-addlabel(ll, type, str)
-	labellist *ll;
-	int type;
-	char *str;
+addlabel(labellist *ll, int type, char *str)
 {
 	int i;
 	label *lb;
@@ -133,9 +93,9 @@ addlabel(ll, type, str)
 			strcmp(ll->ll_label[i].lb_str, str) == 0)
 			return i;
 	}
-	RESIZE(ll->ll_label, label, ll->ll_nlabels + 1);
+	PyMem_RESIZE(ll->ll_label, label, ll->ll_nlabels + 1);
 	if (ll->ll_label == NULL)
-		fatal("no mem to resize labellist in addlabel");
+		Py_FatalError("no mem to resize labellist in addlabel");
 	lb = &ll->ll_label[ll->ll_nlabels++];
 	lb->lb_type = type;
 	lb->lb_str = str; /* XXX strdup(str) ??? */
@@ -145,10 +105,7 @@ addlabel(ll, type, str)
 /* Same, but rather dies than adds */
 
 int
-findlabel(ll, type, str)
-	labellist *ll;
-	int type;
-	char *str;
+findlabel(labellist *ll, int type, char *str)
 {
 	int i;
 	
@@ -158,20 +115,19 @@ findlabel(ll, type, str)
 			return i;
 	}
 	fprintf(stderr, "Label %d/'%s' not found\n", type, str);
-	fatal("grammar.c:findlabel()");
-	/*NOTREACHED*/
+	Py_FatalError("grammar.c:findlabel()");
+	return 0; /* Make gcc -Wall happy */
 }
 
 /* Forward */
-static void translabel PROTO((grammar *, label *));
+static void translabel(grammar *, label *);
 
 void
-translatelabels(g)
-	grammar *g;
+translatelabels(grammar *g)
 {
 	int i;
 
-#ifdef DEBUG
+#ifdef Py_DEBUG
 	printf("Translating labels ...\n");
 #endif
 	/* Don't translate EMPTY */
@@ -180,30 +136,29 @@ translatelabels(g)
 }
 
 static void
-translabel(g, lb)
-	grammar *g;
-	label *lb;
+translabel(grammar *g, label *lb)
 {
 	int i;
 	
-	if (debugging)
-		printf("Translating label %s ...\n", labelrepr(lb));
+	if (Py_DebugFlag)
+		printf("Translating label %s ...\n", PyGrammar_LabelRepr(lb));
 	
 	if (lb->lb_type == NAME) {
 		for (i = 0; i < g->g_ndfas; i++) {
 			if (strcmp(lb->lb_str, g->g_dfa[i].d_name) == 0) {
-				if (debugging)
-					printf("Label %s is non-terminal %d.\n",
-						lb->lb_str,
-						g->g_dfa[i].d_type);
+				if (Py_DebugFlag)
+					printf(
+					    "Label %s is non-terminal %d.\n",
+					    lb->lb_str,
+					    g->g_dfa[i].d_type);
 				lb->lb_type = g->g_dfa[i].d_type;
 				lb->lb_str = NULL;
 				return;
 			}
 		}
 		for (i = 0; i < (int)N_TOKENS; i++) {
-			if (strcmp(lb->lb_str, tok_name[i]) == 0) {
-				if (debugging)
+			if (strcmp(lb->lb_str, _PyParser_TokenNames[i]) == 0) {
+				if (Py_DebugFlag)
 					printf("Label %s is terminal %d.\n",
 						lb->lb_str, i);
 				lb->lb_type = i;
@@ -216,9 +171,9 @@ translabel(g, lb)
 	}
 	
 	if (lb->lb_type == STRING) {
-		if (isalpha(lb->lb_str[1])) {
+		if (isalpha((int)(lb->lb_str[1])) || lb->lb_str[1] == '_') {
 			char *p;
-			if (debugging)
+			if (Py_DebugFlag)
 				printf("Label %s is a keyword\n", lb->lb_str);
 			lb->lb_type = NAME;
 			lb->lb_str++;
@@ -227,7 +182,7 @@ translabel(g, lb)
 				*p = '\0';
 		}
 		else if (lb->lb_str[2] == lb->lb_str[0]) {
-			int type = (int) tok_1char(lb->lb_str[1]);
+			int type = (int) PyToken_OneChar(lb->lb_str[1]);
 			if (type != OP) {
 				lb->lb_type = type;
 				lb->lb_str = NULL;
@@ -237,7 +192,7 @@ translabel(g, lb)
 					lb->lb_str);
 		}
 		else if (lb->lb_str[2] && lb->lb_str[3] == lb->lb_str[0]) {
-			int type = (int) tok_2char(lb->lb_str[1],
+			int type = (int) PyToken_TwoChars(lb->lb_str[1],
 						   lb->lb_str[2]);
 			if (type != OP) {
 				lb->lb_type = type;
@@ -247,10 +202,23 @@ translabel(g, lb)
 				printf("Unknown OP label %s\n",
 					lb->lb_str);
 		}
+		else if (lb->lb_str[2] && lb->lb_str[3] && lb->lb_str[4] == lb->lb_str[0]) {
+			int type = (int) PyToken_ThreeChars(lb->lb_str[1],
+							    lb->lb_str[2],
+							    lb->lb_str[3]);
+			if (type != OP) {
+				lb->lb_type = type;
+				lb->lb_str = NULL;
+			}
+			else
+				printf("Unknown OP label %s\n",
+					lb->lb_str);
+		}
 		else
 			printf("Can't translate STRING label %s\n",
 				lb->lb_str);
 	}
 	else
-		printf("Can't translate label '%s'\n", labelrepr(lb));
+		printf("Can't translate label '%s'\n",
+		       PyGrammar_LabelRepr(lb));
 }
