@@ -1,33 +1,3 @@
-/***********************************************************
-Copyright 1991-1995 by Stichting Mathematisch Centrum, Amsterdam,
-The Netherlands.
-
-                        All Rights Reserved
-
-Permission to use, copy, modify, and distribute this software and its
-documentation for any purpose and without fee is hereby granted,
-provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in
-supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI or Corporation for National Research Initiatives or
-CNRI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior
-permission.
-
-While CWI is the initial source for this software, a modified version
-is made available by the Corporation for National Research Initiatives
-(CNRI) at the Internet address ftp://ftp.python.org.
-
-STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
-REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
-MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
-CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
-DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
-PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
-TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-PERFORMANCE OF THIS SOFTWARE.
-
-******************************************************************/
 
 /* Parser generator */
 /* XXX This file is not yet fully PROTOized */
@@ -42,7 +12,7 @@ PERFORMANCE OF THIS SOFTWARE.
 #include "metagrammar.h"
 #include "pgen.h"
 
-extern int debugging;
+extern int Py_DebugFlag;
 
 
 /* PART ONE -- CONSTRUCT NFA -- Cf. Algorithm 3.2 from [Aho&Ullman 77] */
@@ -66,20 +36,23 @@ typedef struct _nfa {
 } nfa;
 
 /* Forward */
-static compile_rhs PROTO((labellist *ll, nfa *nf, node *n, int *pa, int *pb));
-static compile_alt PROTO((labellist *ll, nfa *nf, node *n, int *pa, int *pb));
-static compile_item PROTO((labellist *ll, nfa *nf, node *n, int *pa, int *pb));
-static compile_atom PROTO((labellist *ll, nfa *nf, node *n, int *pa, int *pb));
+static void compile_rhs(labellist *ll,
+			nfa *nf, node *n, int *pa, int *pb);
+static void compile_alt(labellist *ll,
+			nfa *nf, node *n, int *pa, int *pb);
+static void compile_item(labellist *ll,
+			 nfa *nf, node *n, int *pa, int *pb);
+static void compile_atom(labellist *ll,
+			 nfa *nf, node *n, int *pa, int *pb);
 
 static int
-addnfastate(nf)
-	nfa *nf;
+addnfastate(nfa *nf)
 {
 	nfastate *st;
 	
-	RESIZE(nf->nf_state, nfastate, nf->nf_nstates + 1);
+	PyMem_RESIZE(nf->nf_state, nfastate, nf->nf_nstates + 1);
 	if (nf->nf_state == NULL)
-		fatal("out of mem");
+		Py_FatalError("out of mem");
 	st = &nf->nf_state[nf->nf_nstates++];
 	st->st_narcs = 0;
 	st->st_arc = NULL;
@@ -87,32 +60,29 @@ addnfastate(nf)
 }
 
 static void
-addnfaarc(nf, from, to, lbl)
-	nfa *nf;
-	int from, to, lbl;
+addnfaarc(nfa *nf, int from, int to, int lbl)
 {
 	nfastate *st;
 	nfaarc *ar;
 	
 	st = &nf->nf_state[from];
-	RESIZE(st->st_arc, nfaarc, st->st_narcs + 1);
+	PyMem_RESIZE(st->st_arc, nfaarc, st->st_narcs + 1);
 	if (st->st_arc == NULL)
-		fatal("out of mem");
+		Py_FatalError("out of mem");
 	ar = &st->st_arc[st->st_narcs++];
 	ar->ar_label = lbl;
 	ar->ar_arrow = to;
 }
 
 static nfa *
-newnfa(name)
-	char *name;
+newnfa(char *name)
 {
 	nfa *nf;
-	static type = NT_OFFSET; /* All types will be disjunct */
+	static int type = NT_OFFSET; /* All types will be disjunct */
 	
-	nf = NEW(nfa, 1);
+	nf = PyMem_NEW(nfa, 1);
 	if (nf == NULL)
-		fatal("no mem for new nfa");
+		Py_FatalError("no mem for new nfa");
 	nf->nf_type = type++;
 	nf->nf_name = name; /* XXX strdup(name) ??? */
 	nf->nf_nstates = 0;
@@ -128,16 +98,16 @@ typedef struct _nfagrammar {
 } nfagrammar;
 
 /* Forward */
-static compile_rule PROTO((nfagrammar *gr, node *n));
+static void compile_rule(nfagrammar *gr, node *n);
 
 static nfagrammar *
-newnfagrammar()
+newnfagrammar(void)
 {
 	nfagrammar *gr;
 	
-	gr = NEW(nfagrammar, 1);
+	gr = PyMem_NEW(nfagrammar, 1);
 	if (gr == NULL)
-		fatal("no mem for new nfa grammar");
+		Py_FatalError("no mem for new nfa grammar");
 	gr->gr_nnfas = 0;
 	gr->gr_nfa = NULL;
 	gr->gr_ll.ll_nlabels = 0;
@@ -147,29 +117,27 @@ newnfagrammar()
 }
 
 static nfa *
-addnfa(gr, name)
-	nfagrammar *gr;
-	char *name;
+addnfa(nfagrammar *gr, char *name)
 {
 	nfa *nf;
 	
 	nf = newnfa(name);
-	RESIZE(gr->gr_nfa, nfa *, gr->gr_nnfas + 1);
+	PyMem_RESIZE(gr->gr_nfa, nfa *, gr->gr_nnfas + 1);
 	if (gr->gr_nfa == NULL)
-		fatal("out of mem");
+		Py_FatalError("out of mem");
 	gr->gr_nfa[gr->gr_nnfas++] = nf;
 	addlabel(&gr->gr_ll, NAME, nf->nf_name);
 	return nf;
 }
 
-#ifdef DEBUG
+#ifdef Py_DEBUG
 
 static char REQNFMT[] = "metacompile: less than %d children\n";
 
 #define REQN(i, count) \
  	if (i < count) { \
 		fprintf(stderr, REQNFMT, count); \
-		fatal("REQN"); \
+		Py_FatalError("REQN"); \
 	} else
 
 #else
@@ -177,8 +145,7 @@ static char REQNFMT[] = "metacompile: less than %d children\n";
 #endif
 
 static nfagrammar *
-metacompile(n)
-	node *n;
+metacompile(node *n)
 {
 	nfagrammar *gr;
 	int i;
@@ -195,10 +162,8 @@ metacompile(n)
 	return gr;
 }
 
-static
-compile_rule(gr, n)
-	nfagrammar *gr;
-	node *n;
+static void
+compile_rule(nfagrammar *gr, node *n)
 {
 	nfa *nf;
 	
@@ -216,12 +181,8 @@ compile_rule(gr, n)
 	REQ(n, NEWLINE);
 }
 
-static
-compile_rhs(ll, nf, n, pa, pb)
-	labellist *ll;
-	nfa *nf;
-	node *n;
-	int *pa, *pb;
+static void
+compile_rhs(labellist *ll, nfa *nf, node *n, int *pa, int *pb)
 {
 	int i;
 	int a, b;
@@ -253,12 +214,8 @@ compile_rhs(ll, nf, n, pa, pb)
 	}
 }
 
-static
-compile_alt(ll, nf, n, pa, pb)
-	labellist *ll;
-	nfa *nf;
-	node *n;
-	int *pa, *pb;
+static void
+compile_alt(labellist *ll, nfa *nf, node *n, int *pa, int *pb)
 {
 	int i;
 	int a, b;
@@ -284,12 +241,8 @@ compile_alt(ll, nf, n, pa, pb)
 	}
 }
 
-static
-compile_item(ll, nf, n, pa, pb)
-	labellist *ll;
-	nfa *nf;
-	node *n;
-	int *pa, *pb;
+static void
+compile_item(labellist *ll, nfa *nf, node *n, int *pa, int *pb)
 {
 	int i;
 	int a, b;
@@ -325,12 +278,8 @@ compile_item(ll, nf, n, pa, pb)
 	}
 }
 
-static
-compile_atom(ll, nf, n, pa, pb)
-	labellist *ll;
-	nfa *nf;
-	node *n;
-	int *pa, *pb;
+static void
+compile_atom(labellist *ll, nfa *nf, node *n, int *pa, int *pb)
 {
 	int i;
 	
@@ -356,10 +305,7 @@ compile_atom(ll, nf, n, pa, pb)
 }
 
 static void
-dumpstate(ll, nf, istate)
-	labellist *ll;
-	nfa *nf;
-	int istate;
+dumpstate(labellist *ll, nfa *nf, int istate)
 {
 	nfastate *st;
 	int i;
@@ -375,16 +321,14 @@ dumpstate(ll, nf, istate)
 		if (i > 0)
 			printf("\n    ");
 		printf("-> %2d  %s", ar->ar_arrow,
-			labelrepr(&ll->ll_label[ar->ar_label]));
+			PyGrammar_LabelRepr(&ll->ll_label[ar->ar_label]));
 		ar++;
 	}
 	printf("\n");
 }
 
 static void
-dumpnfa(ll, nf)
-	labellist *ll;
-	nfa *nf;
+dumpnfa(labellist *ll, nfa *nf)
 {
 	int i;
 	
@@ -398,10 +342,7 @@ dumpnfa(ll, nf)
 /* PART TWO -- CONSTRUCT DFA -- Algorithm 3.1 from [Aho&Ullman 77] */
 
 static void
-addclosure(ss, nf, istate)
-	bitset ss;
-	nfa *nf;
-	int istate;
+addclosure(bitset ss, nfa *nf, int istate)
 {
 	if (addbit(ss, istate)) {
 		nfastate *st = &nf->nf_state[istate];
@@ -437,16 +378,13 @@ typedef struct _ss_dfa {
 } ss_dfa;
 
 /* Forward */
-static printssdfa PROTO((int xx_nstates, ss_state *xx_state, int nbits,
-			 labellist *ll, char *msg));
-static simplify PROTO((int xx_nstates, ss_state *xx_state));
-static convert PROTO((dfa *d, int xx_nstates, ss_state *xx_state));
+static void printssdfa(int xx_nstates, ss_state *xx_state, int nbits,
+		       labellist *ll, char *msg);
+static void simplify(int xx_nstates, ss_state *xx_state);
+static void convert(dfa *d, int xx_nstates, ss_state *xx_state);
 
-static
-makedfa(gr, nf, d)
-	nfagrammar *gr;
-	nfa *nf;
-	dfa *d;
+static void
+makedfa(nfagrammar *gr, nfa *nf, dfa *d)
 {
 	int nbits = nf->nf_nstates;
 	bitset ss;
@@ -459,9 +397,9 @@ makedfa(gr, nf, d)
 	
 	ss = newbitset(nbits);
 	addclosure(ss, nf, nf->nf_start);
-	xx_state = NEW(ss_state, 1);
+	xx_state = PyMem_NEW(ss_state, 1);
 	if (xx_state == NULL)
-		fatal("no mem for xx_state in makedfa");
+		Py_FatalError("no mem for xx_state in makedfa");
 	xx_nstates = 1;
 	yy = &xx_state[0];
 	yy->ss_ss = ss;
@@ -497,9 +435,10 @@ makedfa(gr, nf, d)
 						goto found;
 				}
 				/* Add new arc for this state */
-				RESIZE(yy->ss_arc, ss_arc, yy->ss_narcs + 1);
+				PyMem_RESIZE(yy->ss_arc, ss_arc,
+					     yy->ss_narcs + 1);
 				if (yy->ss_arc == NULL)
-					fatal("out of mem");
+					Py_FatalError("out of mem");
 				zz = &yy->ss_arc[yy->ss_narcs++];
 				zz->sa_label = ar->ar_label;
 				zz->sa_bitset = newbitset(nbits);
@@ -519,9 +458,9 @@ makedfa(gr, nf, d)
 					goto done;
 				}
 			}
-			RESIZE(xx_state, ss_state, xx_nstates + 1);
+			PyMem_RESIZE(xx_state, ss_state, xx_nstates + 1);
 			if (xx_state == NULL)
-				fatal("out of mem");
+				Py_FatalError("out of mem");
 			zz->sa_arrow = xx_nstates;
 			yy = &xx_state[xx_nstates++];
 			yy->ss_ss = zz->sa_bitset;
@@ -533,13 +472,13 @@ makedfa(gr, nf, d)
 		}
 	}
 	
-	if (debugging)
+	if (Py_DebugFlag)
 		printssdfa(xx_nstates, xx_state, nbits, &gr->gr_ll,
 						"before minimizing");
 	
 	simplify(xx_nstates, xx_state);
 	
-	if (debugging)
+	if (Py_DebugFlag)
 		printssdfa(xx_nstates, xx_state, nbits, &gr->gr_ll,
 						"after minimizing");
 	
@@ -548,13 +487,9 @@ makedfa(gr, nf, d)
 	/* XXX cleanup */
 }
 
-static
-printssdfa(xx_nstates, xx_state, nbits, ll, msg)
-	int xx_nstates;
-	ss_state *xx_state;
-	int nbits;
-	labellist *ll;
-	char *msg;
+static void
+printssdfa(int xx_nstates, ss_state *xx_state, int nbits,
+	   labellist *ll, char *msg)
 {
 	int i, ibit, iarc;
 	ss_state *yy;
@@ -578,7 +513,8 @@ printssdfa(xx_nstates, xx_state, nbits, ll, msg)
 			zz = &yy->ss_arc[iarc];
 			printf("  Arc to state %d, label %s\n",
 				zz->sa_arrow,
-				labelrepr(&ll->ll_label[zz->sa_label]));
+				PyGrammar_LabelRepr(
+					&ll->ll_label[zz->sa_label]));
 		}
 	}
 }
@@ -590,12 +526,11 @@ printssdfa(xx_nstates, xx_state, nbits, ll, msg)
    equivalent to another oner.  This is NOT Algorithm 3.3 from
    [Aho&Ullman 77].  It does not always finds the minimal DFA,
    but it does usually make a much smaller one...  (For an example
-   of sub-optimal behaviour, try S: x a b+ | y a b+.)
+   of sub-optimal behavior, try S: x a b+ | y a b+.)
 */
 
 static int
-samestate(s1, s2)
-	ss_state *s1, *s2;
+samestate(ss_state *s1, ss_state *s2)
 {
 	int i;
 	
@@ -610,14 +545,11 @@ samestate(s1, s2)
 }
 
 static void
-renamestates(xx_nstates, xx_state, from, to)
-	int xx_nstates;
-	ss_state *xx_state;
-	int from, to;
+renamestates(int xx_nstates, ss_state *xx_state, int from, int to)
 {
 	int i, j;
 	
-	if (debugging)
+	if (Py_DebugFlag)
 		printf("Rename state %d to %d.\n", from, to);
 	for (i = 0; i < xx_nstates; i++) {
 		if (xx_state[i].ss_deleted)
@@ -629,10 +561,8 @@ renamestates(xx_nstates, xx_state, from, to)
 	}
 }
 
-static
-simplify(xx_nstates, xx_state)
-	int xx_nstates;
-	ss_state *xx_state;
+static void
+simplify(int xx_nstates, ss_state *xx_state)
 {
 	int changes;
 	int i, j;
@@ -647,7 +577,8 @@ simplify(xx_nstates, xx_state)
 					continue;
 				if (samestate(&xx_state[i], &xx_state[j])) {
 					xx_state[i].ss_deleted++;
-					renamestates(xx_nstates, xx_state, i, j);
+					renamestates(xx_nstates, xx_state,
+						     i, j);
 					changes++;
 					break;
 				}
@@ -661,11 +592,8 @@ simplify(xx_nstates, xx_state)
 
 /* Convert the DFA into a grammar that can be used by our parser */
 
-static
-convert(d, xx_nstates, xx_state)
-	dfa *d;
-	int xx_nstates;
-	ss_state *xx_state;
+static void
+convert(dfa *d, int xx_nstates, ss_state *xx_state)
 {
 	int i, j;
 	ss_state *yy;
@@ -699,8 +627,7 @@ convert(d, xx_nstates, xx_state)
 /* PART FIVE -- GLUE IT ALL TOGETHER */
 
 static grammar *
-maketables(gr)
-	nfagrammar *gr;
+maketables(nfagrammar *gr)
 {
 	int i;
 	nfa *nf;
@@ -715,7 +642,7 @@ maketables(gr)
 	
 	for (i = 0; i < gr->gr_nnfas; i++) {
 		nf = gr->gr_nfa[i];
-		if (debugging) {
+		if (Py_DebugFlag) {
 			printf("Dump of NFA for '%s' ...\n", nf->nf_name);
 			dumpnfa(&gr->gr_ll, nf);
 		}
@@ -728,8 +655,7 @@ maketables(gr)
 }
 
 grammar *
-pgen(n)
-	node *n;
+pgen(node *n)
 {
 	nfagrammar *gr;
 	grammar *g;
