@@ -1,33 +1,3 @@
-/***********************************************************
-Copyright 1991-1995 by Stichting Mathematisch Centrum, Amsterdam,
-The Netherlands.
-
-                        All Rights Reserved
-
-Permission to use, copy, modify, and distribute this software and its
-documentation for any purpose and without fee is hereby granted,
-provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in
-supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI or Corporation for National Research Initiatives or
-CNRI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior
-permission.
-
-While CWI is the initial source for this software, a modified version
-is made available by the Corporation for National Research Initiatives
-(CNRI) at the Internet address ftp://ftp.python.org.
-
-STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
-REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
-MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
-CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
-DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
-PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
-TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-PERFORMANCE OF THIS SOFTWARE.
-
-******************************************************************/
 
 /* Parser generator main program */
 
@@ -49,63 +19,66 @@ PERFORMANCE OF THIS SOFTWARE.
 #include "parsetok.h"
 #include "pgen.h"
 
-int debugging;
+int Py_DebugFlag;
+int Py_VerboseFlag;
 
 /* Forward */
-grammar *getgrammar PROTO((char *filename));
+grammar *getgrammar(char *filename);
 #ifdef THINK_C
-int main PROTO((int, char **));
-char *askfile PROTO((void));
+int main(int, char **);
+char *askfile(void);
 #endif
 
 void
-goaway(sts)
-	int sts;
+Py_Exit(int sts)
 {
 	exit(sts);
 }
 
 int
-main(argc, argv)
-	int argc;
-	char **argv;
+main(int argc, char **argv)
 {
 	grammar *g;
 	FILE *fp;
-	char *filename;
+	char *filename, *graminit_h, *graminit_c;
 	
 #ifdef THINK_C
 	filename = askfile();
+	graminit_h = askfile();
+	graminit_c = askfile();
 #else
-	if (argc != 2) {
-		fprintf(stderr, "usage: %s grammar\n", argv[0]);
-		goaway(2);
+	if (argc != 4) {
+		fprintf(stderr,
+			"usage: %s grammar graminit.h graminit.c\n", argv[0]);
+		Py_Exit(2);
 	}
 	filename = argv[1];
+	graminit_h = argv[2];
+	graminit_c = argv[3];
 #endif
 	g = getgrammar(filename);
-	fp = fopen("graminit.c", "w");
+	fp = fopen(graminit_c, "w");
 	if (fp == NULL) {
-		perror("graminit.c");
-		goaway(1);
+		perror(graminit_c);
+		Py_Exit(1);
 	}
-	printf("Writing graminit.c ...\n");
+	printf("Writing %s ...\n", graminit_c);
 	printgrammar(g, fp);
 	fclose(fp);
-	fp = fopen("graminit.h", "w");
+	fp = fopen(graminit_h, "w");
 	if (fp == NULL) {
-		perror("graminit.h");
-		goaway(1);
+		perror(graminit_h);
+		Py_Exit(1);
 	}
-	printf("Writing graminit.h ...\n");
+	printf("Writing %s ...\n", graminit_h);
 	printnonterminals(g, fp);
 	fclose(fp);
-	goaway(0);
+	Py_Exit(0);
+	return 0; /* Make gcc -Wall happy */
 }
 
 grammar *
-getgrammar(filename)
-	char *filename;
+getgrammar(char *filename)
 {
 	FILE *fp;
 	node *n;
@@ -115,17 +88,17 @@ getgrammar(filename)
 	fp = fopen(filename, "r");
 	if (fp == NULL) {
 		perror(filename);
-		goaway(1);
+		Py_Exit(1);
 	}
 	g0 = meta_grammar();
-	n = parsefile(fp, filename, g0, g0->g_start,
+	n = PyParser_ParseFile(fp, filename, g0, g0->g_start,
 		      (char *)NULL, (char *)NULL, &err);
 	fclose(fp);
 	if (n == NULL) {
 		fprintf(stderr, "Parsing error %d, line %d.\n",
 			err.error, err.lineno);
 		if (err.text != NULL) {
-			int i;
+			size_t i;
 			fprintf(stderr, "%s", err.text);
 			i = strlen(err.text);
 			if (i == 0 || err.text[i-1] != '\n')
@@ -137,51 +110,49 @@ getgrammar(filename)
 					putc(' ', stderr);
 			}
 			fprintf(stderr, "^\n");
-			free(err.text);
+			PyMem_DEL(err.text);
 		}
-		goaway(1);
+		Py_Exit(1);
 	}
 	g = pgen(n);
 	if (g == NULL) {
 		printf("Bad grammar.\n");
-		goaway(1);
+		Py_Exit(1);
 	}
 	return g;
 }
 
 #ifdef THINK_C
 char *
-askfile()
+askfile(void)
 {
 	char buf[256];
 	static char name[256];
 	printf("Input file name: ");
 	if (fgets(buf, sizeof buf, stdin) == NULL) {
 		printf("EOF\n");
-		goaway(1);
+		Py_Exit(1);
 	}
 	/* XXX The (unsigned char *) case is needed by THINK C 3.0 */
 	if (sscanf(/*(unsigned char *)*/buf, " %s ", name) != 1) {
 		printf("No file\n");
-		goaway(1);
+		Py_Exit(1);
 	}
 	return name;
 }
 #endif
 
 void
-fatal(msg)
-	char *msg;
+Py_FatalError(char *msg)
 {
 	fprintf(stderr, "pgen: FATAL ERROR: %s\n", msg);
-	goaway(1);
+	Py_Exit(1);
 }
 
 #ifdef macintosh
 /* ARGSUSED */
 int
-guesstabsize(path)
-	char *path;
+guesstabsize(char *path)
 {
 	return 4;
 }
@@ -190,11 +161,10 @@ guesstabsize(path)
 /* No-nonsense my_readline() for tokenizer.c */
 
 char *
-my_readline(prompt)
-	char *prompt;
+PyOS_Readline(char *prompt)
 {
-	int n = 1000;
-	char *p = malloc(n);
+	size_t n = 1000;
+	char *p = PyMem_MALLOC(n);
 	char *q;
 	if (p == NULL)
 		return NULL;
@@ -207,5 +177,17 @@ my_readline(prompt)
 	n = strlen(p);
 	if (n > 0 && p[n-1] != '\n')
 		p[n-1] = '\n';
-	return realloc(p, n+1);
+	return PyMem_REALLOC(p, n+1);
+}
+
+#include <stdarg.h>
+
+void
+PySys_WriteStderr(const char *format, ...)
+{
+	va_list va;
+
+	va_start(va, format);
+	vfprintf(stderr, format, va);
+	va_end(va);
 }
