@@ -1,33 +1,3 @@
-/***********************************************************
-Copyright 1991-1995 by Stichting Mathematisch Centrum, Amsterdam,
-The Netherlands.
-
-                        All Rights Reserved
-
-Permission to use, copy, modify, and distribute this software and its
-documentation for any purpose and without fee is hereby granted,
-provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in
-supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI or Corporation for National Research Initiatives or
-CNRI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior
-permission.
-
-While CWI is the initial source for this software, a modified version
-is made available by the Corporation for National Research Initiatives
-(CNRI) at the Internet address ftp://ftp.python.org.
-
-STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
-REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
-MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
-CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
-DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
-PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
-TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-PERFORMANCE OF THIS SOFTWARE.
-
-******************************************************************/
 
 /* Tokenizer implementation */
 
@@ -38,7 +8,7 @@ PERFORMANCE OF THIS SOFTWARE.
 #include "tokenizer.h"
 #include "errcode.h"
 
-extern char *my_readline PROTO((char *));
+extern char *PyOS_Readline(char *);
 /* Return malloc'ed string including trailing \n;
    empty malloc'ed string for EOF;
    NULL if interrupted */
@@ -46,14 +16,22 @@ extern char *my_readline PROTO((char *));
 /* Don't ever change this -- it would break the portability of Python code */
 #define TABSIZE 8
 
+/* Convert a possibly signed character to a nonnegative int */
+/* XXX This assumes characters are 8 bits wide */
+#ifdef __CHAR_UNSIGNED__
+#define Py_CHARMASK(c)		(c)
+#else
+#define Py_CHARMASK(c)		((c) & 0xff)
+#endif
+
 /* Forward */
-static struct tok_state *tok_new PROTO((void));
-static int tok_nextc PROTO((struct tok_state *tok));
-static void tok_backup PROTO((struct tok_state *tok, int c));
+static struct tok_state *tok_new(void);
+static int tok_nextc(struct tok_state *tok);
+static void tok_backup(struct tok_state *tok, int c);
 
 /* Token names */
 
-char *tok_name[] = {
+char *_PyParser_TokenNames[] = {
 	"ENDMARKER",
 	"NAME",
 	"NUMBER",
@@ -91,6 +69,17 @@ char *tok_name[] = {
 	"LEFTSHIFT",
 	"RIGHTSHIFT",
 	"DOUBLESTAR",
+	"PLUSEQUAL",
+	"MINEQUAL",
+	"STAREQUAL",
+	"SLASHEQUAL",
+	"PERCENTEQUAL",
+	"AMPEREQUAL",
+	"VBAREQUAL",
+	"CIRCUMFLEXEQUAL",
+	"LEFTSHIFTEQUAL",
+	"RIGHTSHIFTEQUAL",
+	"DOUBLESTAREQUAL",
 	/* This table must match the #defines in token.h! */
 	"OP",
 	"<ERRORTOKEN>",
@@ -101,9 +90,9 @@ char *tok_name[] = {
 /* Create and initialize a new tok_state structure */
 
 static struct tok_state *
-tok_new()
+tok_new(void)
 {
-	struct tok_state *tok = NEW(struct tok_state, 1);
+	struct tok_state *tok = PyMem_NEW(struct tok_state, 1);
 	if (tok == NULL)
 		return NULL;
 	tok->buf = tok->cur = tok->end = tok->inp = tok->start = NULL;
@@ -117,6 +106,11 @@ tok_new()
 	tok->prompt = tok->nextprompt = NULL;
 	tok->lineno = 0;
 	tok->level = 0;
+	tok->filename = NULL;
+	tok->altwarning = 0;
+	tok->alterror = 0;
+	tok->alttabsize = 1;
+	tok->altindstack[0] = 0;
 	return tok;
 }
 
@@ -124,8 +118,7 @@ tok_new()
 /* Set up tokenizer for string */
 
 struct tok_state *
-tok_setups(str)
-	char *str;
+PyTokenizer_FromString(char *str)
 {
 	struct tok_state *tok = tok_new();
 	if (tok == NULL)
@@ -138,15 +131,13 @@ tok_setups(str)
 /* Set up tokenizer for file */
 
 struct tok_state *
-tok_setupf(fp, ps1, ps2)
-	FILE *fp;
-	char *ps1, *ps2;
+PyTokenizer_FromFile(FILE *fp, char *ps1, char *ps2)
 {
 	struct tok_state *tok = tok_new();
 	if (tok == NULL)
 		return NULL;
-	if ((tok->buf = NEW(char, BUFSIZ)) == NULL) {
-		DEL(tok);
+	if ((tok->buf = PyMem_NEW(char, BUFSIZ)) == NULL) {
+		PyMem_DEL(tok);
 		return NULL;
 	}
 	tok->cur = tok->inp = tok->buf;
@@ -161,24 +152,22 @@ tok_setupf(fp, ps1, ps2)
 /* Free a tok_state structure */
 
 void
-tok_free(tok)
-	struct tok_state *tok;
+PyTokenizer_Free(struct tok_state *tok)
 {
 	if (tok->fp != NULL && tok->buf != NULL)
-		DEL(tok->buf);
-	DEL(tok);
+		PyMem_DEL(tok->buf);
+	PyMem_DEL(tok);
 }
 
 
 /* Get next char, updating state; error code goes into tok->done */
 
 static int
-tok_nextc(tok)
-	register struct tok_state *tok;
+tok_nextc(register struct tok_state *tok)
 {
 	for (;;) {
 		if (tok->cur != tok->inp) {
-			return *tok->cur++; /* Fast path */
+			return Py_CHARMASK(*tok->cur++); /* Fast path */
 		}
 		if (tok->done != E_OK)
 			return EOF;
@@ -197,35 +186,36 @@ tok_nextc(tok)
 				tok->buf = tok->cur;
 			tok->lineno++;
 			tok->inp = end;
-			return *tok->cur++;
+			return Py_CHARMASK(*tok->cur++);
 		}
 		if (tok->prompt != NULL) {
-			char *new = my_readline(tok->prompt);
+			char *new = PyOS_Readline(tok->prompt);
 			if (tok->nextprompt != NULL)
 				tok->prompt = tok->nextprompt;
 			if (new == NULL)
 				tok->done = E_INTR;
 			else if (*new == '\0') {
-				free(new);
+				PyMem_FREE(new);
 				tok->done = E_EOF;
 			}
 			else if (tok->start != NULL) {
-				int start = tok->start - tok->buf;
-				int oldlen = tok->cur - tok->buf;
-				int newlen = oldlen + strlen(new);
-				char *buf = realloc(tok->buf, newlen+1);
+				size_t start = tok->start - tok->buf;
+				size_t oldlen = tok->cur - tok->buf;
+				size_t newlen = oldlen + strlen(new);
+				char *buf = tok->buf;
+				PyMem_RESIZE(buf, char, newlen+1);
 				tok->lineno++;
 				if (buf == NULL) {
-					free(tok->buf);
+					PyMem_DEL(tok->buf);
 					tok->buf = NULL;
-					free(new);
+					PyMem_FREE(new);
 					tok->done = E_NOMEM;
 					return EOF;
 				}
 				tok->buf = buf;
 				tok->cur = tok->buf + oldlen;
 				strcpy(tok->buf + oldlen, new);
-				free(new);
+				PyMem_FREE(new);
 				tok->inp = tok->buf + newlen;
 				tok->end = tok->inp + 1;
 				tok->start = tok->buf + start;
@@ -233,7 +223,7 @@ tok_nextc(tok)
 			else {
 				tok->lineno++;
 				if (tok->buf != NULL)
-					free(tok->buf);
+					PyMem_DEL(tok->buf);
 				tok->buf = new;
 				tok->cur = tok->buf;
 				tok->inp = strchr(tok->buf, '\0');
@@ -246,7 +236,7 @@ tok_nextc(tok)
 			char *pt;
 			if (tok->start == NULL) {
 				if (tok->buf == NULL) {
-					tok->buf = NEW(char, BUFSIZ);
+					tok->buf = PyMem_NEW(char, BUFSIZ);
 					if (tok->buf == NULL) {
 						tok->done = E_NOMEM;
 						return EOF;
@@ -279,10 +269,9 @@ tok_nextc(tok)
 				int curstart = tok->start == NULL ? -1 :
 					       tok->start - tok->buf;
 				int curvalid = tok->inp - tok->buf;
-				int cursize = tok->end - tok->buf;
 				int newsize = curvalid + BUFSIZ;
 				char *newbuf = tok->buf;
-				RESIZE(newbuf, char, newsize);
+				PyMem_RESIZE(newbuf, char, newsize);
 				if (newbuf == NULL) {
 					tok->done = E_NOMEM;
 					tok->cur = tok->inp;
@@ -304,17 +293,20 @@ tok_nextc(tok)
 				done = tok->inp[-1] == '\n';
 			}
 			tok->cur = tok->buf + cur;
+#ifndef macintosh
 			/* replace "\r\n" with "\n" */
+			/* For Mac we leave the \r, giving a syntax error */
 			pt = tok->inp - 2;
 			if (pt >= tok->buf && *pt == '\r') {
 				*pt++ = '\n';
 				*pt = '\0';
 				tok->inp = pt;
 			}
+#endif
 		}
 		if (tok->done != E_OK) {
 			if (tok->prompt != NULL)
-				fprintf(stderr, "\n");
+				PySys_WriteStderr("\n");
 			tok->cur = tok->inp;
 			return EOF;
 		}
@@ -326,13 +318,11 @@ tok_nextc(tok)
 /* Back-up one character */
 
 static void
-tok_backup(tok, c)
-	register struct tok_state *tok;
-	register int c;
+tok_backup(register struct tok_state *tok, register int c)
 {
 	if (c != EOF) {
 		if (--tok->cur < tok->buf)
-			fatal("tok_backup: begin of buffer");
+			Py_FatalError("tok_backup: begin of buffer");
 		if (*tok->cur != c)
 			*tok->cur = c;
 	}
@@ -342,8 +332,7 @@ tok_backup(tok, c)
 /* Return the token corresponding to a single character */
 
 int
-tok_1char(c)
-	int c;
+PyToken_OneChar(int c)
 {
 	switch (c) {
 	case '(':	return LPAR;
@@ -375,8 +364,7 @@ tok_1char(c)
 
 
 int
-tok_2char(c1, c2)
-	int c1, c2;
+PyToken_TwoChars(int c1, int c2)
 {
 	switch (c1) {
 	case '=':
@@ -402,22 +390,114 @@ tok_2char(c1, c2)
 		case '>':	return RIGHTSHIFT;
 		}
 		break;
+	case '+':
+		switch (c2) {
+		case '=':	return PLUSEQUAL;
+		}
+		break;
+	case '-':
+		switch (c2) {
+		case '=':	return MINEQUAL;
+		}
+		break;
 	case '*':
 		switch (c2) {
 		case '*':	return DOUBLESTAR;
+		case '=':	return STAREQUAL;
+		}
+		break;
+	case '/':
+		switch (c2) {
+		case '=':	return SLASHEQUAL;
+		}
+		break;
+	case '|':
+		switch (c2) {
+		case '=':	return VBAREQUAL;
+		}
+		break;
+	case '%':
+		switch (c2) {
+		case '=':	return PERCENTEQUAL;
+		}
+		break;
+	case '&':
+		switch (c2) {
+		case '=':	return AMPEREQUAL;
+		}
+		break;
+	case '^':
+		switch (c2) {
+		case '=':	return CIRCUMFLEXEQUAL;
 		}
 		break;
 	}
 	return OP;
 }
 
+int
+PyToken_ThreeChars(int c1, int c2, int c3)
+{
+	switch (c1) {
+	case '<':
+		switch (c2) {
+		case '<':
+			switch (c3) {
+			case '=':
+				return LEFTSHIFTEQUAL;
+				break;
+			}
+			break;
+		}
+		break;
+	case '>':
+		switch (c2) {
+		case '>':
+			switch (c3) {
+			case '=':
+				return RIGHTSHIFTEQUAL;
+				break;
+			}
+			break;
+		}
+		break;
+	case '*':
+		switch (c2) {
+		case '*':
+			switch (c3) {
+			case '=':
+				return DOUBLESTAREQUAL;
+				break;
+			}
+			break;
+		}
+		break;
+	}
+	return OP;
+}
+
+static int
+indenterror(struct tok_state *tok)
+{
+	if (tok->alterror) {
+		tok->done = E_TABSPACE;
+		tok->cur = tok->inp;
+		return 1;
+	}
+	if (tok->altwarning) {
+		PySys_WriteStderr("%s: inconsistent use of tabs and spaces "
+                                  "in indentation\n", tok->filename);
+		tok->altwarning = 0;
+	}
+	return 0;
+}
+
 
 /* Get next token, after space stripping etc. */
 
 int
-tok_get(tok, p_start, p_end)
-	register struct tok_state *tok; /* In/out: tokenizer state */
-	char **p_start, **p_end; /* Out: point to start/end of token */
+PyTokenizer_Get(register struct tok_state *tok, char **p_start,
+		char **p_end)
 {
 	register int c;
 	int blankline;
@@ -430,15 +510,19 @@ tok_get(tok, p_start, p_end)
 	/* Get indentation level */
 	if (tok->atbol) {
 		register int col = 0;
+		register int altcol = 0;
 		tok->atbol = 0;
 		for (;;) {
 			c = tok_nextc(tok);
 			if (c == ' ')
-				col++;
-			else if (c == '\t')
+				col++, altcol++;
+			else if (c == '\t') {
 				col = (col/tok->tabsize + 1) * tok->tabsize;
+				altcol = (altcol/tok->alttabsize + 1)
+					* tok->alttabsize;
+			}
 			else if (c == '\014') /* Control-L (formfeed) */
-				col = 0; /* For Emacs users */
+				col = altcol = 0; /* For Emacs users */
 			else
 				break;
 		}
@@ -459,31 +543,42 @@ tok_get(tok, p_start, p_end)
 		if (!blankline && tok->level == 0) {
 			if (col == tok->indstack[tok->indent]) {
 				/* No change */
+				if (altcol != tok->altindstack[tok->indent]) {
+					if (indenterror(tok))
+						return ERRORTOKEN;
+				}
 			}
 			else if (col > tok->indstack[tok->indent]) {
 				/* Indent -- always one */
 				if (tok->indent+1 >= MAXINDENT) {
-					fprintf(stderr, "excessive indent\n");
-					tok->done = E_TOKEN;
+					tok->done = E_TOODEEP;
 					tok->cur = tok->inp;
 					return ERRORTOKEN;
 				}
+				if (altcol <= tok->altindstack[tok->indent]) {
+					if (indenterror(tok))
+						return ERRORTOKEN;
+				}
 				tok->pendin++;
 				tok->indstack[++tok->indent] = col;
+				tok->altindstack[tok->indent] = altcol;
 			}
 			else /* col < tok->indstack[tok->indent] */ {
 				/* Dedent -- any number, must be consistent */
 				while (tok->indent > 0 &&
 					col < tok->indstack[tok->indent]) {
-					tok->indent--;
 					tok->pendin--;
+					tok->indent--;
 				}
 				if (col != tok->indstack[tok->indent]) {
-					fprintf(stderr, "inconsistent dedent\n");
-					tok->done = E_TOKEN;
+					tok->done = E_DEDENT;
 					tok->cur = tok->inp;
 					return ERRORTOKEN;
 				}
+				if (altcol != tok->altindstack[tok->indent]) {
+					if (indenterror(tok))
+						return ERRORTOKEN;
+				}
 			}
 		}
 	}
@@ -512,24 +607,40 @@ tok_get(tok, p_start, p_end)
 	/* Set start of current token */
 	tok->start = tok->cur - 1;
 	
-	/* Skip comment */
+	/* Skip comment, while looking for tab-setting magic */
 	if (c == '#') {
-		/* Hack to allow overriding the tabsize in the file.
-		   This is also recognized by vi, when it occurs near the
-		   beginning or end of the file.  (Will vi never die...?)
-		   For Python it must be at the beginning of the file! */
-		/* XXX The real vi syntax is actually different :-( */
-		/* XXX Should recognize Emacs syntax, too */
-		int x;
-		if (sscanf(tok->cur,
-				" vi:set tabsize=%d:", &x) == 1 &&
-						x >= 1 && x <= 40) {
-			/* fprintf(stderr, "# vi:set tabsize=%d:\n", x); */
-			tok->tabsize = x;
-		}
+		static char *tabforms[] = {
+			"tab-width:",		/* Emacs */
+			":tabstop=",		/* vim, full form */
+			":ts=",			/* vim, abbreviated form */
+			"set tabsize=",		/* will vi never die? */
+		/* more templates can be added here to support other editors */
+		};
+		char cbuf[80];
+		char *tp, **cp;
+		tp = cbuf;
 		do {
+			*tp++ = c = tok_nextc(tok);
+		} while (c != EOF && c != '\n' &&
+			 tp - cbuf + 1 < sizeof(cbuf));
+		*tp = '\0';
+		for (cp = tabforms; 
+		     cp < tabforms + sizeof(tabforms)/sizeof(tabforms[0]);
+		     cp++) {
+			if ((tp = strstr(cbuf, *cp))) {
+				int newsize = atoi(tp + strlen(*cp));
+
+				if (newsize >= 1 && newsize <= 40) {
+					tok->tabsize = newsize;
+					if (Py_VerboseFlag)
+					    PySys_WriteStderr(
+						"Tab size set to %d\n",
+						newsize);
+				}
+			}
+		}
+		while (c != EOF && c != '\n')
 			c = tok_nextc(tok);
-		} while (c != EOF && c != '\n');
 	}
 	
 	/* Check for EOF and errors now */
@@ -539,9 +650,26 @@ tok_get(tok, p_start, p_end)
 	
 	/* Identifier (most frequent token!) */
 	if (isalpha(c) || c == '_') {
-		do {
+		/* Process r"", u"" and ur"" */
+		switch (c) {
+		case 'r':
+		case 'R':
 			c = tok_nextc(tok);
-		} while (isalnum(c) || c == '_');
+			if (c == '"' || c == '\'')
+				goto letter_quote;
+			break;
+		case 'u':
+		case 'U':
+			c = tok_nextc(tok);
+			if (c == 'r' || c == 'R')
+				c = tok_nextc(tok);
+			if (c == '"' || c == '\'')
+				goto letter_quote;
+			break;
+		}
+		while (isalnum(c) || c == '_') {
+			c = tok_nextc(tok);
+		}
 		tok_backup(tok, c);
 		*p_start = tok->start;
 		*p_end = tok->cur;
@@ -558,6 +686,15 @@ tok_get(tok, p_start, p_end)
 		return NEWLINE;
 	}
 	
+#ifdef macintosh
+	if (c == '\r') {
+		PySys_WriteStderr(
+		  "File contains \\r characters (incorrect line endings?)\n");
+		tok->done = E_TOKEN;
+		tok->cur = tok->inp;
+		return ERRORTOKEN;
+	}
+#endif	
 	/* Period or number starting with period? */
 	if (c == '.') {
 		c = tok_nextc(tok);
@@ -641,9 +778,11 @@ tok_get(tok, p_start, p_end)
 		*p_end = tok->cur;
 		return NUMBER;
 	}
-	
+
+  letter_quote:
 	/* String */
 	if (c == '\'' || c == '"') {
+		int quote2 = tok->cur - tok->start + 1;
 		int quote = c;
 		int triple = 0;
 		int tripcount = 0;
@@ -664,7 +803,7 @@ tok_get(tok, p_start, p_end)
 			}
 			else if (c == quote) {
 				tripcount++;
-				if (tok->cur == tok->start+2) {
+				if (tok->cur - tok->start == quote2) {
 					c = tok_nextc(tok);
 					if (c == quote) {
 						triple = 1;
@@ -707,8 +846,15 @@ tok_get(tok, p_start, p_end)
 	/* Check for two-character token */
 	{
 		int c2 = tok_nextc(tok);
-		int token = tok_2char(c, c2);
+		int token = PyToken_TwoChars(c, c2);
 		if (token != OP) {
+			int c3 = tok_nextc(tok);
+			int token3 = PyToken_ThreeChars(c, c2, c3);
+			if (token3 != OP) {
+				token = token3;
+			} else {
+				tok_backup(tok, c3);
+			}
 			*p_start = tok->start;
 			*p_end = tok->cur;
 			return token;
@@ -733,18 +879,16 @@ tok_get(tok, p_start, p_end)
 	/* Punctuation character */
 	*p_start = tok->start;
 	*p_end = tok->cur;
-	return tok_1char(c);
+	return PyToken_OneChar(c);
 }
 
 
-#ifdef DEBUG
+#ifdef Py_DEBUG
 
 void
-tok_dump(type, start, end)
-	int type;
-	char *start, *end;
+tok_dump(int type, char *start, char *end)
 {
-	printf("%s", tok_name[type]);
+	printf("%s", _PyParser_TokenNames[type]);
 	if (type == NAME || type == NUMBER || type == STRING || type == OP)
 		printf("(%.*s)", (int)(end - start), start);
 }
