@@ -1,115 +1,54 @@
-/***********************************************************
-Copyright 1991-1995 by Stichting Mathematisch Centrum, Amsterdam,
-The Netherlands.
-
-                        All Rights Reserved
-
-Permission to use, copy, modify, and distribute this software and its
-documentation for any purpose and without fee is hereby granted,
-provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in
-supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI or Corporation for National Research Initiatives or
-CNRI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior
-permission.
-
-While CWI is the initial source for this software, a modified version
-is made available by the Corporation for National Research Initiatives
-(CNRI) at the Internet address ftp://ftp.python.org.
-
-STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
-REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
-MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
-CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
-DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
-PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
-TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-PERFORMANCE OF THIS SOFTWARE.
-
-******************************************************************/
 
 /* New getargs implementation */
 
 /* XXX There are several unchecked sprintf or strcat calls in this file.
    XXX The only way these can become a danger is if some C code in the
    XXX Python source (or in an extension) uses ridiculously long names
-   XXX or riduculously deep nesting in format strings. */
+   XXX or ridiculously deep nesting in format strings. */
 
 #include "Python.h"
 
 #include <ctype.h>
 
 
-int PyArg_Parse Py_PROTO((PyObject *, char *, ...));
-int PyArg_ParseTuple Py_PROTO((PyObject *, char *, ...));
-int PyArg_VaParse Py_PROTO((PyObject *, char *, va_list));
+int PyArg_Parse(PyObject *, char *, ...);
+int PyArg_ParseTuple(PyObject *, char *, ...);
+int PyArg_VaParse(PyObject *, char *, va_list);
 
-int PyArg_ParseTupleAndKeywords Py_PROTO((PyObject *, PyObject *,
-				       char *, char **, ...));
+int PyArg_ParseTupleAndKeywords(PyObject *, PyObject *,
+				char *, char **, ...);
 
 /* Forward */
-static int vgetargs1 Py_PROTO((PyObject *, char *, va_list *, int));
-static void seterror Py_PROTO((int, char *, int *, char *, char *));
-static char *convertitem Py_PROTO((PyObject *, char **, va_list *,
-				   int *, char *));
-static char *converttuple Py_PROTO((PyObject *, char **, va_list *,
-				 int *, char *, int));
-static char *convertsimple Py_PROTO((PyObject *, char **, va_list *, char *));
-static char *convertsimple1 Py_PROTO((PyObject *, char **, va_list *));
+static int vgetargs1(PyObject *, char *, va_list *, int);
+static void seterror(int, char *, int *, char *, char *);
+static char *convertitem(PyObject *, char **, va_list *, int *, char *);
+static char *converttuple(PyObject *, char **, va_list *,
+			  int *, char *, int);
+static char *convertsimple(PyObject *, char **, va_list *, char *);
+static char *convertsimple1(PyObject *, char **, va_list *);
 
-static int vgetargskeywords Py_PROTO((PyObject *, PyObject *,
-				   char *, char **, va_list *));
-static char *skipitem Py_PROTO((char **, va_list *));
+static int vgetargskeywords(PyObject *, PyObject *,
+			    char *, char **, va_list *);
+static char *skipitem(char **, va_list *);
 
-#ifdef HAVE_STDARG_PROTOTYPES
-/* VARARGS2 */
 int PyArg_Parse(PyObject *args, char *format, ...)
-#else
-/* VARARGS */
-int PyArg_Parse(va_alist) va_dcl
-#endif
 {
 	int retval;
 	va_list va;
-#ifdef HAVE_STDARG_PROTOTYPES
 	
 	va_start(va, format);
-#else
-	PyObject *args;
-	char *format;
-	
-	va_start(va);
-	args = va_arg(va, PyObject *);
-	format = va_arg(va, char *);
-#endif
 	retval = vgetargs1(args, format, &va, 1);
 	va_end(va);
 	return retval;
 }
 
 
-#ifdef HAVE_STDARG_PROTOTYPES
-/* VARARGS2 */
 int PyArg_ParseTuple(PyObject *args, char *format, ...)
-#else
-/* VARARGS */
-int PyArg_ParseTuple(va_alist) va_dcl
-#endif
 {
 	int retval;
 	va_list va;
-#ifdef HAVE_STDARG_PROTOTYPES
 	
 	va_start(va, format);
-#else
-	PyObject *args;
-	char *format;
-	
-	va_start(va);
-	args = va_arg(va, PyObject *);
-	format = va_arg(va, char *);
-#endif
 	retval = vgetargs1(args, format, &va, 0);
 	va_end(va);
 	return retval;
@@ -117,10 +56,7 @@ int PyArg_ParseTuple(va_alist) va_dcl
 
 
 int
-PyArg_VaParse(args, format, va)
-	PyObject *args;
-	char *format;
-	va_list va;
+PyArg_VaParse(PyObject *args, char *format, va_list va)
 {
 	va_list lva;
 
@@ -135,11 +71,7 @@ PyArg_VaParse(args, format, va)
 
 
 static int
-vgetargs1(args, format, p_va, compat)
-	PyObject *args;
-	char *format;
-	va_list *p_va;
-	int compat;
+vgetargs1(PyObject *args, char *format, va_list *p_va, int compat)
 {
 	char msgbuf[256];
 	int levels[32];
@@ -152,6 +84,8 @@ vgetargs1(args, format, p_va, compat)
 	int i, len;
 	char *msg;
 	
+	assert(compat || (args != (PyObject*)NULL));
+
 	for (;;) {
 		int c = *format++;
 		if (c == '(' /* ')' */) {
@@ -178,6 +112,8 @@ vgetargs1(args, format, p_va, compat)
 		}
 		else if (level != 0)
 			; /* Pass */
+		else if (c == 'e')
+			; /* Pass */
 		else if (isalpha(c))
 			max++;
 		else if (c == '|')
@@ -196,16 +132,18 @@ vgetargs1(args, format, p_va, compat)
 		if (max == 0) {
 			if (args == NULL)
 				return 1;
-			sprintf(msgbuf, "%s requires no arguments",
-				fname==NULL ? "function" : fname);
+			sprintf(msgbuf, "%s%s takes no arguments",
+				fname==NULL ? "function" : fname,
+				fname==NULL ? "" : "()");
 			PyErr_SetString(PyExc_TypeError, msgbuf);
 			return 0;
 		}
 		else if (min == 1 && max == 1) {
 			if (args == NULL) {
 				sprintf(msgbuf,
-					"%s requires at least one argument",
-					fname==NULL ? "function" : fname);
+					"%s%s takes at least one argument",
+					fname==NULL ? "function" : fname,
+					fname==NULL ? "" : "()");
 				PyErr_SetString(PyExc_TypeError, msgbuf);
 				return 0;
 			}
@@ -228,13 +166,14 @@ vgetargs1(args, format, p_va, compat)
 		return 0;
 	}
 	
-	len = PyTuple_Size(args);
+	len = PyTuple_GET_SIZE(args);
 	
 	if (len < min || max < len) {
 		if (message == NULL) {
 			sprintf(msgbuf,
-				"%s requires %s %d argument%s; %d given",
+				"%s%s takes %s %d argument%s (%d given)",
 				fname==NULL ? "function" : fname,
+				fname==NULL ? "" : "()",
 				min==max ? "exactly"
 				         : len < min ? "at least" : "at most",
 				len < min ? min : max,
@@ -249,8 +188,8 @@ vgetargs1(args, format, p_va, compat)
 	for (i = 0; i < len; i++) {
 		if (*format == '|')
 			format++;
-		msg = convertitem(PyTuple_GetItem(args, i), &format, p_va,
-				 levels, msgbuf);
+		msg = convertitem(PyTuple_GET_ITEM(args, i), &format, p_va,
+				  levels, msgbuf);
 		if (msg) {
 			seterror(i+1, msg, levels, fname, message);
 			return 0;
@@ -271,12 +210,7 @@ vgetargs1(args, format, p_va, compat)
 
 
 static void
-seterror(iarg, msg, levels, fname, message)
-	int iarg;
-	char *msg;
-	int *levels;
-	char *fname;
-	char *message;
+seterror(int iarg, char *msg, int *levels, char *fname, char *message)
 {
 	char buf[256];
 	int i;
@@ -284,22 +218,26 @@ seterror(iarg, msg, levels, fname, message)
 
 	if (PyErr_Occurred())
 		return;
-	if (iarg == 0 && message == NULL)
-		message = msg;
 	else if (message == NULL) {
 		if (fname != NULL) {
-			sprintf(p, "%s, ", fname);
+			sprintf(p, "%s() ", fname);
 			p += strlen(p);
 		}
-		sprintf(p, "argument %d", iarg);
-		i = 0;
-		p += strlen(p);
-		while (levels[i] > 0) {
-			sprintf(p, ", item %d", levels[i]-1);
+		if (iarg != 0) {
+			sprintf(p, "argument %d", iarg);
+			i = 0;
 			p += strlen(p);
-			i++;
+			while (levels[i] > 0) {
+				sprintf(p, ", item %d", levels[i]-1);
+				p += strlen(p);
+				i++;
+			}
 		}
-		sprintf(p, ": expected %s found", msg);
+		else {
+			sprintf(p, "argument");
+			p += strlen(p);
+		}
+		sprintf(p, " %s", msg);
 		message = buf;
 	}
 	PyErr_SetString(PyExc_TypeError, message);
@@ -318,21 +256,15 @@ seterror(iarg, msg, levels, fname, message)
       *p_va is undefined,
       *levels is a 0-terminated list of item numbers,
       *msgbuf contains an error message, whose format is:
-         "<typename1>, <typename2>", where:
+         "must be <typename1>, not <typename2>", where:
             <typename1> is the name of the expected type, and
             <typename2> is the name of the actual type,
-         (so you can surround it by "expected ... found"),
       and msgbuf is returned.
 */
 
 static char *
-converttuple(arg, p_format, p_va, levels, msgbuf, toplevel)
-	PyObject *arg;
-	char **p_format;
-	va_list *p_va;
-	int *levels;
-	char *msgbuf;
-	int toplevel;
+converttuple(PyObject *arg, char **p_format, va_list *p_va, int *levels,
+	     char *msgbuf, int toplevel)
 {
 	int level = 0;
 	int n = 0;
@@ -357,22 +289,24 @@ converttuple(arg, p_format, p_va, levels, msgbuf, toplevel)
 			n++;
 	}
 	
-	if (!PySequence_Check(arg)) {
+	if (!PySequence_Check(arg) || PyString_Check(arg)) {
 		levels[0] = 0;
 		sprintf(msgbuf,
-			toplevel ? "%d arguments, %s" : "%d-sequence, %s",
+			toplevel ? "expected %d arguments, not %s" :
+				   "must be %d-item sequence, not %s",
 			n, arg == Py_None ? "None" : arg->ob_type->tp_name);
 		return msgbuf;
 	}
 	
-	if ((i = PySequence_Length(arg)) != n) {
+	if ((i = PySequence_Size(arg)) != n) {
 		levels[0] = 0;
 		sprintf(msgbuf,
-		    toplevel ? "%d arguments, %d" : "%d-sequence, %d-sequence",
-		    n, i);
+			toplevel ? "expected %d arguments, not %d" :
+				   "must be sequence of length %d, not %d",
+			n, i);
 		return msgbuf;
 	}
-	
+
 	format = *p_format;
 	for (i = 0; i < n; i++) {
 		char *msg;
@@ -386,7 +320,7 @@ converttuple(arg, p_format, p_va, levels, msgbuf, toplevel)
 			return msg;
 		}
 	}
-	
+
 	*p_format = format;
 	return NULL;
 }
@@ -395,12 +329,8 @@ converttuple(arg, p_format, p_va, levels, msgbuf, toplevel)
 /* Convert a single item. */
 
 static char *
-convertitem(arg, p_format, p_va, levels, msgbuf)
-	PyObject *arg;
-	char **p_format;
-	va_list *p_va;
-	int *levels;
-	char *msgbuf;
+convertitem(PyObject *arg, char **p_format, va_list *p_va, int *levels,
+	    char *msgbuf)
 {
 	char *msg;
 	char *format = *p_format;
@@ -423,18 +353,14 @@ convertitem(arg, p_format, p_va, levels, msgbuf)
 
 
 /* Convert a non-tuple argument.  Adds to convertsimple1 functionality
-   by appending ", <actual argument type>" to error message. */
+   by formatting messages as "must be <desired type>, not <actual type>". */
 
 static char *
-convertsimple(arg, p_format, p_va, msgbuf)
-	PyObject *arg;
-	char **p_format;
-	va_list *p_va;
-	char *msgbuf;
+convertsimple(PyObject *arg, char **p_format, va_list *p_va, char *msgbuf)
 {
 	char *msg = convertsimple1(arg, p_format, p_va);
 	if (msg != NULL) {
-		sprintf(msgbuf, "%.50s, %.50s", msg,
+		sprintf(msgbuf, "must be %.50s, not %.50s", msg,
 			arg == Py_None ? "None" : arg->ob_type->tp_name);
 		msg = msgbuf;
 	}
@@ -442,55 +368,128 @@ convertsimple(arg, p_format, p_va, msgbuf)
 }
 
 
+/* Internal API needed by convertsimple1(): */
+extern 
+PyObject *_PyUnicode_AsDefaultEncodedString(PyObject *unicode,
+				  const char *errors);
+
 /* Convert a non-tuple argument.  Return NULL if conversion went OK,
    or a string representing the expected type if the conversion failed.
    When failing, an exception may or may not have been raised.
    Don't call if a tuple is expected. */
 
 static char *
-convertsimple1(arg, p_format, p_va)
-	PyObject *arg;
-	char **p_format;
-	va_list *p_va;
+convertsimple1(PyObject *arg, char **p_format, va_list *p_va)
 {
 	char *format = *p_format;
 	char c = *format++;
 	
 	switch (c) {
 	
-	case 'b': /* byte -- very short int */
+	case 'b': /* unsigned byte -- very short int */
 		{
 			char *p = va_arg(*p_va, char *);
 			long ival = PyInt_AsLong(arg);
 			if (ival == -1 && PyErr_Occurred())
 				return "integer<b>";
+			else if (ival < 0) {
+				PyErr_SetString(PyExc_OverflowError,
+			      "unsigned byte integer is less than minimum");
+				return "integer<b>";
+			}
+			else if (ival > UCHAR_MAX) {
+				PyErr_SetString(PyExc_OverflowError,
+			      "unsigned byte integer is greater than maximum");
+				return "integer<b>";
+			}
 			else
-				*p = (char) ival;
+				*p = (unsigned char) ival;
 			break;
 		}
 	
-	case 'h': /* short int */
+	case 'B': /* byte sized bitfield - both signed and unsigned values allowed */
+		{
+			char *p = va_arg(*p_va, char *);
+			long ival = PyInt_AsLong(arg);
+			if (ival == -1 && PyErr_Occurred())
+				return "integer<b>";
+			else if (ival < SCHAR_MIN) {
+				PyErr_SetString(PyExc_OverflowError,
+			      "byte-sized integer bitfield is less than minimum");
+				return "integer<B>";
+			}
+			else if (ival > (int)UCHAR_MAX) {
+				PyErr_SetString(PyExc_OverflowError,
+			      "byte-sized integer bitfield is greater than maximum");
+				return "integer<B>";
+			}
+			else
+				*p = (unsigned char) ival;
+			break;
+		}
+	
+	case 'h': /* signed short int */
 		{
 			short *p = va_arg(*p_va, short *);
 			long ival = PyInt_AsLong(arg);
 			if (ival == -1 && PyErr_Occurred())
 				return "integer<h>";
+			else if (ival < SHRT_MIN) {
+				PyErr_SetString(PyExc_OverflowError,
+			      "signed short integer is less than minimum");
+				return "integer<h>";
+			}
+			else if (ival > SHRT_MAX) {
+				PyErr_SetString(PyExc_OverflowError,
+			      "signed short integer is greater than maximum");
+				return "integer<h>";
+			}
 			else
 				*p = (short) ival;
 			break;
 		}
 	
-	case 'i': /* int */
+	case 'H': /* short int sized bitfield, both signed and unsigned allowed */
+		{
+			unsigned short *p = va_arg(*p_va, unsigned short *);
+			long ival = PyInt_AsLong(arg);
+			if (ival == -1 && PyErr_Occurred())
+				return "integer<H>";
+			else if (ival < SHRT_MIN) {
+				PyErr_SetString(PyExc_OverflowError,
+			      "short integer bitfield is less than minimum");
+				return "integer<H>";
+			}
+			else if (ival > USHRT_MAX) {
+				PyErr_SetString(PyExc_OverflowError,
+			      "short integer bitfield is greater than maximum");
+				return "integer<H>";
+			}
+			else
+				*p = (unsigned short) ival;
+			break;
+		}
+	
+	case 'i': /* signed int */
 		{
 			int *p = va_arg(*p_va, int *);
 			long ival = PyInt_AsLong(arg);
 			if (ival == -1 && PyErr_Occurred())
 				return "integer<i>";
+			else if (ival > INT_MAX) {
+				PyErr_SetString(PyExc_OverflowError,
+				    "signed integer is greater than maximum");
+				return "integer<i>";
+			}
+			else if (ival < INT_MIN) {
+				PyErr_SetString(PyExc_OverflowError,
+				    "signed integer is less than minimum");
+				return "integer<i>";
+			}
 			else
 				*p = ival;
 			break;
 		}
-	
 	case 'l': /* long int */
 		{
 			long *p = va_arg(*p_va, long *);
@@ -564,28 +563,49 @@ convertsimple1(arg, p_format, p_va)
 	
 	case 's': /* string */
 		{
-			if (*format == '#') { /* any buffer-like object */
-			        void **p = (void **)va_arg(*p_va, char **);
-			        PyBufferProcs *pb = arg->ob_type->tp_as_buffer;
+			if (*format == '#') {
+				void **p = (void **)va_arg(*p_va, char **);
 				int *q = va_arg(*p_va, int *);
-				int count;
 
-				if ( pb == NULL ||
-				     pb->bf_getreadbuffer == NULL ||
-				     pb->bf_getsegcount == NULL )
-				  return "read-only buffer";
-				if ( (*pb->bf_getsegcount)(arg, NULL) != 1 )
-				  return "single-segment read-only buffer";
-				if ( (count =
-				      (*pb->bf_getreadbuffer)(arg, 0, p)) < 0 )
-				  return "(unspecified)";
-				*q = count;
+				if (PyString_Check(arg)) {
+				    *p = PyString_AS_STRING(arg);
+				    *q = PyString_GET_SIZE(arg);
+				}
+				else if (PyUnicode_Check(arg)) {
+				    arg = _PyUnicode_AsDefaultEncodedString(
+							            arg, NULL);
+				    if (arg == NULL)
+					return "(unicode conversion error)";
+				    *p = PyString_AS_STRING(arg);
+				    *q = PyString_GET_SIZE(arg);
+				}
+				else { /* any buffer-like object */
+				    PyBufferProcs *pb = arg->ob_type->tp_as_buffer;
+				    int count;
+				    if ( pb == NULL ||
+					 pb->bf_getreadbuffer == NULL ||
+					 pb->bf_getsegcount == NULL )
+					return "string or read-only buffer";
+				    if ( (*pb->bf_getsegcount)(arg, NULL) != 1 )
+					return "string or single-segment read-only buffer";
+				    if ( (count =
+					  (*pb->bf_getreadbuffer)(arg, 0, p)) < 0 )
+					return "(unspecified)";
+				    *q = count;
+				}
 				format++;
 			} else {
-			        char **p = va_arg(*p_va, char **);
+				char **p = va_arg(*p_va, char **);
 			
-			        if (PyString_Check(arg))
-				  *p = PyString_AsString(arg);
+				if (PyString_Check(arg))
+				    *p = PyString_AS_STRING(arg);
+				else if (PyUnicode_Check(arg)) {
+				    arg = _PyUnicode_AsDefaultEncodedString(
+							            arg, NULL);
+				    if (arg == NULL)
+					return "(unicode conversion error)";
+				    *p = PyString_AS_STRING(arg);
+				}
 				else
 				  return "string";
 				if ((int)strlen(*p) != PyString_Size(arg))
@@ -597,36 +617,56 @@ convertsimple1(arg, p_format, p_va)
 	case 'z': /* string, may be NULL (None) */
 		{
 			if (*format == '#') { /* any buffer-like object */
-			        void **p = (void **)va_arg(*p_va, char **);
-			        PyBufferProcs *pb = arg->ob_type->tp_as_buffer;
+				void **p = (void **)va_arg(*p_va, char **);
 				int *q = va_arg(*p_va, int *);
-				int count;
 
 				if (arg == Py_None) {
 				  *p = 0;
 				  *q = 0;
-				} else {
-				  if ( pb == NULL ||
-				       pb->bf_getreadbuffer == NULL ||
-				       pb->bf_getsegcount == NULL )
-				    return "read-only buffer";
-				  if ( (*pb->bf_getsegcount)(arg, NULL) != 1 )
-				  return "single-segment read-only buffer";
-				  if ( (count = (*pb->bf_getreadbuffer)
-					                    (arg, 0, p)) < 0 )
-				    return "(unspecified)";
-				  *q = count;
+				}
+				else if (PyString_Check(arg)) {
+				    *p = PyString_AS_STRING(arg);
+				    *q = PyString_GET_SIZE(arg);
+				}
+				else if (PyUnicode_Check(arg)) {
+				    arg = _PyUnicode_AsDefaultEncodedString(
+							            arg, NULL);
+				    if (arg == NULL)
+					return "(unicode conversion error)";
+				    *p = PyString_AS_STRING(arg);
+				    *q = PyString_GET_SIZE(arg);
+				}
+				else { /* any buffer-like object */
+				    PyBufferProcs *pb = arg->ob_type->tp_as_buffer;
+				    int count;
+				    if ( pb == NULL ||
+					 pb->bf_getreadbuffer == NULL ||
+					 pb->bf_getsegcount == NULL )
+					return "string or read-only buffer";
+				    if ( (*pb->bf_getsegcount)(arg, NULL) != 1 )
+					return "string or single-segment read-only buffer";
+				    if ( (count =
+					  (*pb->bf_getreadbuffer)(arg, 0, p)) < 0 )
+					return "(unspecified)";
+				    *q = count;
 				}
 				format++;
 			} else {
-			        char **p = va_arg(*p_va, char **);
+				char **p = va_arg(*p_va, char **);
 			
-			        if (arg == Py_None)
+				if (arg == Py_None)
 				  *p = 0;
 				else if (PyString_Check(arg))
 				  *p = PyString_AsString(arg);
+				else if (PyUnicode_Check(arg)) {
+				  arg = _PyUnicode_AsDefaultEncodedString(
+								  arg, NULL);
+				  if (arg == NULL)
+				      return "(unicode conversion error)";
+				  *p = PyString_AS_STRING(arg);
+				}
 				else
-				  return "None or string";
+				  return "string or None";
 				if (*format == '#') {
 				  int *q = va_arg(*p_va, int *);
 				  if (arg == Py_None)
@@ -637,11 +677,158 @@ convertsimple1(arg, p_format, p_va)
 				}
 				else if (*p != NULL &&
 					 (int)strlen(*p) != PyString_Size(arg))
-				  return "None or string without null bytes";
+				  return "string without null bytes or None";
 			}
 			break;
 		}
 	
+	case 'e': /* encoded string */
+		{
+			char **buffer;
+			const char *encoding;
+			PyObject *u, *s;
+			int size;
+
+			/* Get 'e' parameter: the encoding name */
+			encoding = (const char *)va_arg(*p_va, const char *);
+			if (encoding == NULL)
+			    	encoding = PyUnicode_GetDefaultEncoding();
+			
+			/* Get 's' parameter: the output buffer to use */
+			if (*format != 's')
+				return "(unknown parser marker combination)";
+			buffer = (char **)va_arg(*p_va, char **);
+			format++;
+			if (buffer == NULL)
+				return "(buffer is NULL)";
+			
+			/* Convert object to Unicode */
+			u = PyUnicode_FromObject(arg);
+			if (u == NULL)
+				return "string or unicode or text buffer";
+			
+			/* Encode object; use default error handling */
+			s = PyUnicode_AsEncodedString(u,
+						      encoding,
+						      NULL);
+			Py_DECREF(u);
+			if (s == NULL)
+				return "(encoding failed)";
+			if (!PyString_Check(s)) {
+				Py_DECREF(s);
+				return "(encoder failed to return a string)";
+			}
+			size = PyString_GET_SIZE(s);
+
+			/* Write output; output is guaranteed to be
+			   0-terminated */
+			if (*format == '#') { 
+				/* Using buffer length parameter '#':
+
+				   - if *buffer is NULL, a new buffer
+				   of the needed size is allocated and
+				   the data copied into it; *buffer is
+				   updated to point to the new buffer;
+				   the caller is responsible for
+				   PyMem_Free()ing it after usage
+
+				   - if *buffer is not NULL, the data
+				   is copied to *buffer; *buffer_len
+				   has to be set to the size of the
+				   buffer on input; buffer overflow is
+				   signalled with an error; buffer has
+				   to provide enough room for the
+				   encoded string plus the trailing
+				   0-byte
+
+				   - in both cases, *buffer_len is
+				   updated to the size of the buffer
+				   /excluding/ the trailing 0-byte
+
+				*/
+				int *buffer_len = va_arg(*p_va, int *);
+
+				format++;
+				if (buffer_len == NULL)
+					return "(buffer_len is NULL)";
+				if (*buffer == NULL) {
+					*buffer = PyMem_NEW(char, size + 1);
+					if (*buffer == NULL) {
+						Py_DECREF(s);
+						return "(memory error)";
+					}
+				} else {
+					if (size + 1 > *buffer_len) {
+						Py_DECREF(s);
+						return "(buffer overflow)";
+					}
+				}
+				memcpy(*buffer,
+				       PyString_AS_STRING(s),
+				       size + 1);
+				*buffer_len = size;
+			} else {
+				/* Using a 0-terminated buffer:
+
+				   - the encoded string has to be
+				   0-terminated for this variant to
+				   work; if it is not, an error raised
+
+				   - a new buffer of the needed size
+				   is allocated and the data copied
+				   into it; *buffer is updated to
+				   point to the new buffer; the caller
+				   is responsible for PyMem_Free()ing it
+				   after usage
+
+				 */
+				if ((int)strlen(PyString_AS_STRING(s)) != size)
+					return "(encoded string without NULL bytes)";
+				*buffer = PyMem_NEW(char, size + 1);
+				if (*buffer == NULL) {
+					Py_DECREF(s);
+					return "(memory error)";
+				}
+				memcpy(*buffer,
+				       PyString_AS_STRING(s),
+				       size + 1);
+			}
+			Py_DECREF(s);
+			break;
+		}
+
+	case 'u': /* raw unicode buffer (Py_UNICODE *) */
+		{
+			if (*format == '#') { /* any buffer-like object */
+				void **p = (void **)va_arg(*p_va, char **);
+				PyBufferProcs *pb = arg->ob_type->tp_as_buffer;
+				int *q = va_arg(*p_va, int *);
+				int count;
+
+				if ( pb == NULL ||
+				     pb->bf_getreadbuffer == NULL ||
+				     pb->bf_getsegcount == NULL )
+				  return "unicode or read-only buffer";
+				if ( (*pb->bf_getsegcount)(arg, NULL) != 1 )
+				  return "unicode or single-segment read-only buffer";
+				if ( (count =
+				      (*pb->bf_getreadbuffer)(arg, 0, p)) < 0 )
+				  return "(unspecified)";
+				/* buffer interface returns bytes, we want
+				   length in characters */
+				*q = count/(sizeof(Py_UNICODE)); 
+				format++;
+			} else {
+				Py_UNICODE **p = va_arg(*p_va, Py_UNICODE **);
+			
+				if (PyUnicode_Check(arg))
+				    *p = PyUnicode_AS_UNICODE(arg);
+				else
+				  return "unicode";
+			}
+			break;
+		}
+
 	case 'S': /* string object */
 		{
 			PyObject **p = va_arg(*p_va, PyObject **);
@@ -652,6 +839,16 @@ convertsimple1(arg, p_format, p_va)
 			break;
 		}
 	
+	case 'U': /* Unicode object */
+		{
+			PyObject **p = va_arg(*p_va, PyObject **);
+			if (PyUnicode_Check(arg))
+				*p = arg;
+			else
+				return "unicode";
+			break;
+		}
+	
 	case 'O': /* object */
 		{
 			PyTypeObject *type;
@@ -673,12 +870,11 @@ convertsimple1(arg, p_format, p_va)
 				if ((*pred)(arg)) 
 					*p = arg;
 				else
-				         return "(unspecified)";
+					return "(unspecified)";
 				
 			}
 			else if (*format == '&') {
-				typedef int (*converter)
-					Py_PROTO((PyObject *, void *));
+				typedef int (*converter)(PyObject *, void *);
 				converter convert = va_arg(*p_va, converter);
 				void *addr = va_arg(*p_va, void *);
 				format++;
@@ -729,9 +925,9 @@ convertsimple1(arg, p_format, p_va)
 			     pb == NULL ||
 			     pb->bf_getcharbuffer == NULL ||
 			     pb->bf_getsegcount == NULL )
-				return "read-only character buffer";
+				return "string or read-only character buffer";
 			if ( (*pb->bf_getsegcount)(arg, NULL) != 1 )
-				return "single-segment read-only buffer";
+				return "string or single-segment read-only buffer";
 			if ( (count = pb->bf_getcharbuffer(arg, 0, p)) < 0 )
 				return "(unspecified)";
 
@@ -754,34 +950,15 @@ convertsimple1(arg, p_format, p_va)
 /* Support for keyword arguments donated by
    Geoff Philbrick <philbric@delphi.hks.com> */
 
-#ifdef HAVE_STDARG_PROTOTYPES
-/* VARARGS2 */
 int PyArg_ParseTupleAndKeywords(PyObject *args,
 				PyObject *keywords,
 				char *format, 
 				char **kwlist, ...)
-#else
-/* VARARGS */
-int PyArg_ParseTupleAndKeywords(va_alist) va_dcl
-#endif
 {
 	int retval;
 	va_list va;
-#ifdef HAVE_STDARG_PROTOTYPES
 	
 	va_start(va, kwlist);
-#else
-	PyObject *args;
-	PyObject *keywords;
-	char *format;
-	char **kwlist;
-	
-	va_start(va);
-	args = va_arg(va, PyObject *);
-	keywords = va_arg(va, PyObject *);
-	format = va_arg(va, char *);
-	kwlist = va_arg(va, char **);
-#endif
 	retval = vgetargskeywords(args, keywords, format, kwlist, &va);	
 	va_end(va);
 	return retval;
@@ -789,12 +966,8 @@ int PyArg_ParseTupleAndKeywords(va_alist) va_dcl
 
 
 static int
-vgetargskeywords(args, keywords, format, kwlist, p_va)
-	PyObject *args;
-	PyObject *keywords;
-	char *format;
-	char **kwlist;
-	va_list *p_va;
+vgetargskeywords(PyObject *args, PyObject *keywords, char *format,
+	         char **kwlist, va_list *p_va)
 {
 	char msgbuf[256];
 	int levels[32];
@@ -822,11 +995,11 @@ vgetargskeywords(args, keywords, format, kwlist, p_va)
 		else if (c == ':') {
 			fname = format;
 			break;
-		}
-		else if (c == ';') {
+		} else if (c == ';') {
 			message = format;
 			break;
-		}
+		} else if (c == 'e')
+			; /* Pass */
 		else if (isalpha(c))
 			max++;
 		else if (c == '|')
@@ -844,14 +1017,19 @@ vgetargskeywords(args, keywords, format, kwlist, p_va)
 		return 0;
 	}	
 	
-	tplen = PyTuple_Size(args);
+	tplen = PyTuple_GET_SIZE(args);
 	
 	/* do a cursory check of the keywords just to see how many we got */
 	   
 	if (keywords) { 	
 		if (!PyDict_Check(keywords)) {
-			PyErr_SetString(PyExc_SystemError,
-	  "non-dictionary object received when keyword dictionary expected");
+			if (keywords == NULL)
+				PyErr_SetString(PyExc_SystemError,
+		     "NULL received when keyword dictionary expected");
+			else
+				PyErr_Format(PyExc_SystemError,
+		     "%s received when keyword dictionary expected",
+					     keywords->ob_type->tp_name);
 			return 0;
 		}	
 		kwlen = PyDict_Size(keywords);
@@ -898,8 +1076,9 @@ vgetargskeywords(args, keywords, format, kwlist, p_va)
 	if (len < min || max < len) {
 		if (message == NULL) {
 			sprintf(msgbuf,
-				"%s requires %s %d argument%s; %d given",
+				"%s%s takes %s %d argument%s (%d given)",
 				fname==NULL ? "function" : fname,
+				fname==NULL ? "" : "()",
 				min==max ? "exactly"
 				         : len < min ? "at least" : "at most",
 				len < min ? min : max,
@@ -914,7 +1093,7 @@ vgetargskeywords(args, keywords, format, kwlist, p_va)
 	for (i = 0; i < tplen; i++) {
 		if (*format == '|')
 			format++;
-		msg = convertitem(PyTuple_GetItem(args, i), &format, p_va,
+		msg = convertitem(PyTuple_GET_ITEM(args, i), &format, p_va,
 				 levels, msgbuf);
 		if (msg) {
 			seterror(i+1, msg, levels, fname, message);
@@ -958,6 +1137,7 @@ vgetargskeywords(args, keywords, format, kwlist, p_va)
 				return 0;
 			}
 			converted++;
+			Py_DECREF(item);
 		}
 		else {
 			PyErr_Clear();
@@ -984,7 +1164,7 @@ vgetargskeywords(args, keywords, format, kwlist, p_va)
 			}
 			if (!match) {
 				sprintf(msgbuf,
-		         "%s is an invalid keyword argument for this function",
+			"%s is an invalid keyword argument for this function",
 					ks);
 				PyErr_SetString(PyExc_TypeError, msgbuf);
 				return 0;
@@ -997,9 +1177,7 @@ vgetargskeywords(args, keywords, format, kwlist, p_va)
 
 
 static char *
-skipitem(p_format, p_va)
-	char **p_format;
-	va_list *p_va;
+skipitem(char **p_format, va_list *p_va)
 {
 	char *format = *p_format;
 	char c = *format++;
@@ -1007,6 +1185,7 @@ skipitem(p_format, p_va)
 	switch (c) {
 	
 	case 'b': /* byte -- very short int */
+	case 'B': /* byte as bitfield */
 		{
 			(void) va_arg(*p_va, char *);
 			break;
@@ -1018,6 +1197,12 @@ skipitem(p_format, p_va)
 			break;
 		}
 	
+	case 'H': /* short int as bitfield */
+		{
+			(void) va_arg(*p_va, unsigned short *);
+			break;
+		}
+	
 	case 'i': /* int */
 		{
 			(void) va_arg(*p_va, int *);
@@ -1108,8 +1293,7 @@ skipitem(p_format, p_va)
 			}
 #endif
 			else if (*format == '&') {
-				typedef int (*converter)
-					Py_PROTO((PyObject *, void *));
+				typedef int (*converter)(PyObject *, void *);
 				(void) va_arg(*p_va, converter);
 				(void) va_arg(*p_va, void *);
 				format++;
