@@ -5,6 +5,7 @@ import sys
 import string
 import getopt
 import re
+import warnings
 
 import linecache
 from code import InteractiveInterpreter
@@ -15,7 +16,10 @@ import tkMessageBox
 from EditorWindow import EditorWindow, fixwordbreaks
 from FileList import FileList
 from ColorDelegator import ColorDelegator
+from UndoDelegator import UndoDelegator
 from OutputWindow import OutputWindow
+from IdleConf import idleconf
+import idlever
 
 # We need to patch linecache.checkcache, because we don't want it
 # to throw away our <pyshell#...> entries.
@@ -113,33 +117,87 @@ class ModifiedColorDelegator(ColorDelegator):
         ColorDelegator.recolorize_main(self)
 
     tagdefs = ColorDelegator.tagdefs.copy()
+    cconf = idleconf.getsection('Colors')
 
     tagdefs.update({
-        ##"stdin":   {"background": "yellow"},
-        "stdout":  {"foreground": "blue"},
-        "stderr":  {"foreground": "#007700"},
-        "console": {"foreground": "#770000"},
-        "ERROR":   {"background": "#FF7777"},
-        None:      {"foreground": "purple"}, # default
+        "stdin": cconf.getcolor("stdin"),
+        "stdout": cconf.getcolor("stdout"),
+        "stderr": cconf.getcolor("stderr"),
+        "console": cconf.getcolor("console"),
+        "ERROR": cconf.getcolor("ERROR"),
+        None: cconf.getcolor("normal"),
     })
 
 
+class ModifiedUndoDelegator(UndoDelegator):
+
+    # Forbid insert/delete before the I/O mark
+
+    def insert(self, index, chars, tags=None):
+        try:
+            if self.delegate.compare(index, "<", "iomark"):
+                self.delegate.bell()
+                return
+        except TclError:
+            pass
+        UndoDelegator.insert(self, index, chars, tags)
+
+    def delete(self, index1, index2=None):
+        try:
+            if self.delegate.compare(index1, "<", "iomark"):
+                self.delegate.bell()
+                return
+        except TclError:
+            pass
+        UndoDelegator.delete(self, index1, index2)
+
 class ModifiedInterpreter(InteractiveInterpreter):
 
     def __init__(self, tkconsole):
         self.tkconsole = tkconsole
-        InteractiveInterpreter.__init__(self)
+        locals = sys.modules['__main__'].__dict__
+        InteractiveInterpreter.__init__(self, locals=locals)
+        self.save_warnings_filters = None
 
     gid = 0
 
+    def execsource(self, source):
+        # Like runsource() but assumes complete exec source
+        filename = self.stuffsource(source)
+        self.execfile(filename, source)
+
+    def execfile(self, filename, source=None):
+        # Execute an existing file
+        if source is None:
+            source = open(filename, "r").read()
+        try:
+            code = compile(source, filename, "exec")
+        except (OverflowError, SyntaxError):
+            self.tkconsole.resetoutput()
+            InteractiveInterpreter.showsyntaxerror(self, filename)
+        else:
+            self.runcode(code)
+
     def runsource(self, source):
-        # Extend base class to stuff the source in the line cache
+        # Extend base class to stuff the source in the line cache first
+        filename = self.stuffsource(source)
+        self.more = 0
+        self.save_warnings_filters = warnings.filters[:]
+        warnings.filterwarnings(action="error", category=SyntaxWarning)
+        try:
+            return InteractiveInterpreter.runsource(self, source, filename)
+        finally:
+            if self.save_warnings_filters is not None:
+                warnings.filters[:] = self.save_warnings_filters
+                self.save_warnings_filters = None
+
+    def stuffsource(self, source):
+        # Stuff source in the filename cache
         filename = "<pyshell#%d>" % self.gid
         self.gid = self.gid + 1
         lines = string.split(source, "\n")
         linecache.cache[filename] = len(source)+1, 0, lines, filename
-        self.more = 0
-        return InteractiveInterpreter.runsource(self, source, filename)
+        return filename
 
     def showsyntaxerror(self, filename=None):
         # Extend base class to color the offending position
@@ -159,14 +217,14 @@ class ModifiedInterpreter(InteractiveInterpreter):
         text.tag_add("ERROR", pos)
         text.see(pos)
         char = text.get(pos)
-        if char in string.letters + string.digits + "_":
+        if char and char in string.letters + string.digits + "_":
             text.tag_add("ERROR", pos + " wordstart", pos)
         self.tkconsole.resetoutput()
         self.write("SyntaxError: %s\n" % str(msg))
 
     def unpackerror(self):
         type, value, tb = sys.exc_info()
-        ok = type == SyntaxError
+        ok = type is SyntaxError
         if ok:
             try:
                 msg, (dummy_filename, lineno, offset, line) = value
@@ -200,6 +258,9 @@ class ModifiedInterpreter(InteractiveInterpreter):
 
     def runcode(self, code):
         # Override base class method
+        if self.save_warnings_filters is not None:
+            warnings.filters[:] = self.save_warnings_filters
+            self.save_warnings_filters = None
         debugger = self.debugger
         try:
             self.tkconsole.beginexecuting()
@@ -234,15 +295,18 @@ class ModifiedInterpreter(InteractiveInterpreter):
 
 class PyShell(OutputWindow):
 
+    shell_title = "Python Shell"
+
     # Override classes
     ColorDelegator = ModifiedColorDelegator
+    UndoDelegator = ModifiedUndoDelegator
 
     # Override menu bar specs
     menu_specs = PyShellEditorWindow.menu_specs[:]
     menu_specs.insert(len(menu_specs)-2, ("debug", "_Debug"))
 
     # New classes
-    from History import History
+    from IdleHistory import History
 
     def __init__(self, flist=None):
         self.interp = ModifiedInterpreter(self)
@@ -258,7 +322,7 @@ class PyShell(OutputWindow):
         __builtin__.quit = __builtin__.exit = "To exit, type Ctrl-D."
 
         self.auto = self.extensions["AutoIndent"] # Required extension
-        self.auto.config(prefertabs=1)
+        self.auto.config(usetabs=1, indentwidth=8, context_use_ps1=1)
 
         text = self.text
         text.configure(wrap="char")
@@ -272,6 +336,9 @@ class PyShell(OutputWindow):
         text.bind("<<open-python-shell>>", self.flist.open_shell)
         text.bind("<<toggle-jit-stack-viewer>>", self.toggle_jit_stack_viewer)
 
+        self.save_stdout = sys.stdout
+        self.save_stderr = sys.stderr
+        self.save_stdin = sys.stdin
         sys.stdout = PseudoFile(self, "stdout")
         sys.stderr = PseudoFile(self, "stderr")
         sys.stdin = self
@@ -297,10 +364,11 @@ class PyShell(OutputWindow):
                 self.close_debugger()
             else:
                 self.open_debugger()
-    
+
     def set_debugger_indicator(self):
         db = self.interp.getdebugger()
         self.setvar("<<toggle-debugger>>", not not db)
+
     def toggle_jit_stack_viewer( self, event=None):
         pass # All we need is the variable
 
@@ -326,12 +394,13 @@ class PyShell(OutputWindow):
         # Helper for ModifiedInterpreter
         self.resetoutput()
         self.executing = 1
-        self._cancel_check = self.cancel_check
+        ##self._cancel_check = self.cancel_check
         ##sys.settrace(self._cancel_check)
 
     def endexecuting(self):
         # Helper for ModifiedInterpreter
-        sys.settrace(None)
+        ##sys.settrace(None)
+        ##self._cancel_check = None
         self.executing = 0
         self.canceled = 0
 
@@ -349,29 +418,37 @@ class PyShell(OutputWindow):
             if self.reading:
                 self.top.quit()
             return "cancel"
-        reply = PyShellEditorWindow.close(self)
-        if reply != "cancel":
-            self.flist.pyshell = None
-            # Restore std streams
-            sys.stdout = sys.__stdout__
-            sys.stderr = sys.__stderr__
-            sys.stdin = sys.__stdin__
-            # Break cycles
-            self.interp = None
-            self.console = None
-        return reply
+        return PyShellEditorWindow.close(self)
+
+    def _close(self):
+        self.close_debugger()
+        # Restore std streams
+        sys.stdout = self.save_stdout
+        sys.stderr = self.save_stderr
+        sys.stdin = self.save_stdin
+        # Break cycles
+        self.interp = None
+        self.console = None
+        self.auto = None
+        self.flist.pyshell = None
+        self.history = None
+        OutputWindow._close(self) # Really EditorWindow._close
 
     def ispythonsource(self, filename):
         # Override this so EditorWindow never removes the colorizer
         return 1
 
     def short_title(self):
-        return "Python Shell"
+        return self.shell_title
+
+    COPYRIGHT = \
+              'Type "copyright", "credits" or "license" for more information.'
 
     def begin(self):
         self.resetoutput()
-        self.write("Python %s on %s\n%s\n" %
-                   (sys.version, sys.platform, sys.copyright))
+        self.write("Python %s on %s\n%s\nIDLE %s -- press F1 for help\n" %
+                   (sys.version, sys.platform, self.COPYRIGHT,
+                    idlever.IDLE_VERSION))
         try:
             sys.ps1
         except AttributeError:
@@ -401,6 +478,9 @@ class PyShell(OutputWindow):
             return ""
         return line
 
+    def isatty(self):
+        return 1
+
     def cancel_callback(self, event):
         try:
             if self.text.compare("sel.first", "!=", "sel.last"):
@@ -486,6 +566,11 @@ class PyShell(OutputWindow):
             # No stdin mark -- just get the current line
             self.recall(self.text.get("insert linestart", "insert lineend"))
             return "break"
+        # If we're in the current input and there's only whitespace
+        # beyond the cursor, erase that whitespace first
+        s = self.text.get("insert", "end-1c")
+        if s and not string.strip(s):
+            self.text.delete("insert", "end-1c")
         # If we're in the current input before its last line,
         # insert a newline right at the insert point
         if self.text.compare("insert", "<", "end-1c linestart"):
@@ -596,38 +681,102 @@ class PseudoFile:
     def flush(self):
         pass
 
+    def isatty(self):
+        return 1
+
+
+usage_msg = """\
+usage: idle.py [-c command] [-d] [-e] [-s] [-t title] [arg] ...
+
+-c command  run this command
+-d          enable debugger
+-e          edit mode; arguments are files to be edited
+-s          run $IDLESTARTUP or $PYTHONSTARTUP before anything else
+-t title    set title of shell window
+
+When neither -c nor -e is used, and there are arguments, and the first
+argument is not '-', the first argument is run as a script.  Remaining
+arguments are arguments to the script or to the command run by -c.
+"""
 
 def main():
+    cmd = None
+    edit = 0
     debug = 0
+    startup = 0
+
     try:
-        opts, args = getopt.getopt(sys.argv[1:], "d")
+        opts, args = getopt.getopt(sys.argv[1:], "c:deist:")
     except getopt.error, msg:
         sys.stderr.write("Error: %s\n" % str(msg))
+        sys.stderr.write(usage_msg)
         sys.exit(2)
+
     for o, a in opts:
-        if o == "-d":
+        if o == '-c':
+            cmd = a
+        if o == '-d':
             debug = 1
+        if o == '-e':
+            edit = 1
+        if o == '-s':
+            startup = 1
+        if o == '-t':
+            PyShell.shell_title = a
+
+    for i in range(len(sys.path)):
+        sys.path[i] = os.path.abspath(sys.path[i])
+
+    pathx = []
+    if edit:
+        for filename in args:
+            pathx.append(os.path.dirname(filename))
+    elif args and args[0] != "-":
+        pathx.append(os.path.dirname(args[0]))
+    else:
+        pathx.append(os.curdir)
+    for dir in pathx:
+        dir = os.path.abspath(dir)
+        if not dir in sys.path:
+            sys.path.insert(0, dir)
+
     global flist, root
-    root = Tk()
+    root = Tk(className="Idle")
     fixwordbreaks(root)
     root.withdraw()
     flist = PyShellFileList(root)
-    if args:
-        for filename in sys.argv[1:]:
+
+    if edit:
+        for filename in args:
             flist.open(filename)
-            aPath = os.path.abspath(os.path.dirname(filename))
-            if not aPath in sys.path:
-                sys.path.insert(0, aPath)
     else:
-        aPath = os.getcwd()
-        if not aPath in sys.path:
-            sys.path.insert(0, aPath)
-    t = PyShell(flist)
-    flist.pyshell = t
-    t.begin()
+        if cmd:
+            sys.argv = ["-c"] + args
+        else:
+            sys.argv = args or [""]
+
+
+    shell = PyShell(flist)
+    interp = shell.interp
+    flist.pyshell = shell
+
+    if startup:
+        filename = os.environ.get("IDLESTARTUP") or \
+                   os.environ.get("PYTHONSTARTUP")
+        if filename and os.path.isfile(filename):
+            interp.execfile(filename)
+
     if debug:
-        t.open_debugger()
+        shell.open_debugger()
+    if cmd:
+        interp.execsource(cmd)
+    elif not edit and args and args[0] != "-":
+        interp.execfile(args[0])
+
+    shell.begin()
     root.mainloop()
+    root.destroy()
+
 
 if __name__ == "__main__":
     main()
