@@ -1,6 +1,10 @@
 /* Leave this blank line here -- autoheader needs it! */
 
 
+/* Define for AIX if your compiler is a genuine IBM xlC/xlC_r
+   and you want support for AIX C++ shared extension modules. */
+#undef AIX_GENUINE_CPLUSPLUS
+
 /* Define if your <unistd.h> contains bad prototypes for exec*()
    (as it does on SGI IRIX 4.x) */
 #undef BAD_EXEC_PROTOTYPES
@@ -18,11 +22,8 @@
 /* Define to `long' if <time.h> doesn't define.  */
 #undef clock_t
 
-/* Used for BeOS configuration */
-#undef DL_EXPORT_HEADER
-#ifdef DL_EXPORT_HEADER
-#include DL_EXPORT_HEADER
-#endif
+/* Defined on Solaris to see additional function prototypes. */
+#undef __EXTENSIONS__
 
 /* Define if getpgrp() must be called as getpgrp(0). */
 #undef GETPGRP_HAVE_ARG
@@ -34,8 +35,14 @@
 /* Define this if your time.h defines altzone */
 #undef HAVE_ALTZONE
 
+/* Defined when any dynamic module loading is enabled */
+#undef HAVE_DYNAMIC_LOADING
+
+/* Define this if you have flockfile(), getc_unlocked(), and funlockfile() */
+#undef HAVE_GETC_UNLOCKED
+
 /* Define this if you have gethostbyname() */
-#undef HAVE_GETHOSTBTNAME
+#undef HAVE_GETHOSTBYNAME
 
 /* Define this if you have some version of gethostbyname_r() */
 #undef HAVE_GETHOSTBYNAME_R
@@ -49,25 +56,57 @@
 /* Define this if you have the 6-arg version of gethostbyname_r() */
 #undef HAVE_GETHOSTBYNAME_R_6_ARG
 
+/* Defined to enable large file support when an off_t is bigger than a long
+   and long long is available and at least as big as an off_t. You may need
+   to add some flags for configuration and compilation to enable this mode.
+   E.g, for Solaris 2.7:
+   CFLAGS="-D_LARGEFILE_SOURCE -D_FILE_OFFSET_BITS=64" OPT="-O2 $CFLAGS" \
+	configure
+*/
+#undef HAVE_LARGEFILE_SUPPORT
+
 /* Define this if you have the type long long */
 #undef HAVE_LONG_LONG
 
-/* Define this if you have a K&R style C preprocessor */
-#undef HAVE_OLD_CPP
-
 /* Define if your compiler supports function prototypes */
 #undef HAVE_PROTOTYPES
 
+/* Define if you have GNU PTH threads */
+#undef HAVE_PTH
+
+/* Define if you have readline 4.2 */
+#undef HAVE_RL_COMPLETION_MATCHES
+
 /* Define if your compiler supports variable length function prototypes
    (e.g. void fprintf(FILE *, char *, ...);) *and* <stdarg.h> */
 #undef HAVE_STDARG_PROTOTYPES
 
+/* Define if you have termios available */
+#undef HAVE_TERMIOS_H
+
+/* Define this if you have the type uintptr_t */
+#undef HAVE_UINTPTR_T
+
+/* Define if you have a useable wchar_t type defined in wchar.h; useable
+   means wchar_t must be 16-bit unsigned type. (see
+   Include/unicodeobject.h). */
+#undef HAVE_USABLE_WCHAR_T
+
+/* Define if the compiler provides a wchar.h header file. */
+#undef HAVE_WCHAR_H
+
+/* Define if nice() returns success/failure instead of the new priority. */
+#undef HAVE_BROKEN_NICE
+
 /* Define if malloc(0) returns a NULL pointer */
 #undef MALLOC_ZERO_RETURNS_NULL
 
 /* Define if you have POSIX threads */
 #undef _POSIX_THREADS
 
+/* Define if you want to build an interpreter with many run-time checks  */
+#undef Py_DEBUG
+
 /* Define to force use of thread-safe errno, h_errno, and other functions */
 #undef _REENTRANT
 
@@ -77,9 +116,29 @@
 /* Define to empty if the keyword does not work.  */
 #undef signed
 
+/* Define if i>>j for signed int i does not extend the sign bit
+   when i < 0
+*/
+#undef SIGNED_RIGHT_SHIFT_ZERO_FILLS
+
+/* The number of bytes in an off_t. */
+#undef SIZEOF_OFF_T
+
+/* The number of bytes in a time_t. */
+#undef SIZEOF_TIME_T
+
+/* The number of bytes in a pthread_t. */
+#undef SIZEOF_PTHREAD_T
+
+/* Defined if PTHREAD_SCOPE_SYSTEM supported. */
+#undef PTHREAD_SYSTEM_SCHED_SUPPORTED
+
 /* sizeof(void *) */
 #undef SIZEOF_VOID_P
 
+/* Define to `int' if <sys/types.h> doesn't define.  */
+#undef socklen_t
+
 /* Define for SOLARIS 2.x */
 #undef SOLARIS
 
@@ -96,12 +155,12 @@
 /* Define if you want SIGFPE handled (see Include/pyfpe.h). */
 #undef WANT_SIGFPE_HANDLER
 
-/* Define if you want to use SGI (IRIX 4) dynamic linking.
-   This requires the "dl" library by Jack Jansen,
-   ftp://ftp.cwi.nl/pub/dynload/dl-1.6.tar.Z.
-   Don't bother on IRIX 5, it already has dynamic linking using SunOS
-   style shared libraries */ 
-#undef WITH_SGI_DL
+/* Define if you want wctype.h functions to be used instead of the
+   one supplied by Python itself. (see Include/unicodectype.h). */
+#undef WANT_WCTYPE_FUNCTIONS
+
+/* Define if you want to compile in cycle garbage collection */
+#undef WITH_CYCLE_GC
 
 /* Define if you want to emulate SGI (IRIX 4) dynamic linking.
    This is rumoured to work on VAX (Ultrix), Sun3 (SunOS 3.4),
@@ -119,24 +178,40 @@
    linker (rld). Dyld is necessary to support frameworks. */
 #undef WITH_DYLD
 
-/* Define if you want to compile in rudimentary thread support */
-#undef WITH_THREAD
+/* Define if you want to use BSD db. */
+#undef WITH_LIBDB
+
+/* Define if you want to use ndbm. */
+#undef WITH_LIBNDBM
+
+/* Define if you want to compile in Python-specific mallocs */
+#undef WITH_PYMALLOC
 
 /* Define if you want to produce an OpenStep/Rhapsody framework
    (shared library plus accessory files). */
 #undef WITH_NEXT_FRAMEWORK
 
-/* The number of bytes in an off_t. */
-#undef SIZEOF_OFF_T
+/* Define if you want to use SGI (IRIX 4) dynamic linking.
+   This requires the "dl" library by Jack Jansen,
+   ftp://ftp.cwi.nl/pub/dynload/dl-1.6.tar.Z.
+   Don't bother on IRIX 5, it already has dynamic linking using SunOS
+   style shared libraries */ 
+#undef WITH_SGI_DL
 
-/* Defined to enable large file support when an off_t is bigger than a long
-   and long long is available and at least as big as an off_t. You may need
-   to add some flags for configuration and compilation to enable this mode.
-   E.g, for Solaris 2.7:
-   CFLAGS="-D_LARGEFILE_SOURCE -D_FILE_OFFSET_BITS=64" OPT="-O2 $CFLAGS" \
-	configure
-*/
-#undef HAVE_LARGEFILE_SUPPORT
+/* Define if you want to compile in rudimentary thread support */
+#undef WITH_THREAD
 
 
 /* Leave that blank line there-- autoheader needs it! */
+
+@BOTTOM@
+
+#ifdef __CYGWIN__
+#ifdef USE_DL_IMPORT
+#define DL_IMPORT(RTYPE) __declspec(dllimport) RTYPE
+#define DL_EXPORT(RTYPE) __declspec(dllexport) RTYPE
+#else
+#define DL_IMPORT(RTYPE) __declspec(dllexport) RTYPE
+#define DL_EXPORT(RTYPE) __declspec(dllexport) RTYPE
+#endif
+#endif
